// UAdorners.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UADORNERS__
#define __UADORNERS__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __GEOMETRY__
#include <Geometry.h>
#endif

#ifndef __UDESIGNATOR__
#include <UDesignator.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif


//----------------------------------------------------------------------------------------
// Constants used to indicate adorner priorities
//----------------------------------------------------------------------------------------

const short kLowestAdornPriority = 127;
	// Low priority adorners are considered last

const short kLowAdornPriority = kLowestAdornPriority - 32;
	// Low priority adorners are considered last

const short kViewAdornPriority = 64;
	// View priority is special case

const short kHighAdornPriority = kViewAdornPriority - 32;
	// High priority adorners take precedence

const short kHighestAdornPriority = 0;
	// High priority adorners take precedence


//----------------------------------------------------------------------------------------
// Constants for indicating whether an adorner should be freed if its reference cound goes
// to 0.
//----------------------------------------------------------------------------------------

const Boolean kFreeOnDeletion = True;
	// free this adorner if fReferenceCount = 0

const Boolean kDontFreeOnDeletion = !kFreeOnDeletion;


//----------------------------------------------------------------------------------------
// Constant signatures for some default adorners.
//----------------------------------------------------------------------------------------

const IDType kDimAdorner		= 'dima';
const IDType kDrawAdorner		= 'draw';
const IDType kEraseAdorner		= 'eras';
const IDType kFrameAdorner		= 'fram';
const IDType kHiliteAdorner		= 'hili';
const IDType kLineBottomAdorner = 'lnbo';
const IDType kLineLeftAdorner	= 'lnlf';
const IDType kLineRightAdorner	= 'lnrt';
const IDType kLineTopAdorner	= 'lntp';
const IDType kOvalAdorner		= 'oval';
const IDType kPrintAdorner		= 'prnt';
const IDType kResizeIconAdorner = 'resz';
const IDType kRRectAdorner		= 'rrct';
const IDType kSelectionAdorner	= 'sele';


//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

typedef char AdornPriority;


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TList;
class TView;
class TAdornment;
class TAdorner;
class TAdornerList;


//----------------------------------------------------------------------------------------
// CAdornmentIterator
//----------------------------------------------------------------------------------------

class CAdornmentIterator : public CArrayIterator
{
public:
	CAdornmentIterator(TAdornment* theAdornment,
					   ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					   Boolean itsForward);

	CAdornmentIterator(TAdornment* theAdornment, Boolean itsForward);

	CAdornmentIterator(TAdornment* theAdornment);

	CAdornmentIterator(TAdornerList* theAdornerList);
	
	TAdorner* CurrentAdorner(void);
		// returns the current adorner
	
	TAdorner* FirstAdorner(void);
		// Reset the iteration and return the first adorner in the iteration

	TAdorner* NextAdorner(void);
		// advances the iteration and then returns the adorner
};


//----------------------------------------------------------------------------------------
// TAdorner: abstract superclass for adorners which are basically  rendering algorithms  
//----------------------------------------------------------------------------------------

class TAdorner : public TObject
{
public:
	IDType fID;									// The ID of this adorner, to distinguish
                                                // it from other instances.

	Boolean fFreeOnDeletion;					// Should the adorner be freed when
                                                // removed from an adornment


	//------------------------------------------------------------------------------------
	// constructor and destructor
	//----------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// sets fAdornPriority to kLowestAdornPriority

	virtual pascal void IAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes the adorner 

	virtual pascal void AssociateWithAdornment(TAdornment* itsAdornment,
											   Boolean associate);
		// Associate this adorner with an adornment


	//------------------------------------------------------------------------------------
	// rendering
	//------------------------------------------------------------------------------------

	virtual pascal void DrawContents(TView* itsView, const VRect& area);
		// Draws the adorner before the view's Draw. 
	
	virtual pascal void Draw(TView* itsView, const VRect& area);

	virtual pascal void DrawLine(VHSelect vhs, short lh, short lv, short lto);
		// Utility routine for adorners - draws a line. 

	virtual pascal void DoHighlightSelection(TView* itsView, 
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL);
		// Adorns the selection before the view's DoHighlightSelection. 

	virtual pascal void InvalidateAdorner(TView* itsView);
		// Invalidates the adorner in itsView. 

	virtual pascal void ViewChangedSize (TView* itsView,
											const VPoint& delta,
											Boolean	invalidate);
		// This method is called when the owning view changes size 
};


//----------------------------------------------------------------------------------------
// TDimAdorner
//----------------------------------------------------------------------------------------

class TDimAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		// Initializes this adorner with ID kDimAdorner and fFreeOnDeletion FALSE

	virtual pascal void IDimAdorner();
		// Initializes this adorner with ID kDimAdorner and fFreeOnDeletion FALSE

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// calls itsView.Dim
};


//----------------------------------------------------------------------------------------
// TDrawAdorner
//----------------------------------------------------------------------------------------

class TDrawAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		// Initializes this adorner with ID kDrawAdorner and fFreeOnDeletion FALSE

	virtual pascal void IDrawAdorner();
		// Initializes this adorner with ID kDrawAdorner and fFreeOnDeletion FALSE

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// calls itsView.Draw(area)
};


//----------------------------------------------------------------------------------------
// TEraseAdorner
//----------------------------------------------------------------------------------------

class TEraseAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		// Initializes this adorner with ID kEraseAdorner and fFreeOnDeletion FALSE

	virtual pascal void IEraseAdorner();
		// Initializes this adorner with ID kEraseAdorner and fFreeOnDeletion FALSE

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// calls itsView.Draw(area)
};


//----------------------------------------------------------------------------------------
// THiliteAdorner
//----------------------------------------------------------------------------------------

class THiliteAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		//  Initializes this adorner with ID kHiliteAdorner and fFreeOnDeletion FALSE  

	virtual pascal void IHiliteAdorner();
		//  Initializes this adorner with ID kHiliteAdorner and fFreeOnDeletion FALSE  

	virtual pascal void Draw(TView* itsView, const VRect& area);
		//  calls itsView.Hilite  
};


//----------------------------------------------------------------------------------------
// TPrintAdorner
//----------------------------------------------------------------------------------------

class TPrintAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		// Initializes this adorner with ID kPrintAdorner and fFreeOnDeletion FALSE

	virtual pascal void IPrintAdorner();
		// Initializes this adorner with ID kPrintAdorner and fFreeOnDeletion FALSE

	virtual pascal void DoHighlightSelection(TView* itsView, 
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL);
		// calls itsView.DoHighlightSelection(fromHL, toHL)
};


//----------------------------------------------------------------------------------------
// TResizeIconAdorner
//----------------------------------------------------------------------------------------

class TResizeIconAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		// Initializes this adorner with ID kResizeIconAdorner and fFreeOnDeletion FALSE

	virtual pascal void IResizeIconAdorner();
		// Initializes this adorner with ID kResizeIconAdorner and fFreeOnDeletion FALSE

	virtual pascal void DoHighlightSelection(TView* itsView, 
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL);
		// calls TWindow(itsView).DrawResizeIcon
};


//----------------------------------------------------------------------------------------
// TSelectionAdorner
//----------------------------------------------------------------------------------------

class TSelectionAdorner : public TAdorner
{
public:
	virtual pascal void Initialize();	// Override
		//  Initializes this adorner with ID kSelectionAdorner and fFreeOnDeletion FALSE  

	virtual pascal void ISelectionAdorner();
		//  Initializes this adorner with ID kSelectionAdorner and fFreeOnDeletion FALSE  

	virtual pascal void DoHighlightSelection(TView* itsView, 
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL);
		//  calls itsView.DoHighlightSelection(fromHL, toHL)  
};


//----------------------------------------------------------------------------------------
// TFrameAdorner
//----------------------------------------------------------------------------------------

class TFrameAdorner : public TAdorner
{
public:
	virtual pascal void IFrameAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// frames the extent of itsView 
};


//----------------------------------------------------------------------------------------
// TLineBottomAdorner
//----------------------------------------------------------------------------------------

class TLineBottomAdorner : public TAdorner
{
public:
	virtual pascal void ILineBottomAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// draws bottom line 
};


//----------------------------------------------------------------------------------------
// TLineLeftAdorner
//----------------------------------------------------------------------------------------

class TLineLeftAdorner : public TAdorner
{
public:
	virtual pascal void ILineLeftAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// draws left line 
};


//----------------------------------------------------------------------------------------
// TLineRightAdorner
//----------------------------------------------------------------------------------------

class TLineRightAdorner : public TAdorner
{
public:
	virtual pascal void ILineRightAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// draws right line 
};


//----------------------------------------------------------------------------------------
// TLineTopAdorner
//----------------------------------------------------------------------------------------

class TLineTopAdorner : public TAdorner
{
public:
	virtual pascal void ILineTopAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// draws top line 
};


//----------------------------------------------------------------------------------------
// TOvalAdorner
//----------------------------------------------------------------------------------------

class TOvalAdorner : public TAdorner
{
public:
	virtual pascal void IOvalAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// draws oval adorner 
};


//----------------------------------------------------------------------------------------
// TRRectAdorner
//----------------------------------------------------------------------------------------

class TRRectAdorner : public TAdorner
{
public:
	virtual pascal void IRRectAdorner(IDType itsID, Boolean freeOnDeletion);
		// initializes this adorner with itsView 

	virtual pascal void Draw(TView* itsView, const VRect& area);
		// adorns itsView with a round rectangle 
};


//----------------------------------------------------------------------------------------
// AdornerElement
//----------------------------------------------------------------------------------------

struct AdornerElement
{
	AdornPriority	priority;
	TAdorner* 		adorner;
	
	AdornerElement(AdornPriority itsPriority, TAdorner* itsAdorner);
		// Constructor
};

typedef AdornerElement* AdornerElementPtr;


//----------------------------------------------------------------------------------------
// AdornerElement inline function definitions.
//----------------------------------------------------------------------------------------

inline AdornerElement::AdornerElement(AdornPriority itsPriority,
									 TAdorner* 	itsAdorner)
{
	priority = itsPriority;
	adorner = itsAdorner;
}


//----------------------------------------------------------------------------------------
// TAdornerList: A list for keeping TAdorner objects ordered by priority 
//----------------------------------------------------------------------------------------

class TAdornerList : public TSortedDynamicArray
{
public:
	virtual pascal void IAdornerList();
		// Initialize the AdornerList procedurally. 

	virtual pascal short CompareElements(void* Element1, void* Element2);	// Override
		// Compares Two TAdorner objects based on their priority 

	virtual pascal void AddAdorner(TAdorner* itsAdorner, AdornPriority itsPriority);
		// Add the adorner to the list 
	
	virtual pascal TAdorner* AdornerAt(ArrayIndex i);
	
	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// TAdornment
//----------------------------------------------------------------------------------------

class TAdornment : public TAdorner
{
public:
	TView* 			fView;

	TAdornerList* fAdorners;					// Sorted array of TAdorner objects

	Point fPenSize;								// pen size for adorners to use

	Boolean fDrawShadow;						// Draw a shadow on the result of all the
                                                // adorners


	//------------------------------------------------------------------------------------
	// constructor and destructor
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// sets fAdorners to NIL 

	virtual pascal void IAdornment(IDType itsID, TView* itsView, Point itsPenSize);
		// initializes the adorner manager, allocate an adorner list 

	virtual pascal void Free();
		// calls FreeList on fAdorners 


	//------------------------------------------------------------------------------------
	// adorner mgmt 
	//------------------------------------------------------------------------------------

	virtual pascal void AddAdorner(TAdorner* anAdorner,
									AdornPriority itsPriority,
								   	Boolean invalidate);

	virtual pascal void DeleteAdorner(TAdorner* anAdorner, Boolean invalidate);
		// removes an adorner from the adornment's adorner list 

	virtual pascal void DeleteAdornerByID(IDType itsID, Boolean invalidate);
		// removes an adorner from the adornment's adorner list 

	virtual pascal TAdorner* AdornerAt(ArrayIndex i);
		// returns the i'th adorner in the list 

	virtual pascal TAdorner* AdornerWithID(IDType itsID);


	//------------------------------------------------------------------------------------
	// rendering 
	//------------------------------------------------------------------------------------

	virtual pascal void PreDraw(TView* itsView, const VRect& area);	

	virtual pascal void DrawContents(TView* itsView, const VRect& area);	// Override

	virtual pascal void Draw(TView* itsView, const VRect& area);	// Override
		// iterates over fAdorners asking each adorner to draw itself 

	virtual pascal void PostDraw(TView* itsView, const VRect& area);	

	virtual pascal void DoHighlightSelection(TView* itsView, 
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL);	// Override
		// iterates over fAdorners calling DoPreHighlightSelection on each adorner 

	virtual pascal void ViewChangedSize (TView* itsView,
											const VPoint& delta,
											Boolean	invalidate);
		// Overridden to iterate over all adorners calling their ViewChangedSize methods 
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern pascal void InitUAdorners();

extern pascal TAdorner* NewStdAdorner(IDType signature,
									  MAName className,
									  IDType itsID,
									  Boolean freeOnDeletion);


//----------------------------------------------------------------------------------------
// Global, preallocated, shared adorners:
//----------------------------------------------------------------------------------------

extern TDimAdorner* gDimAdorner;				// Calls the view's Dim method  

extern TDrawAdorner* gDrawAdorner;				// Calls the view's Draw method  

extern TEraseAdorner* gEraseAdorner;			// Erases the area passed in the DoDraw
                                                // method

extern THiliteAdorner* gHiliteAdorner;			// Calls the view's Hilite method  

extern TPrintAdorner* gPrintAdorner;			// Calls the view's DoDrawPrintFeedback
                                                // method

extern TResizeIconAdorner* gResizeIconAdorner;	// Calls the TWindow's DrawResizeIcon
                                                // method

extern TSelectionAdorner* gSelectionAdorner;	// Calls the view's DoHighlightSelection
                                                // method


//----------------------------------------------------------------------------------------
// Convenient RGB color settings 
//----------------------------------------------------------------------------------------

extern RGBColor gRGBVeryLtGray;					// Greyscale very light gray 

extern RGBColor gRGBLtGray;						// Greyscale light gray 

extern RGBColor gRGBGray;						// Greyscale regular gray 

extern RGBColor gRGBDkGray;						// Greyscale dark gray 

#endif


