// UAssociation.h
// Copyright  1988-1991 by Apple Computer Inc.All rights reserved.

#ifndef __UASSOCIATION__
#define __UASSOCIATION__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations.
//----------------------------------------------------------------------------------------

class TEntry;
class TEntriesList;


//----------------------------------------------------------------------------------------
// Typedefs for procedural parameters passed to methods in TAssociation.
//----------------------------------------------------------------------------------------

typedef pascal void(* DoToEntryType)(TEntry* theEntry, void* staticLink);
typedef pascal Boolean(* TestEntryType)(TEntry* theEntry, void* staticLink);


//----------------------------------------------------------------------------------------
// TEntry: This class associates a key string with a value string.
//----------------------------------------------------------------------------------------

class TEntry : public TObject
{
public:
	StringHandle fKey;							//  A handle to the 'keyStr'

	StringHandle fValue;						//  A handle to the 'valueStr'
	
	TEntry();
		// Default constructor for C++ users.

	TEntry(const Str255& itsKey, const Str255& itsValue);
	
	virtual pascal void Initialize();
		// Puts a newly created TEntry into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.
		
	virtual pascal void IEntry(const Str255& itsKey, const Str255& itsValue);
		//  Initialization routine

	virtual pascal void Free();
		//  Frees the fKey & fValue string handles then calls inherited free

	virtual pascal void SetValue(const Str255& value);
		//  Sets the fValue field to theValue

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};

inline TEntry::TEntry() { }

inline TEntry::TEntry(const Str255& itsKey, const Str255& itsValue)
{
	IEntry(itsKey, itsValue);
}


//----------------------------------------------------------------------------------------
// TEntriesList: Subclass of TSortedList that manages a sorted list of TEntry objects,
// used as an instance variable in TAssociation
//----------------------------------------------------------------------------------------

class TEntriesList : public TSortedList
{
public:
	TEntriesList();
		// Constructor for use from C++.
		
	virtual pascal void IEntriesList();
		// Initialize the list

	virtual pascal short Compare(TObject* item1, TObject* item2);
		// Compares 'item'1 with 'item2' returning an integer indicating the results of
        // the comparison

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};

inline TEntriesList::TEntriesList() { IEntriesList(); }


//----------------------------------------------------------------------------------------
// TAssociation: This class is used to create lists of strings that are accessed via their
// associated keys. The keys & strings are stored in TEntry objects which are in the
// TEntriesList instance variable
//----------------------------------------------------------------------------------------

class TAssociation : public TObject
{
public:
	TEntriesList* fEntries;						// The list of TEntries that stores the
                                                // associations
												

	virtual pascal void Initialize();
		// Puts a newly created TAssociation into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IAssociation();
		// Initialization routine

	virtual pascal void Free();
		//  Frees the instance variable fEntries

	virtual pascal Boolean ValueAt(const Str255& keyStr, Str255& valueStr);
		// Given 'keyStr' returns the associated string in 'valueStr', with the result
        // true if an associated string was found

	virtual pascal Boolean KeyAt(const Str255& valueStr, Str255& keyStr);
		// Given 'valueStr' returns the associated key in 'keyStr', with the result true
        // if an associated key was found

	virtual pascal void EachEntryDo(DoToEntryType DoToEntry, void* staticLink);
		//  Iterates over each entry passing it to "DoToEntry"

	virtual pascal TEntry* EntryWithKey(const Str255& keyStr);
		//  Returns the Entry containing 'keyStr'

	virtual pascal TEntry* EntryWithValue(const Str255& valueStr);
		//  Returns the entry containing 'valueStr'

	virtual pascal TEntry* FirstEntryThat(TestEntryType TestEntry, void* staticLink);
		// Iterates over each entry passing it to "TestEntry" until the result of the Test
        // is TRUE

	virtual pascal void InsertEntry(const Str255& keyStr, const Str255& valueStr);
		// Inserts an entry into thei association, if 'keyStr' already exists then the
        // value is merely replaced with 'valueStr'. Otherwise a new Entry is created with
        // the above 'keyStr' and 'valueStr'

	virtual pascal void RemoveValueAt(const Str255& keyStr);
		// Removes the value associated with 'keyStr'

	virtual pascal void RemoveKeyAt(const Str255& valueStr);
		// Removes the key associated with 'valueStr'

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


#endif


