////////////////////////////////////////////////////////////////////////////
//	UBehavior.h
//
// Contains:		Header file for Behavior abstract class
//
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.
//
//	Change History:
//
//			2/22/91	rtm	Initial creation of file
//			3/5/91	rtm	Commented in preparation for merging
//
////////////////////////////////////////////////////////////////////////////
/*

	BEHAVIOR OVERVIEW
	_______________________________________________________________________________________
	
	Behaviors provide a mechanism for modifying the behavior of any EvtHandler
	object.  This is accomplished by a simple filtering mechanism whereby
	the behavior gets to look at certain events as they occur.  The behavior
	can then choose to handle the event and return control to MacApp or
	prevent control from being passed on down the line.
	
	TBehavior is the abstract class from which standard behaviors will be
	derived.  It also serves as the superclass for TViewBehavior which is the
	abstract class from which behaviors that are attached to views are derived.
	View behaviors add additional event filtering that is specific to views and 
	should only be attached to views and their subclasses.  TViewBehavior is
	likely to be the most commonly subclassed behavior as behaviors provide the
	greatest flexibility for modifying the behavior of views.
	
	Multiple behaviors can be attached to objects.  A doubly linked chain of
	behaviors is created if there are more than one.
	
	BEHAVIOR USAGE
	_______________________________________________________________________________________
	
	To implement a behavior one would write the following code fragment :
	
			//  Create the behavior
			TMisBehavior *aMisBehavior = new TMisBehavior;
			aMisBehavior->IViewBehavior ( viewToAttachToo, NULL );

	then attach the behavior to the appropriate view:
	
			//  Attach behavior to view
			viewToAttachToo->SetBehavior ( aMisBehavior );
			
	thats it, now when this view gets any event that the behavior is 
	interested in it will be passed to the behavior first.  If a
	behavior handles an event it interrupts the chain for that event by NOT
	calling the inherited method.  Thus to pass the event along the chain one
	should always be sure to call INHERITED.
	
	The methods of TBehavior that will be the most commonly overridden in
	subclasses have been designed to facilitate this process.  For every
	type of event that EvtHandlers handle there are two methods that
	participate in the handling of the event. First there is the dispatching
	or navigational method which will always begin with the word 'Handle'.
	Then there are the corresponding 'Do' methods which are the action
	methods, in other words where the event is actually handled.  Behaviors 
	have 'Do' methods that are the same as the ones in EvtHandlers, these should
	be overridden to handle the events in behaviors.  Likewise to change the way a 
	methods is handled in an EvtHandler the 'Do' method should be overridden.  The
	'Handle' methods, of TEvtHandlers should only be overridden if the dispatching
	or navigational mechanism needs to be modified in some way.
	
	When an event handler is called to handle an event the 'Handle' method will
	be called.  This method will first call its attached Behaviors 'Do' method
	to handle the event.  The behavior will inturn hand it off to the next
	behavior if there is one, otherwise, the behavior will call the EvtHandler's
	'Do' method.  TEvtHandler's 'Do' methods simply call the next handlers
	'Handle' method.
	
	EXAMPLE
	_______________________________________________________________________________________
	
	As an example of where behaviors could be used with MacApp to great
	advantage lets look at how we could use behaviors to implement a
	view manipulation tool.
	
	What we would like to be able to do is allow the user to create views
	in our tool and then allow them to be re-located or resized.  Typically
	to achieve this we would have to write some "wrapping" mechanism around 
	the standard MacApp view objects.  The purpose of this mechanism would
	be to filter events that are passed to the view in case we need to 
	intercept them so we can implement the editing features.  This means a 
	lot of wasted effort writing these "wrappers".
	
	Enter a behavior, by capturing the view manipulation capabilities ( or
	certainly the points from which manipulation commands can be dispatched )
	in a behavior.  We can then attach that behavior to any view that we would
	like to be able to manipulate.  Now because there is a behavior attached 
	to the view it will get a look at certain events allowing it to modify the
	views behavior when appropriate.  Because the field that behaviors are
	attached to has been added to TEvtHandler all existing MacApp classes that
	are descendants of TEvtHandler will be able to take advantage of this 
	feature.
	
	Eventually MacApp will provide some behaviors that people can use, in particular
	one that can be used to perform the operations associated with manipulating
	views.
	
	Now you too can construct your own View editing tool using this built in MacApp
	mechanism!!!!!!
	
	SUMMARY
	_______________________________________________________________________________________

	In summary, behaviors provide a mechanism for capturing common behavior in an
	object that can then be attached to other objects to extend their behavior without
	requiring that they be sub-classed.  This allows existing objects to stay lean
	and mean but be extended when required.  In many cases one can avoid having to 
	create a sub-class of an object by instead using a behavior to provide the required
	extensions.  This is especially useful in the case where this behavior is used
	in a number of different objects.
	
	One very important point that has not been mentioned is the fact that one could 
	very easily support the addition of behavior to objects - dynamically - within
	your application.
	
	Eventually a mechanism will also be provided for creating behaviors from resources
	and for adding them to views from resources.
	
	We feel that people will find many uses for behaviors and we intend to use them
	ourselves for future MacApp tools, as always we are interested in getting your
	feedback on how you have used behaviors and what can be done to make them even
	more powerful!!!!
	
	Go forth and multiply!!!!!
	
	Look at print handlers in MacApp which have been implemented as behaviors.
			
*/

#ifndef __UBEHAVIOR__
#define __UBEHAVIOR__

#ifndef __UOBJECT__
#include <UObject.h>
#endif
#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif
#ifndef __UVIEW__
#include <UView.h>
#endif


//----------------------------------------------------------------------------------------
// TBehavior
//----------------------------------------------------------------------------------------

class	TBehavior:	public TObject 
/*

	 This is the abstract class for the general Behaviors that can be attached to any
	 TEvtHandler.  This must be subclassed to do anything useful.  It will also be 
	 subclassed to create a specific abstract class from which view behaviors will 
	 be sub-classed.
	 
*/
{
	public:
	
		TEvtHandler*	fOwner;			//	The event handler that owns this behavior	
		TBehavior*		fPreviousBehavior;	//	The previous behavior in the chain
		TBehavior*		fNextBehavior;	//	The next behavior in the chain

		
		virtual pascal void Initialize ( void );	//	OVERRIDE
		/* Overridden so that we can initialize our fields	*/
		
		virtual pascal void IBehavior ( void );
		/* Initialization method	*/
		
		virtual pascal TObject* Clone ( void );	// OVERRIDE
		/* We have overridden this to do a deep clone of the behavior */
		
		virtual pascal void Free ( void );
		/* Pass the Free message on to all behaviors in the chain
		 before calling inherited Free	*/
		
		//  Chain Management Methods
		virtual pascal void SetNextBehavior ( TBehavior* nextBehavior );
		/* Set the fNextBehavior field if it is NULL, otherwise calls fNextBehavior->SetNextBehavior	*/

		virtual pascal void SetPreviousBehavior ( TBehavior* previousBehavior );
		/* Set the fPreviousBehavior field	*/
		
		virtual pascal TBehavior* GetNextBehavior ( void );
		/* Return the next behavior	*/

		virtual pascal void SetOwner ( TEvtHandler* itsOwner );
		/* Set the fOwner field	*/
		
		virtual pascal void InsertBefore (  TBehavior* 	insertMe,
														TBehavior*	before );
		/* Insert behavior 'insertMe' ahead of behavior 'before' in the chain	*/
		
		virtual pascal void InsertAfter (  	TBehavior* 	insertMe,
														TBehavior*	after );
		/* Insert behavior 'insertMe' after of behavior 'after' in the chain	*/
											
		virtual pascal void Remove ( TBehavior*	removeMe );
		/* Remove behavior 'removeMe' from the chain	*/
		
		//  Event Filtering Methods
		virtual pascal void DoAppleCommand ( 	CmdNumber aCmdNumber,
																	const AppleEvent& message,
																	const AppleEvent& reply );
		/* Allows behaviors to respond to AppleEvents, if there is a next
		behavior it passes the message on down the chain otherwise it calls
		the corresponding method of its Owner	*/
		
		virtual pascal void DoCommandKey ( TToolboxEvent* event );
		/* Allows behaviors to respond to CommandKey events, if there is a next
		behavior it passes the message on down the chain otherwise it calls
		the corresponding method of its Owner	*/
		
		virtual pascal void DoEvent (	EvtNumber eventNumber,
												TEvtHandler* source,
												TEvent* event );
		/* Allows behaviors to handle events, if there is a next
		behavior it passes the message on down the chain otherwise it calls
		the corresponding method of its Owner	*/
		
		
		virtual pascal void DoKeyCommand ( TToolboxEvent* event );
		/* Allows behaviors to respond to KeyCommands, if there is a next
		behavior it passes the message on down the chain otherwise it calls
		the corresponding method of its Owner	*/
		
		virtual pascal void DoMenuCommand ( CmdNumber aCmdNumber );
		/* Allows behaviors to handle menu events, if there is a next 
		behavior passes the message on down the chain otherwise it calls
		the corresponding method of its Owner	*/
		
		virtual pascal void DoSetupMenus ( void );
		/* Allows behaviors to handle menu setup if there is a next 
		behavior passes the message on down the chain otherwise it calls
		the corresponding method of its Owner	*/
	
};	//	<-- TBehavior


//----------------------------------------------------------------------------------------
// CBehaviorIterator: A simple iterator for the behavior chain.
//----------------------------------------------------------------------------------------

class CBehaviorIterator : public CIterator
{
private:
	TBehavior*	fFirstBehavior;

	TBehavior*	fCurrentBehavior;

	TBehavior*	fNextBehavior;
	
public:
	inline CBehaviorIterator(TEvtHandler* itsEvtHandler);
	
	virtual Boolean More(void);					// override
		// Returns TRUE if there are more elements to iterate over

	virtual void Reset(void);					// override
		// Resets the iterator to begin again
	
	inline TBehavior* CurrentBehavior(void);
		// returns the current Behavior in the Behavior chain

	inline TBehavior* FirstBehavior(void);
		// Resets the iterator to begin again and returns the first Behavior in the Behavior chain

	inline TBehavior* NextBehavior(void);
		// returns the next Behavior in the Behavior chain
	
protected:
	virtual void Advance(void);					// override
		// Advances the iteration
};

//----------------------------------------------------------------------------------------
// CBehaviorIterator: Inline method definitions.
//----------------------------------------------------------------------------------------

inline CBehaviorIterator::CBehaviorIterator(TEvtHandler* itsEvtHandler)
{
	fFirstBehavior = NULL;
	fCurrentBehavior = NULL;
	fNextBehavior = NULL;

	if (itsEvtHandler)
	{
		fFirstBehavior = itsEvtHandler->fBehavior;
		fCurrentBehavior = fFirstBehavior;
		if (fFirstBehavior)
			fNextBehavior = fFirstBehavior->GetNextBehavior();
	}
}

inline TBehavior* CBehaviorIterator::CurrentBehavior()
{
	return fCurrentBehavior;
}

inline TBehavior* CBehaviorIterator::FirstBehavior()
{
	this->Reset();
	return fFirstBehavior;
}

inline TBehavior* CBehaviorIterator::NextBehavior()
{
	this->Advance();
	return fCurrentBehavior;
}


#endif __UBEHAVIOR__