// UCommand.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UCOMMAND__
#define __UCOMMAND__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __APPLEEVENTS__
#include "AppleEvents.h"
#endif

#ifndef __UEVENT__
#include "UEvent.h"
#endif

class TDocument;

class TView;

class TScroller;

class TAppleEvent;


typedef EventPriority CommandPriority;
/*  Command priorities  */

class TCommand : public TEvent
{
public:
	/*  Handles a user command, including undoing
	  and mouse-tracking if required. Usually
	  created by the part of the Application
	  with the greatest specific knowledge about
	  the action to take in response to events
	  and passed back to be performed at several
	  well defined places. */
	TView* fView;								/*  The view affected by this command, and in
												  which tracking takes place, if any.  */
	TObject* fChangedObject;					/*  object to notify of changes  */
	Boolean fCmdDone;							/*  TRUE if the last "execution" message sent
												  to the command was DoIt or RedoIt  */
	Boolean fCanUndo;							/*  Defaults to TRUE  */
	Boolean fCausesChange;						/*  Defaults to TRUE; Marks document changed
												  when command is done  */
	Boolean fChangesClipboard;					/*  Defaults to FALSE. Set to true for command
												  subclasses representing CUT and/ or COPY
												  commands which change the Clipboard.  */
	Boolean fFreeOnCompletion;					/*  TRUE to cause the command to be freed on
												  completion (the default). Completion is
												  after DoIt for non-undoable commands and
												  Commit for undoable commands.  */
	Boolean fReadyToExecute;					/*  TRUE (the default) will enable this
												  command to be returned from any queues
												  that may be holding it. Most programs
												  won't have to mess with this.  */
	Boolean fRecurring;							/*  FALSE (the default) will enable this
												  command to be removed from any command
												  queue that holds it because it is only
												  executed once.  */

	/*  Init & Free  */
	virtual pascal void Initialize();
	/*  Puts a newly created command into a known state. Subclasses should
	  override it and call INHERITED. Typically used to put the object into a state where
	  it can be safely FREED.  */

	virtual pascal void ICommand(CmdNumber itsCmdNumber,
								 TObject* notify,
								 TView* itsView);


	/*  Initialize a command procedurally.  */

	/*  Inspecting  */
	virtual pascal void Fields(TObject* obj);
	/*  Used by the Inspector and the Debugger to display the contents of this class's
	  fields.  */

	/*  command processing  */

	virtual pascal void Process(); // Override

	/*  command processing - usually overridden  */

	virtual pascal void Commit();
	/*  Stub  */

	virtual pascal void DoIt();
	/*  Stub  */

	virtual pascal void RedoIt();
	/*  Stub  */

	virtual pascal void UndoIt();
	/*  Stub  */

	/*  Scheduling  */
	virtual pascal Boolean IsReadyToExecute();
	/*  Returns TRUE (the Default) when the command is ready to execute.	Override to base
	  the execution of the command on more sophisticated criteria.
	  Most programs won't have to mess with this..  */

	virtual pascal Boolean ShouldFreeOnCompletion();
	/* Overridden to return the value stored in fFreeOnCompletion  */

	virtual pascal Boolean IsRecurring();
	/* Overridden to return the value stored in fRecurring  */

	/*  Change notification  */
	virtual pascal ChangeID GetChangeID();
	/*  Returns a ChangeID for TApplication.PerformCommand to pass to Changed.
	  Returns fID as a default.  */
};

/* -------------------------------------------------------------------------------------------------- */
class TNoChangesCommand : public TCommand
{
public:
	/*  Doesn't "change" the document and is not
	  itself undoable. Thus it doesn't flush the
	  last command that did. Useful for
	  deferring actions out to the
	  PerformCommand bottleneck where the stack
	  is unwound and the heap is unloaded for
	  maximum elbow room.  */
	virtual pascal void INoChangesCommand(CmdNumber itsCmdNumber,
										  TObject* notify,
										  TView* itsView);



	virtual pascal void Fields(TObject* obj);
	/*  Used by the Inspector and the Debugger to display the contents of this class's
	  fields.  */

};


/* -------------------------------------------------------------------------------------------------- */class TTracker : public TCommand
{
public:
	/*  Handles a user command, including undoing
	  and mouse-tracking if required. Usually
	  created by the part of the Application
	  with the greatest specific knowledge about
	  the action to take in response to events
	  and passed back to be performed at several
	  well defined places. */
	Boolean fTracksMouse;						/*  TRUE if the command should be tracked.  */
	VPoint fInitialPt;							/*  Where to track from (usually where the
												  mouse went down).In coordinates of the
												  fView (that's being tracked) else if the
												  fView is nil in global coordinates.  */

	Boolean fConstrainsMouse;					/*  Defaults to FALSE; if you set to TRUE then
												  TrackConstrain will be called to constrain
												  the mouse.  */
	Boolean fViewConstrain;						/*  Defaults to TRUE, which means constrain
												  mouse to view limits  */
	Boolean fTrackNonMovement;					/*  Defaults to FALSE, which means don't track
												  mouse unless it moves.  */
	TScroller* fScroller;						/*  The scroller that handles auto-scrolling  */

	/*  Init & Free  */
	virtual pascal void Initialize();
	/*  Puts a newly created TTracker into a known state. Subclasses should
	  override it and call INHERITED. Typically used to put the object into a state where
	  it can be safely FREED.  */

	virtual pascal void ITracker(CmdNumber itsCmdNumber,
								 TObject* notify,
								 TView* itsView,
								 TScroller* itsScroller,
								 const VPoint& itsMouse);




	/*  Initialize a command procedurally.  */

	/*  Inspecting  */
	virtual pascal void Fields(TObject* obj);
	/*  Used by the Inspector and the Debugger to display the contents of this class's
	  fields.  */

	/*	Command Processing */
	
	virtual pascal void Process(); // Override
	/* Overridden to call TApplication::TrackMouse. */

	/*  Mouse Tracking  */

	virtual pascal Boolean IsDoneTracking();
	/*  Indicates whether the Command is through tracking.(the command is also through
	  tracking if you return NIL from TrackMouse).Defaults to fView.IsDoneTracking.
	  Most programs won't have to mess with this.
	  NOTE you still have to deal with queued events if you change the criteria for TRUE.  */

	virtual pascal void TrackConstrain(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
									  	const VPoint& previousPoint,
									    VPoint& nextPoint,
										Boolean mouseDidMove);

	/*  Override this if you want to constrain the mouse point to a grid, force drawing a
	  square, etc. This is called only if fConstrainsMouse is TRUE.  */

	virtual pascal void TrackFeedback(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
										const VPoint& previousPoint,
										const VPoint& nextPoint,
										Boolean mouseDidMove,
										Boolean turnItOn);

	/*  Default is: IF NOT mouseDidMove THEN FrameRect(<rect formed by anchorPoint and
	  nextPoint>). Before this is called the pen is set to PenNormal and the PenMode to
	  PatXOR.  */

	virtual pascal TTracker* TrackMouse(TrackPhase aTrackPhase,
										VPoint& anchorPoint,
										VPoint& previousPoint,
										VPoint& nextPoint,
										Boolean mouseDidMove);


	/*  This does the mouse tracking. The default returns SELF. Override it to force
	  gridding, to return a different type of command, or to perform the command on mouse
	  release instead of using the usual DoIt approach. All points are in local
	  coordinates. If aTrackPhase = trackPress then all 3 points will be the same. If
	  aTrackPhase = trackRelease then nextPoint will be the coordinate in the mouseUp
	  event (if a mouseUp event is found), otherwise the same point as previousPoint.
	  Generally, you will ignore nextPoint and just look at previousPoint. mouseDidMove
	  will be TRUE if aTrackPhase = trackPress or trackRelease; otherwise, it will
	  indicate if nextPoint = previousPoint. If you change anchorPoint, the new value
	  will be passed to you next time. The value of nextPoint at the time the routine
	  exits will be passed to you as previousPoint the next time the routine is called.
	  You can change nextPoint if you wish. Usually, however, you will do gridding in the
	  TrackConstrain method.  */

	virtual pascal void AutoScroll(const VPoint& delta);
	/*  Implements autoscrolling by calling fScroller.ScrollBy.  */

};

/* -------------------------------------------------------------------------------------------------- */
class TNoChangesTracker : public TTracker
{
public:
	/*  Doesn't "change" the document and is not
	  itself undoable. Thus it doesn't flush the
	  last command that did. Useful for
	  deferring actions out to the
	  PerformCommand bottleneck where the stack
	  is unwound and the heap is unloaded for
	  maximum elbow room.  */
	virtual pascal void INoChangesTracker(CmdNumber itsCmdNumber,
										  TObject* notify,
										  TView* itsView,
										  TScroller* itsScroller,
										  const VPoint& itsMouse);





	virtual pascal void Fields(TObject* obj);
	/*  Used by the Inspector and the Debugger to display the contents of this class's
	  fields.  */

};


/* -------------------------------------------------------------------------------------------------- */
/*  TAppleCommand can be used to both service an AppleEvent or send an AppleEvent.  */
/*  It should also be used for all commands that can be invoked by an AppleEvent  */
/*  _OR_ some user event (DoMenuCommand).A good example is the 'odoc' command.  */
/*  The 'odoc' command's DoIt method simply opens a list of files.TAppleCommand  */
/*  lets that list come from an AppleEvent or from some user action.  */

class TAppleCommand : public TCommand
{
public:


	TAppleEvent* fMessage;
	TAppleEvent* fReply;

	virtual pascal void Initialize();

	virtual pascal void IAppleCommand(CmdNumber itsCmdNumber,
									  TDocument* itsDocument,
									  TView* itsView);

	virtual pascal void InitFromAppleEvent(CmdNumber itsCmdNumber,
										   TDocument* itsDocument,
										   TView* itsView,
										   const AppleEvent& itsMessage,
										   const AppleEvent& itsReply);

	virtual pascal void Free();
	
	virtual pascal void ReportError(OSErr error, long message);
		// Report any errors in the default reply.

	virtual pascal void SendIt();

	virtual pascal void Fields(TObject* obj);
	/*  Used by the Inspector and the Debugger to display the contents of this class's
	  fields */

};


#endif


