// UControl.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UCONTROL__
#define __UCONTROL__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __CONTROLS__
#include <Controls.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

//----------------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------------

class TScroller;
class TDocument;
class TToolboxEvent;
class TAdornment;
class TList;


//----------------------------------------------------------------------------------------
// ControlTemplate: Must correspond to a PACKED Pascal record. Thus the need for bit
// fields.
//----------------------------------------------------------------------------------------

struct ControlTemplate
{
	CntlAdornment itsAdornment;
	Byte filler1:8;
	Point itsPenSize;
	Boolean isSizable:1;
	Boolean isDimmed:1;
	Boolean isHilited:1;
	Boolean canDismiss:1;
	short filler2:12;
	Rect itsInset;
	Style itsTextFace;
	short itsTextSize;
	RGBColor itsTextColor;
	Str255 itsFontName;							// Actually a variable length P-String
};


typedef ControlTemplate* ControlTemplatePtr;

typedef pascal void(* DoWhileFocusedType)(void* staticLink);


//----------------------------------------------------------------------------------------
// TControl: An abstract class that represents a control.
//----------------------------------------------------------------------------------------

class TControl : public TView
{
public:
	short fDefChoice;							// the choice to automatically pass up the
                                                // line via DoChoice

	Boolean fHilite;							// is this view hilited

	Boolean fDimmed;							// is this view dimmed

	Boolean fSizeable;							// is the control portion of this view
                                                // sizeable

	Boolean fDismissesDialog;					// can this view dismiss it's dialogview

	CntlAdornment fAdornment;					// the adornment to be drawn in addition
                                                // to the view

	Point fPenSize;								// pen size to be used in adorning

	VRect fInset;								// amount to inset the control portion of
                                                // the view

	TextStyle fTextStyle;						// the TextStyle to use when rendering
                                                // this view


	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TControl into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IControl(TView* itsSuperView,
								 const VPoint& itsLocation,
								 const VPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet);
		//  Initialize a control procedurally.

	virtual pascal void IRes(TDocument* itsDocument, TView* itsSuperView, Ptr& itsParams);
		// Initialize a control from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a control view out to a resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a control view to a resource, works in conjunction with tbe
        // previous method.

	virtual pascal void ComputeSize(VPoint& newSize);
		// Called to compute the control's size.

	virtual pascal Boolean ContainsMouse(const VPoint& theMouse);
		// Returns TRUE if 'theMouse is currently in the controls area.

	virtual pascal void ControlArea(VRect& theArea);
		// Returns in 'theArea' the controls active area.

	virtual pascal void Dim();
		// dims the current image in the TControl

	virtual pascal void DimState(Boolean state, Boolean redraw);
		// If we are not currently dimmed then set fDimmed to 'state.and if 'redraw' is
        // TRUE then call DrawContents.

	virtual pascal void DoMouseCommand(VPoint& theMouse, TToolboxEvent* event,
									   Point hysteresis);
		// Creates a control tracker command, which is posted.

	virtual pascal Boolean Focus();
		// Calls inherited Focus and if its TRUE then sets the port's text style and
        // returns TRUE, if it can't be focused returns FALSE.

	virtual pascal void Flash();
		// Causes a control to flash with an 8 tick delay.

	virtual pascal void Hilite();
		// hilites the current image in the TControl

	virtual pascal void HiliteState(Boolean state, Boolean redraw);
		// If our fHilite is not 'state' the set it to 'state' before calling Hilite.
        // Note: hilite is only called if 'redraw is TRUE and we can Focus.

	virtual pascal void Inset(short dh, short dv, Boolean redraw);
		// If we are sizable then offset fInset to 'dh' & 'dv' redrawing if 'redraw' is
        // TRUE.

	virtual pascal Boolean IsDimmed();
		// Returns TRUE if the control is currently dimmed.

	virtual pascal void GetAdornExtent(VRect& itsAdornExtent);
		// returns the adorn extent for adorning this view

	virtual pascal TAdornment* MakeAdornment();
		// creates and returns a TAdornment

	virtual pascal void SetAdornment(/* CntlAdornment */ short newAdornment,
									 Boolean redraw);
		// Set fAdornment to 'newAdornment' redrawing if 'redraw' is TRUE.

	virtual pascal void SetInset(const VRect& newInset, Boolean redraw);
		// Set fInset to 'newInset' redrawing if 'redraw' is TRUE.

	virtual pascal void InstallColor(const RGBColor& theColor, Boolean redraw);
		// Sets the color field of our text style record to 'theColor', if 'redraw' then
        // call DrawContents.

	virtual pascal void InstallTextStyle(const TextStyle& theTextStyle, Boolean redraw);
		// Sets fTextStyle to 'theTextStyle', if 'redraw' then call DrawContents.

	virtual pascal void Resize(const VPoint& newSize, Boolean invalidate);
		// Calls inherited Resize before calling ForceRedraw.

	virtual pascal void TrackFeedback(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
										const VPoint& previousPoint,
										const VPoint& nextPoint,
										Boolean mouseDidMove,
										Boolean turnItOn);
		// Doesn't use the default feedback

	virtual pascal void TrackMouse(TrackPhase aTrackPhase,
								   VPoint& anchorPoint,
								   VPoint& previousPoint,
								   VPoint& nextPoint,
								   Boolean mouseDidMove);
		// Called back from tracking command. Handles tracking of the mouse and then calls
        // DoChoice on TrackRelease.

	virtual pascal long Validate();
		// By default returns 'noErr'.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TCtlMgr: An abstract class that represents a toolbox Control Manager control.
//----------------------------------------------------------------------------------------

class TCtlMgr : public TControl
{
public:
	ControlHandle fCMgrControl;					// the Control Manager Control that is
                                                // being managed

	short fBitsToShift;							// # bits shifted to convert between short
                                                // and long value.

	VCoordinate fLongVal;						// the current value

	VCoordinate fLongMin;						// the minimum setting

	VCoordinate fLongMax;						// the maximum setting


	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TCtlMgr into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void ICtlMgr(TView* itsSuperView,
								const VPoint& itsLocation,
								const VPoint& itsSize,
								SizeDeterminer itsHSizeDet,
								SizeDeterminer itsVSizeDet,
								const Str255& itsTitle,
								long itsVal,
								long itsMin,
								long itsMax,
								short itsProcID);
		// Initialize procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize from a resource template.

	virtual pascal void Free();
		// Set the size of control to zero call DisposeControl, then call inherited Free.


	//------------------------------------------------------------------------------------
	// Rest of methods
	//------------------------------------------------------------------------------------

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a CtlMgr view to a resource.

	virtual pascal void BeInPort(GrafPtr itsPort);
	// Sets the control to be in 'itsPort', by setting the contrlOwner field of the
    // ControlHandle.

	virtual pascal void CreateCMgrControl(const VRect& itsBounds,
										  const Str255& itsTitle,
										  long itsVal,
										  long itsMin,
										  long itsMax,
										  short itsProcID);
		// Used to create a new control, calls NewControl to create the control.

	virtual pascal void DimState(Boolean state, Boolean redraw);
		// Set the dim state to 'state' and redrawing it if 'redraw'.

	virtual pascal void DoMouseCommand(VPoint& theMouse,
									   TToolboxEvent* event,
									   Point hysteresis);
		// Handles mouse downs in the control by calling TestControl and DoChoice

	virtual pascal void Draw(const VRect& area);
		// Draws the control by calling the toolbox routine ShowControl.

	virtual pascal VCoordinate GetLongMax();
		// Returns the current fLongMax value.

	virtual pascal VCoordinate GetLongMin();
		// Returns the current fLongMin value.

	virtual pascal VCoordinate GetLongVal();
		// Returns the current fLongVal value.

	virtual pascal void SetLongMax(VCoordinate itsMax,
								   Boolean redraw);
		// Sets fLongMax to 'itsMax', redrawing if required.

	virtual pascal void SetLongMin(VCoordinate itsMin, Boolean redraw);
		// Sets flongMin to 'itsMin', redrawing if required.

	virtual pascal void SetLongVal(VCoordinate itsVal, Boolean redraw);
		// Sets fLongVal to 'itsVal' redrawing if required.

	virtual pascal void SetLongValues(VCoordinate itsVal,
									  VCoordinate itsMin,
									  VCoordinate itsMax,
									  Boolean redraw);
		// Sets all of the values at once, redrawing if required.


	//------------------------------------------------------------------------------------
	// Methods to manipulate the Control Manager control values
	//------------------------------------------------------------------------------------
	
	virtual pascal short GetMax();
		// Returns the maximum value of the Control Manager control.

	virtual pascal short GetMin();
		// Returns the minimum value of the Control Manager control.

	virtual pascal void GetText(Str255& theText);
		// Returns the controls title, if any.

	virtual pascal short GetVal();
		// Returns the current value of the Control Manager control.

	virtual pascal short GetVariant();
		// Returns the variant control value for the Control Manager control.

	virtual pascal void SetMax(short itsMax, Boolean redraw);
		// While focused sets the maximum value of the Control Manager control to 'itsMax'.

	virtual pascal void SetMin(short itsMin, Boolean redraw);
		// While focused sets the minimum value of the Control Manager control to 'itsMin'.

	virtual pascal void SetText(const Str255& itsText, Boolean redraw);
		// Sets the controls title to 'itsText', redrawing if required.

	virtual pascal void SetVal(short newVal, Boolean redraw);
		// While focused sets the Control Manager controls value to 'newVal', redrawing if
        // required.

	virtual pascal void SetValues(short itsVal,
								  short itsMin,
								  short itsMax,
								  Boolean redraw);
		// While focused sets all the values of the Control Manager control, at once,
        // redrawing if required.

	virtual pascal void HiliteState(Boolean state, Boolean redraw);
		// Set the hilite state to 'state', redrawing if required.If hilite state is
        // currently 'state' then does nothing.

	virtual pascal Boolean IsCMgrVisible();
		// Returns TRUE if the control is currently visible, FALSE if not.

	virtual pascal void Resize(const VPoint& newSize, Boolean invalidate);
		// If the control is resizable calls the toolbox routines MoveControl &
        // SizeControl before calling inherited Resize.

	virtual pascal void SetCMgrVisibility(Boolean beVisible);
		// Sets the visibility of the control to 'isVisible'.

	virtual pascal void WhileFocused(DoWhileFocusedType DoWhileFocused,
									 void* staticLink,
									 Boolean redraw);
		// While focused call "DoWhileFocused", redrawing if required.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// ScrollBarTemplate
//----------------------------------------------------------------------------------------

struct ScrollBarTemplate
{
	long value;
	long minimum;
	long maximum;
};

typedef ScrollBarTemplate* ScrollBarTemplatePtr;


//----------------------------------------------------------------------------------------
// TScrollBar: A subclass of TCtlMgr that maps between longint values for our view
// coordinates and the integers of a toolbox scrollbar.
//----------------------------------------------------------------------------------------

class TScrollBar : public TCtlMgr
{
public:
	VHSelect fDirection;						// is this a horizontal or vertical
                                                // scrollbar


	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TScrollBar into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

	virtual pascal void IScrollBar(TView* itsSuperView,
								   const VPoint& itsLocation,
								   const VPoint& itsSize,
								   SizeDeterminer itsHSizeDet,
								   SizeDeterminer itsVSizeDet,
								   VHSelect itsDirection,
								   long itsVal,
								   long itsMin,
								   long itsMax);
		// Initialize a scrollbar procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize a scrollbar from a resource template.


	//------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a dialog view out to a resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a scrollbar to a resource, works in conjunction with tbe previous
        // method.

	virtual pascal void DeltaValue(VCoordinate delta);
		// Does range checking on 'delta' checking it against fLongMin & fLongMax for the
        // scrollbar.

	virtual pascal void DoMouseCommand(VPoint& theMouse,
									   TToolboxEvent* event,
									   Point hysteresis);
		// Handles the mouse down in the various parts of the scrollbar.

	virtual pascal void TrackScrollBar(short partCode);
		// Tracks the scrollbar by calling DeltaValue.

	virtual pascal void ActionProc(short partCode);
		// Forwarded to from the global routine: ActionProcForTScrollBar which was set as
        // the ScrollBar's action proc.Typically forwards again to TrackScrollBar after
        // bounds checking and Updates before returning to the ScrollBar.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents to this class's
        // fields.
};

//----------------------------------------------------------------------------------------
// TScrollerScrollBar: A subclass of TScrollBar that is associated with one or more
// scrollers.
//----------------------------------------------------------------------------------------

class TScrollerScrollBar : public TScrollBar
{
public:	
	TList* fScrollers;							// the associated scrollers if any


	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TScrollerScrollBar into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IScrollerScrollBar(TView* itsSuperView,
									const VPoint& itsLocation,
									const VPoint& itsSize,
									SizeDeterminer itsHSizeDet,
									SizeDeterminer itsVSizeDet,
									VHSelect itsDirection,
									long itsMax,
									TScroller* itsScroller);
		// Initialize the scrollbar procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the scrollbar from a resource template.

	virtual pascal void Free();
		// If the scrollers are not NIL then first detach the scrollbars from them before
        // freeing the scrollers, then call inherited Free.


	//------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing an sscrollbar out to a resource.

	virtual pascal void AttachScroller(TScroller* itsScroller);
		// IF 'itsScroller' is not NIL then add it to our list and attach ourselves to it.

	virtual pascal void BeInPort(GrafPtr itsPort);
		// Call inherited BeInPort and then set our visibility based on the state of the
        // window we are in.

	virtual pascal void Activate(Boolean entering);
		// If 'entering' is TRUE then show ourselves, otherwise just call Draw to frame
        // our area.

	virtual pascal void DoMouseCommand(VPoint& theMouse,
									   TToolboxEvent* event,
									   Point hysteresis);
		// Handles mouse downs in the scrollbar.

	virtual pascal void Draw(const VRect& area);
		// Draws a frame around the controls area before calling inherited Draw.

	virtual pascal void TrackScrollBar(short partCode);
		// Called to handle tracking of the scrollbar by calling DeltaValue.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TControlTracker: A Tracker adapted for tracking controls.
//----------------------------------------------------------------------------------------

class TControlTracker : public TNoChangesTracker
{
public:
	TControl* fControl;							// the object of our desire


	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TControlTracker into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IControlTracker(TControl* theControl, const VPoint& itsMouse);
		// Initialize the tracker procedurally.


	//------------------------------------------------------------------------------------
	// Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};

#endif


