// UDependencies.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UDEPENDENCIES__
#define __UDEPENDENCIES__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif


//----------------------------------------------------------------------------------------
// The dependency relationships are stored in a single array of triples. This is not
// space-efficient , but each entry is only ten bytes, so you should be able to have
// hundreds of dependencies with no problem. The array is sorted, so we can iterate over
// just the necessary triples, by doing a binary search for a matching triple, and then
// iterating forward and backward from there. */
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kNotifierGreater = kItem1GreaterThanItem2 + 2;
	// Values returned by TDependencies.CompareElements
	
const short kNotifierLess = kItem1LessThanItem2 - 2;

const short kChangeGreater = kItem1GreaterThanItem2 + 1;

const short kChangeLess = kItem1LessThanItem2 - 1;

const long kNoLabel = MAXLONGINT - 1;


//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

typedef long DependencyLabels;
	// The valid range of labels

typedef long LabelRange;
	// Used internally because we flip the label's sign to mark an object

typedef pascal void (*DoToDependencyType)(ArrayIndex index,
										  TObject* notifier,
										  TObject* dependent,
										  DependencyLabels dependencyLabel,
										  void* staticLink);


//----------------------------------------------------------------------------------------
// Dependency
//----------------------------------------------------------------------------------------

struct Dependency
{
	TObject* notifier;
	TObject* dependent;
	LabelRange dependencyLabel;
	
	Dependency(TObject* notifier,
				TObject* dependent,
				LabelRange dependencyLabel);
		// Constructor
};

typedef struct Dependency *DependencyPtr;

inline Dependency::Dependency(TObject* aNotifier,
							TObject* aDependent,
							LabelRange aDependencyLabel)
{
	notifier = aNotifier;
	dependent = aDependent;
	dependencyLabel = aDependencyLabel;
}


//----------------------------------------------------------------------------------------
// TDependencies: An array for storing the dependency relationships
//----------------------------------------------------------------------------------------

class TDependencies : public TSortedDynamicArray
{
public:
	virtual pascal void IDependencies();
		// Initialize the array


	//------------------------------------------------------------------------------------
	// Insertion and Deletion
	//------------------------------------------------------------------------------------

	virtual pascal void AddDependency(TObject* notifier,
									  TObject* dependent,
									  DependencyLabels dependencyLabel);
		// Register an object ("dependent") as dependent on the specified change in
        // another object ("Notifier")

	virtual pascal void InsertDependencyBefore(ArrayIndex index,
											   TObject* notifier,
											   TObject* dependent,
											   DependencyLabels dependencyLabel);
		// Add a dependency relation to the array

	virtual pascal void RemoveDependency(TObject* notifier,
										 TObject* dependent,
										 DependencyLabels dependencyLabel);
		// Remove a dependency relation from the array if it is present


	//------------------------------------------------------------------------------------
	// Searching
	//------------------------------------------------------------------------------------

	virtual pascal Boolean FindDependency(TObject* notifier,
										  TObject* dependent,
										  DependencyLabels dependencyLabel,
										  ArrayIndex& index);
		// Returns TRUE if the specified dependency is in the array. The position of the
        // dependency is returned in "index". If the dependency is not present the
        // position at which it should be inserted is returned in "index"


	//------------------------------------------------------------------------------------
	// Debugging
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

	virtual pascal void DynamicFields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // dynamic area.
};


//----------------------------------------------------------------------------------------
// TDependenciesByNotifier
//----------------------------------------------------------------------------------------

class TDependenciesByNotifier : public TDependencies
{
public:
	virtual pascal CompareResult CompareElements(void* Element1, void* Element2);
		// Orders the elements of the array by the ordinal value of the notifier.

	virtual pascal Boolean FindNotifier(TObject* notifier, ArrayIndex& index);

	virtual pascal void EachDependencyWithNotifier(TObject* notifier,
												   DoToDependencyType DoToDependency,
												   void* staticLink);

	virtual pascal void EachDependentDo(TObject* notifier,
										DoToObjectType DoToDependent,
										void* staticLink);
		// Calls DoToDependent on each of the objects which is dependent on the specified
        // change in the object "Notifier"
};


//----------------------------------------------------------------------------------------
// TDependenciesByDependent
//----------------------------------------------------------------------------------------

class TDependenciesByDependent : public TDependencies
{
public:

	virtual pascal CompareResult CompareElements(void* Element1,
												 void* Element2);
		// Orders the elements of the array by the ordinal value of the dependent.

	virtual pascal Boolean FindDependent(TObject* dependent,
										 ArrayIndex& index);

	virtual pascal void EachDependencyWithDependent(TObject* dependent,
													DoToDependencyType DoToDependency,
													void* staticLink);

	virtual pascal void EachNotifierDo(TObject* dependent,
									   DoToObjectType DoToNotifier,
									   void* staticLink);
		// Calls DoToNotifier on each of the objects on which the specified dependent is
        // dependent
};


//----------------------------------------------------------------------------------------
// Externally available procedural Interface to this unit
//----------------------------------------------------------------------------------------

extern pascal void AddDependentOf(TObject* notifier,
								  TObject* dependent,
								  DependencyLabels dependencyLabel);
	// Register an object ("dependent") as a dependent of another object ("Notifier"),
    // with specified label (can be kNoLabel)

extern pascal void RemoveDependentOf(TObject* notifier,
									 TObject* dependent,
									 DependencyLabels dependencyLabel);
	// Remove the dependency relation if it is present

extern pascal void RemoveDependencies(TObject* anObject);
	// Remove all dependencies containing the object, either as Notifier or dependent.
    // Called by TObject.Free.

extern pascal void EachDependentOf(TObject* notifier,
								   DoToObjectType DoToDependent,
								   void* staticLink);
	// Calls DoToDependent on each of the objects which is dependent on the specified
    // change in the object "Notifier"

extern pascal void EachNotifierOf(TObject* dependent,
								  DoToObjectType DoToNotifier,
								  void* staticLink);
	// Calls DoToNotifier on each of the objects on which the specified dependent depends

extern pascal void MarkObject(TObject* anObject, Boolean state);
	// Used to mark or unmark an object as needing updating. Used during a change/update
    // phase

extern pascal Boolean IsObjectMarked(TObject* anObject);
	// Returns TRUE if the specified object is currently marked

#endif


