// UDesignator.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UDESIGNATOR__
#define __UDESIGNATOR__

//----------------------------------------------------------------------------------------
// OVERVIEW
// ========
// TDesignator is designed as an abstract superclass which designates a portion of the
// document. Each specialized document will require its own specialized designator.
// Specialized subclasses of TDesignator are required to designate portions of different
// documents.
//  
// CLASS HIERARCHY
// ===============
// TDesignatorabstract superclass for designators
// TLinearDesignatora linear data designator (e.g. for text models)
// TRectDesignatora rectangular designator (e.g. for spreadsheet models)
// TRgnDesignatora region designator (e.g. for discontiguous designations)
//
// USAGE
// =====
// TDesignator is used with the Edition Manager support that is built into MacApp.
// Whenever a TSection object is instantiated, it requires a TDesignator object that lets
// the TSection object determine which portion of the document is being designated as a
// section, whether a publisher or a subscriber.
//----------------------------------------------------------------------------------------


#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif


//----------------------------------------------------------------------------------------
// Containment constants
//----------------------------------------------------------------------------------------

const short kNotContained = 0;					// designator is not contained 

const short kPartiallyContained = 1;			// designator is partially contained 

const short kFullyContained = 2;				// designator is fully contained 


//----------------------------------------------------------------------------------------
// Object Signtures: These follow the same reservation rules as resource types. The
// signature is used to search the signature table and create an object "by signature"If
// the signature's entry in the table has been updated with a new class then an object of
// that class will be created.
//----------------------------------------------------------------------------------------

const unsigned long kLinearDesignator = 'ldsg';

const unsigned long kVRectDesignator = 'vdsg';

const unsigned long kRgnDesignator = 'rdsg';


/*  C O N T A I N M E N T  */
typedef char ContainmentType;


//----------------------------------------------------------------------------------------
// Forward and externally class declarations.
//----------------------------------------------------------------------------------------

class TDocument;
class TStream;

//----------------------------------------------------------------------------------------
// TDesignator: is designed as an abstract superclass. Use subclasses of TDesignator.
//----------------------------------------------------------------------------------------

class TDesignator : public TObject
{
public:
	TDocument* fDocument;						// the document this designator refers to


	//----------------------------------------------------------------------------------------
	// contructors/destructors
	//----------------------------------------------------------------------------------------

	virtual pascal void IDesignator(TDocument* itsDocument);
		// initialize this object

	virtual pascal void Initialize();

	virtual pascal void InitFromStream(TDocument* itsDocument, TStream* aStream);
		// initializes this object from a stream specification


	//----------------------------------------------------------------------------------------
	// data accessors
	//----------------------------------------------------------------------------------------

	virtual pascal IDType GetSignature();
		// returns the signature for this class */

	virtual pascal Size GetSize();
		// returns the size of this designation */


	//----------------------------------------------------------------------------------------
	// saving
	//----------------------------------------------------------------------------------------

	virtual pascal void WriteToStream(TStream* aStream);
		// OVERRIDE this; writes specification of designator to stream

	virtual pascal Size GetSpecificationSize();
		// OVERRIDE this; returns the size in bytes required to save the specification 


	//----------------------------------------------------------------------------------------
	// inspecting 
	//----------------------------------------------------------------------------------------
	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields


	//----------------------------------------------------------------------------------------
	// containment
	//----------------------------------------------------------------------------------------

	virtual pascal ContainmentType IsContained(TDesignator* aDesignator);
		// OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in SELF
};


//----------------------------------------------------------------------------------------
// TLinearDesignator: is designed to designate any linear portion of data that may be
// designated by a start position and an end position.
//----------------------------------------------------------------------------------------

class TLinearDesignator : public TDesignator
{
public:
	long fStartPos;								// offset to start
	
	long fEndPos;								// offset to end

	//----------------------------------------------------------------------------------------
	// contructors/ destructors
	//----------------------------------------------------------------------------------------

	virtual pascal void ILinearDesignator(TDocument* itsDocument,
										  long startPos,
										  long endPos);
		// initialize this object

	virtual pascal void Initialize();

	virtual pascal void InitFromStream(TDocument* itsDocument, TStream* aStream);
		// initializes this object from a stream specification


	//----------------------------------------------------------------------------------------
	// data accessors ===
	//----------------------------------------------------------------------------------------

	virtual pascal IDType GetSignature();
		// returns the signature for this class

	virtual pascal Size GetSize();
		// returns the size of this designation


	//----------------------------------------------------------------------------------------
	// saving
	//----------------------------------------------------------------------------------------

	virtual pascal Size GetSpecificationSize();
		// OVERRIDE this; returns the size in bytes required to save the specification

	virtual pascal void WriteToStream(TStream* aStream);
		// OVERRIDE this; writes specification of designator to stream


	//----------------------------------------------------------------------------------------
	// inspecting
	//----------------------------------------------------------------------------------------
	
	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields


	//----------------------------------------------------------------------------------------
	// containment
	//----------------------------------------------------------------------------------------
	virtual pascal ContainmentType IsContained(TDesignator* aDesignator);
		// OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in SELF
};


//----------------------------------------------------------------------------------------
// TRectDesignator: is designed to designate any rectangular portion of data that may be
// designated by top, left, bottom, and right VCoordinates.
//----------------------------------------------------------------------------------------

class TVRectDesignator : public TDesignator
{
public:
	VRect fDesignation;							// rect representing designation

	//----------------------------------------------------------------------------------------
	// contructors/destructors
	//----------------------------------------------------------------------------------------

	virtual pascal void IVRectDesignator(TDocument* itsDocument,
										 const VRect& itsDesignation);
		// initialize this object

	virtual pascal void Initialize();

	virtual pascal void InitFromStream(TDocument* itsDocument, TStream* aStream);
		// initializes this object from a stream specification


	//----------------------------------------------------------------------------------------
	// data accessors
	//----------------------------------------------------------------------------------------

	virtual pascal IDType GetSignature();
		// returns the signature for this class

	virtual pascal Size GetSize();
		// returns the size of this designation


	//----------------------------------------------------------------------------------------
	// saving
	//----------------------------------------------------------------------------------------

	virtual pascal Size GetSpecificationSize();
		// OVERRIDE this; returns the size in bytes required to save the specification

	virtual pascal void WriteToStream(TStream* aStream);
		// OVERRIDE this; writes specification of designator to stream


	//----------------------------------------------------------------------------------------
	// inspecting
	//----------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields


	//----------------------------------------------------------------------------------------
	// containment
	//----------------------------------------------------------------------------------------

	virtual pascal ContainmentType IsContained(TDesignator* aDesignator);
		// OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in SELF
};


//----------------------------------------------------------------------------------------
// TRgnDesignator: is designed to designate any portion of data that may be designated by
// a region. The caller must create the region, and the designator optionally frees the
// region. In general, the TStream passed into this object should be a THandleStream.
//----------------------------------------------------------------------------------------

class TRgnDesignator : public TDesignator
{
public:
	RgnHandle fDesignation;						// rgn representing designation

	Boolean fFreeRgnOnFree;						// whether to free the RgnHandle when this
                                                // designator is freed


	//----------------------------------------------------------------------------------------
	// contructors/destructors
	//----------------------------------------------------------------------------------------

	virtual pascal void IRgnDesignator(TDocument* itsDocument,
									   RgnHandle itsDesignation,
									   Boolean freeRgnOnFree);
		// initialize this object

	virtual pascal void Initialize();
		// Initializes this object

	virtual pascal void InitFromStream(TDocument* itsDocument, TStream* aStream);
		// initializes this object from a stream specification

	virtual pascal void Free();
		// frees fDesignation and calls INHERITED Free

	virtual pascal TObject* Clone();
		// calls INHERITED Clone, creates a copy of fDesignation rgn


	//----------------------------------------------------------------------------------------
	// data accessors
	//----------------------------------------------------------------------------------------

	virtual pascal IDType GetSignature();
		// returns the signature for this class

	virtual pascal Size GetSize();
		// returns the size of this designation


	//----------------------------------------------------------------------------------------
	// saving
	//----------------------------------------------------------------------------------------

	virtual pascal Size GetSpecificationSize();
		// OVERRIDE this; returns the size in bytes required to save the specification

	virtual pascal void WriteToStream(TStream* aStream);
		// OVERRIDE this; writes specification of designator to stream


	//----------------------------------------------------------------------------------------
	// inspecting
	//----------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields


	//----------------------------------------------------------------------------------------
	// containment
	//----------------------------------------------------------------------------------------

	virtual pascal ContainmentType IsContained(TDesignator* aDesignator);
		// OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
        // is contained in SELF
};


//----------------------------------------------------------------------------------------
// Externally available function declarations
//----------------------------------------------------------------------------------------

extern pascal void InitUDesignator();
	// initializes the designator unit, registering the standard types recognized by this
    // unit


#endif


