// UEvtHandler.h
//  Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UEVTHANDLER__
#define __UEVTHANDLER__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif


//----------------------------------------------------------------------------------------
// Some miscellaneous declarations.
//----------------------------------------------------------------------------------------

enum IdlePhase { idleBegin, idleContinue, idleEnd };


//----------------------------------------------------------------------------------------
// Forward and external classes declarations.
//----------------------------------------------------------------------------------------

class TCommand;
class TDocument;
class TView;
class TEvent;
class TToolboxEvent;
class TEvtHandler;
class TBehavior;


//----------------------------------------------------------------------------------------
// Procedural typedefs
//----------------------------------------------------------------------------------------

typedef pascal void (*DoToEvtHandlerType)(TEvtHandler* anEvtHandler, void* staticLink);

typedef pascal Boolean (*TestEvtHandlerType)(TEvtHandler* anEvtHandler, void* staticLink);

typedef pascal void (*DoToViewType)(TView* aView, void* staticLink);


//----------------------------------------------------------------------------------------
// TEvtHandler: This type represents abstract objects that handle certain kinds of events:
// Key events: both key down and auto key events, menu events: both enabling menus and
// menu items and processing menu commands, Read to/ write from Disk, Idle events: when
// there are no other events to handle, Actual events: EvtHandlers - through their method
// 'DoHandleEvent' may intercept and handle actual ToolBox events Termination: when the
// application quits TEvtHandler are linked into a list with the most specific object
// (usually a selection) at the head of the list. 'TApplication.fTarget' contains the head
// of the list. (When a window is deactivated, this global variable is cached in the
// window object, and retrieved when it is later activated.) For these kinds of events,
// the TApplication.fTarget gets the first crack at handling the event. The default
// implementation of the methods of TEvtHandler is to pass the event to the next element
// of the list.
//----------------------------------------------------------------------------------------

class TEvtHandler : public TObject
{
public:
	TEvtHandler* fNextHandler;					// the next element of the list, or NIL
	
	TBehavior*	fBehavior;						// Behavioral modifier object
	
	long fIdleFreq;								// the minimum number of ticks that can
                                                // pass before my DoIdle is called. 0 =
                                                // call as often as possible.
                                                // n..kMaxIdleTime = call every fIdleFreq
                                                // ticks.
												
	long fLastIdle;								// the tick count the last time my DoIdle
                                                // was called.

	Boolean fEnabled;							// is this eventHandler enabled. (does it
                                                // take clicks/keystrokes and other
                                                // events? 
	IDType fIdentifier;							// the ID if any
												

	//------------------------------------------------------------------------------------
	// Initializer, Pascal I<Method> and Free.
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TEvtHandler into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

		
	virtual pascal void IEvtHandler(TEvtHandler* itsNextHandler);
		// Call IObject and set fNextHandler to itsNextHandler.
		
public:
	virtual pascal void Free();
		// Before calling inherited Free, changes the target to the application if the
        // next handler is NIL or the next handler if there is one.


	//------------------------------------------------------------------------------------
	// Termination
	//------------------------------------------------------------------------------------

	virtual pascal void Terminate();
		// Does nothing in this class.


	//------------------------------------------------------------------------------------
	// Chainwalking
	//------------------------------------------------------------------------------------

	virtual pascal TEvtHandler* GetNextHandler();
		// Return the next handler in the chain

	virtual pascal void EachHandler(DoToEvtHandlerType DoToEvtHandler, void* staticLink);
		// Performs 'DoToEvtHandler' to aFirstHandler, then to its fNextHandler, etc.,
        // onward until the fNextHandler chain ends at NIL

	virtual pascal TEvtHandler* FirstHandlerThat(TestEvtHandlerType TestEvtHandler,
												 void* staticLink);
		// Calls TestEvtHandler for each event handler until TestEvtHandler returns true.
        // Returns the TEvtHandler object for which TestEvtHandler returned true, or NIL
        // if TestEvtHandler never returned true.

	virtual pascal TEvtHandler* AddHandler(TEvtHandler* headOfChain);
		// Adds SELF from the chain of handlers starting at headOfChain. Returns the new
        // head of chain.

	virtual pascal TEvtHandler* RemoveHandler(TEvtHandler* headOfChain);
		// Removes SELF from the chain of handlers starting at headOfChain. Returns the
        // new head of chain (if self was the head then there _must_ be a new head.


	//------------------------------------------------------------------------------------
	// Debugging
	//------------------------------------------------------------------------------------

	virtual pascal long LookupSymbol(Str255& sym);
		// Called by the debugger to translate a symbol into a number; default passes to
        // the fNextHandler. sym will be uppercased. Return of -1 means symbol was not
        // found. If sym = '?' then writeln a list of available symbols.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.


	//------------------------------------------------------------------------------------
	// Events
	//------------------------------------------------------------------------------------

	virtual pascal long NextIdle();
		// Returns the number Tickcount for when the handler should next be idled.

	virtual pascal void HandleIdle(IdlePhase phase);
		// Calls DoIdle if the handler is eligible.

	virtual pascal Boolean DoIdle(IdlePhase phase);
		// Returns true if it frees itself so caller can know

	virtual pascal Boolean DoCoHandlerEvent(TEvent* event);
		// Handle the event and return TRUE if I want it, else I don't handle it, and
        // return FALSE

	virtual pascal void DoEvent(EvtNumber eventNumber, TEvtHandler* source,
								TEvent* event);
		// By default, asks the next handler to handle the event

	virtual pascal void HandleEvent (EvtNumber eventNumber,
									 TEvtHandler* source,
									 TEvent* event);
		// Responsible for dispatching DoHandleEvent messages
	
	virtual pascal void SetEnable(Boolean state);
		// Enables based on 'state'

	virtual pascal Boolean IsEnabled();
		// Returns the enabled state.


	//------------------------------------------------------------------------------------
	// Double/ Triple Clicks
	//------------------------------------------------------------------------------------

	virtual pascal Boolean DoMultiClick(Point lastDownPt, Point newDownPt);
		// Called by TApplication.CountClicks. Should return TRUE if the 2 points are
        // close enough to be considered part of a double/ triple click. (Both points are
        // in global coordinates.) This is only called if the mouse down was within the
        // proper time range of the previous mouse up. Default is to pass message to the
        // nextEventHandler (if one exists), otherise to require that the sum of the x & y
        // distances is <= gStdHysteresis.


	//------------------------------------------------------------------------------------
	// Key Events
	//------------------------------------------------------------------------------------

	virtual pascal void DoKeyCommand(TToolboxEvent* event);
		// Handles all keystrokes except those with the command key held down.

	virtual pascal void HandleKeyCommand(TToolboxEvent* event);
		// Handles the dispatching of DoKeyCommand messages between itself and Behaviors
	
	virtual pascal void DoCommandKey(TToolboxEvent* event);
		// Handles command-key combinations only.

	virtual pascal void HandleCommandKey(TToolboxEvent* event);
		// Handles the dispatching of DoCommandKey messages between itself and Behaviors
	
	virtual pascal void KeyEventToComponents(TToolboxEvent* event);
		// Intended to extract character components of an event in a script manager
        // compatible way.Default just forwards to next handler


	//------------------------------------------------------------------------------------
	// MenuEvents
	//------------------------------------------------------------------------------------

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
		// If the handler can perform the command it does so by either performing the
        // command directly or by posting a TCommand.

	virtual pascal void HandleMenuCommand(CmdNumber aCmdNumber);
		// Handles the dispatching of DoMenuCommand messages
	
	virtual pascal void DoSetupMenus();
		// Handles the setup of menus.

	virtual pascal void HandleSetupMenus();
		// Handles the dispatching of DoSetupMenus messages
	

	//------------------------------------------------------------------------------------
	// AppleEvents
	//------------------------------------------------------------------------------------

	virtual pascal void DoAppleCommand(CmdNumber aCmdNumber,
									   const AppleEvent& message,
									   const AppleEvent& reply);
		// If the handler can perform the apple event command it does so by either
        // performing the command directly or by posting a TCommand.

	virtual pascal void HandleAppleCommand(CmdNumber aCmdNumber,
											 const AppleEvent& message,
											 const AppleEvent& reply);
		// Handles the dispatching of DoAppleCommand messages
	

	//------------------------------------------------------------------------------------
	// Help
	//------------------------------------------------------------------------------------

	virtual pascal void DoHelp(TToolboxEvent* event, long& message);
		// If fNextHandler is not NIL calls its DoHelp method, otherwise returns NIL.

	//------------------------------------------------------------------------------------
	// Target management
	//------------------------------------------------------------------------------------

	virtual pascal Boolean WantToBecomeTarget();
		// Called to find out if this object wishes to become the target. Default returns
        // FALSE

	virtual pascal long WillingToResignTarget();
		// Called to find out if this object is willing to cease being the target. Default
        // returns TRUE

	virtual pascal void TargetValidationSucceeded();
	virtual pascal void TargetValidationFailed(long reason);

	virtual pascal void BecameWindowTarget();
		// Called when this object becomes the current target of a window

	virtual pascal void ResignedWindowTarget();
		// Called when this object ceases to be the current target of a window

	virtual pascal void BecameApplicationTarget();
		// Called when this object becomes the current target

	virtual pascal void ResignedApplicationTarget();
		// Called when this object ceases to be the current target


	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual pascal void SetIdleFreq(long newIdleFreq);
		// Call to set the handler's idling frequency.

	virtual pascal void RemoveBehavior (TBehavior* theBehavior);
		// Remove theBehavior from the behavior chain
	
	virtual pascal void AddBehavior (TBehavior* theBehavior);
		// Call to add theBehavior to this object
	
	virtual pascal TBehavior* GetFirstBehavior (void);
		// Call to retrieve the first behavior currently attached to EvtHandler


	//------------------------------------------------------------------------------------
	// Command Management
	//------------------------------------------------------------------------------------

	virtual pascal void CommitLastCommand();
		// Commits the last command by calling its Commit method. Default is to hand off
        // to fNextHandler

	virtual pascal TCommand* GetLastCommand();
		// Returns the last undoable command.The command returned is not yet committed.
        // Default is to hand off to fNextHandler

	virtual pascal TEvent* RetrieveAnEvent();
		// Retrieves a previously posted event/ command from the queue. Nil if there are no
        // queued commands. Default is to hand off to fNextHandler

	virtual pascal void PerformCommand(TCommand* command);
		// Called to perform the given command. Default is to hand off to fNextHandler

	virtual pascal void PostAnEvent(TEvent* event);
		// Called to post an event or command to a queue for later processing. Default is
        // to hand off to fNextHandler

	virtual pascal void PostCommand(TCommand* command);
		// Called to post a command to a queue for later execution. Left in for
        // compatibility - calls PostAnEvent
};


//----------------------------------------------------------------------------------------
// CHandlerIterator: A simple iterator for the handler chain.
//----------------------------------------------------------------------------------------

class CHandlerIterator: public CIterator
{
private:
	TEvtHandler*	fFirstHandler;

	TEvtHandler*	fCurrentHandler;

	TEvtHandler*	fNextHandler;
	
public:
	inline CHandlerIterator(TEvtHandler* chainHead);
	
	virtual Boolean More(void);					// override
		// Returns TRUE if there are more elements to iterate over

	virtual void Reset(void);					// override
		// Resets the iterator to begin again
	
	inline TEvtHandler* CurrentHandler(void);
		// returns the current handler in the handler chain

	inline TEvtHandler* FirstHandler(void);
		// Resets the iterator to begin again and returns the first handler in the handler chain

	inline TEvtHandler* NextHandler(void);
		// returns the next handler in the handler chain
	
protected:
	virtual void Advance(void);					// override
		// Advances the iteration
};


//----------------------------------------------------------------------------------------
// CHandlerIterator: Inline method definitions.
//----------------------------------------------------------------------------------------

inline CHandlerIterator::CHandlerIterator(TEvtHandler* chainHead)
{
	fFirstHandler = chainHead;
	fCurrentHandler = chainHead;
	// Retrieve next handler now since DoToEvtHandler may free fCurrentHandler
	if (chainHead)
		fNextHandler = chainHead->GetNextHandler();
	else
		fNextHandler = NULL;
}

inline TEvtHandler* CHandlerIterator::CurrentHandler()
{
	return fCurrentHandler;
}

inline TEvtHandler* CHandlerIterator::FirstHandler()
{
	this->Reset();
	return fFirstHandler;
}

inline TEvtHandler* CHandlerIterator::NextHandler()
{
	this->Advance();
	return fCurrentHandler;
}


//----------------------------------------------------------------------------------------
// Global variables
//----------------------------------------------------------------------------------------

extern Point gStdHysteresis;					// standard hysteresis used in
                                                // TCommand.ICommand

#endif


