// UFileBasedDocument.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UFILEBASEDDOCUMENT__
#define __UFILEBASEDDOCUMENT__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __GEOMETRY__
#include <Geometry.h>
#endif

#ifndef __UDOCUMENT__
#include <UDocument.h>
#endif


//----------------------------------------------------------------------------------------
//  Constants for the print info resource
//----------------------------------------------------------------------------------------

const unsigned long kPrintInfoRsrcType = 'PRNT';
const short kPrintInfoRsrcID = 1;


//----------------------------------------------------------------------------------------
// Forward and External class declarations
//----------------------------------------------------------------------------------------

class TFile;
class TFileHandler;


//----------------------------------------------------------------------------------------
// TFileBasedDocument: Corresponds to a Finder document. Managed document data in files
// and main memory.
//----------------------------------------------------------------------------------------

class TFileBasedDocument : public TDocument
{
public:
	TFileHandler*	fFileHandler;					// volume/directory/filename

	OSType			fScrapType;						// document's private scrap type


	virtual pascal void Initialize();
		// Puts a newly created TDocument into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

	virtual pascal void IFileBasedDocument(TFile* itsFile,
										   const OSType itsFileType,
										   const OSType itsCreator,
										   const OSType itsScrapType,
										   Boolean usesDataFork,
										   Boolean usesRsrcFork,
										   Boolean keepsDataOpen,
										   Boolean keepsRsrcOpen);
		// Initialization method for TFileBasedDocument.

	virtual pascal TFileHandler* DoMakeFileHandler(TFile* itsFile,
												   const OSType itsFileType,
												   const OSType itsCreator,
												   Boolean usesDataFork,
												   Boolean usesRsrcFork,
												   Boolean keepsDataOpen,
												   Boolean keepsRsrcOpen);
		// Creates and returns a TFileHandler for this object.

	virtual pascal void Free();
		// This does not call FreeData by default, since you may need to control the order
        // in which things are freed. Your override of TDocument.Free can call FreeData if
        // convenient.

	virtual pascal void Fields(TObject* obj);
		// Called by the Inspector and the Debugger to display the contents of this
        // class's fields.


	//------------------------------------------------------------------------------------
	// Opening Documents
	//------------------------------------------------------------------------------------

	virtual pascal void AboutToSaveFile(CmdNumber itsCmd, Boolean& makingCopy);
		// Called from file handler right after the SFPutFile (if any) and before deciding
        // between SaveViaTemp & SaveInPlave. Applications can use the name in an alert
        // and/or modify the makingCopy parameter if they do not want to have the document
        // renamed.

	virtual pascal Boolean AlreadyOpen(TFile* aFile);
		// True if this file is already open

	virtual pascal void DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes);
		// Bytes required to store SAVE file(s) on disk. When called, both parameters are
        // set to 0 for you.

	virtual pascal void DoRead(TFile* aFile, Boolean forPrinting);
		// The default behaviour is to just read the printinfo from the document's
        // data-fork by calling DoReadPrintInfo (useful huh?)

	virtual pascal void DoReadPrintInfo(TFile* aFile, Boolean forPrinting);
		// The default behaviour is to just read the printinfo from the document's
        // data-fork (useful huh?)

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);
		// makingCopy indicates whether this save is for making a copy of the document
        // (vs. a regular Save or Save As); disk-based documents (especially) might
        // optimize their behavior if this is TRUE.

	virtual pascal void DoWritePrintInfo(TFile* aFile, Boolean makingCopy);
		// makingCopy indicates whether this save is for making a copy of the document
        // (vs. a regular Save or Save As); disk-based documents (especially) might
        // optimize their behavior if this is TRUE. The default behaviour is to just write
        // the printinfo to the document's data-fork (useful huh?) */

	virtual pascal void ReadDocument(Boolean forPrinting);
		// Called to read an existing document for display or printing.


	//------------------------------------------------------------------------------------
	// Saving Documents
	//------------------------------------------------------------------------------------

	virtual pascal void SaveDocument(CmdNumber itsCmdNumber);
		// Try to save the document to disk

	virtual pascal void FileHasBeenSaved(const Str255& newName);
		// Called when a save successfully completes and we are switching to the new file
        // (ie., not a Save a Copy In).

	virtual pascal void SFPutParms(CmdNumber itsCmdNumber,
								   Str255& prompt,
								   Str255& defaultName,
								   short& dlgID,
								   Point& where,
								   ProcPtr& dlgHook,
								   ProcPtr& modalFilter,
								   Ptr& activeList,
								   ProcPtr& activateProc,
								   Ptr& yourDataPtr);
		// Called to return all the parameters that should be passed to CustomPutFile or
        // SFPutFile. Defaults to prompt gotten from resource file; defaultName not
        // changed;
		//		dlgID = sfPutDialogID or putDlgID;
		//		where = (-1, -1) or (100, 100);
		//		dlgHook = NIL;
		//		modalFilter = NIL;
		//		activeList = NIL;
		//		activateProc = NIL;
		//		yourDataPtr = NIL.


	//------------------------------------------------------------------------------------
	// Revert
	//------------------------------------------------------------------------------------

	virtual pascal void RevertDocument();
		// IF fSaveExists is TRUE then reverts to the file saved on disk, otherwise merely
        // resets the print handler and calls doInitialState. Calls FreeData to free any
        // existing data and commits any currently active command.


	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual pascal void CheckFile(short rsrcId, short rsrcIndex, Boolean reverting);
		// Check to see if the disk file has changed. If so put up alert phFileChanged
        // after setting ^0 to fTitle and ^1 to the string specified by the rsrcId and
        // rsrcIndex. If user cancels, signal Failure(noErr, msgCancelled). If reverting
        // is TRUE, then I/ O errors and nonmatching file types will cause a Failure.
        // Otherwise, this is a Save and any I/ O errors and nonmatching file type will be
        // ignored.

	virtual pascal void CloseFile();

	virtual pascal TFileHandler* GetFileHandler();
		// accessor for fFileHandler

	virtual pascal void SetTitle(const Str255& aTitle);
		// Does fFile.SetName to aTitle and calls SetTitleForDoc for each window of the
        // document.
};

#endif


