#ifndef __UFILEHANDLER__
// UFileHandler.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#define __UFILEHANDLER__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TFileBasedDocument;
class TFile;


enum SaveChoice { svtAlways, svtAskUser, sipAlways, sipAskUser };


//----------------------------------------------------------------------------------------
//  How to save a document.
//  
//  svtAlways:	Always save with SaveViaTemp.Never overwrite the existing file.
//  svtAskUser:	Try to SaveViaTemp.If not possible then ask before SaveInPlace.
//  sipAlways:	Always save with SaveInPlace.Never create a temporary file.
//  sipAskUser: Always save with SaveInPlace, but only if user confirms.
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// TFileHandler
//----------------------------------------------------------------------------------------

class TFileHandler : public TObject
{
public:
	TFileBasedDocument* fDocument;				// The controller of the model

	TFile* fFile;								// volume/directory/filename

	Boolean fFileExists;						// whether a disk file representing this
                                                // model exists

	SaveChoice fHowToSave;						// how to save files


	//------------------------------------------------------------------------------------
	// Init & Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TDocument into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

	virtual pascal void IFileHandler(TFileBasedDocument* itsDocument,
									 TFile* itsFile,
									 const OSType itsFileType,
									 const OSType itsCreator,
									 Boolean usesDataFork,
									 Boolean usesRsrcFork,
									 Boolean keepsDataOpen,
									 Boolean keepsRsrcOpen);
		// Initialization method for TFileHandler.

	virtual pascal void Free();

	virtual pascal void Fields(TObject* obj);
		// Called by the Inspector and the Debugger to display the contents of this
        // class's fields.


	//------------------------------------------------------------------------------------
	// Opening/ Documents
	//------------------------------------------------------------------------------------

	virtual pascal Boolean FileAlreadyOpen(TFile* aFile);
		// True if the file for the model is already open

	virtual pascal void ReadFile(Boolean forPrinting);
		// Called to read an existing file for display or printing.

	virtual pascal void DoRead(Boolean forPrinting);
		// The default behaviour is to just read the printinfo from the document's
        // data-fork by calling DoReadPrintInfo (useful huh?)

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);
		// makingCopy indicates whether this save is for making a copy of the document
        // (vs. a regular Save or Save As); disk-based documents (especially) might
        // optimize their behavior if this is TRUE.


	//------------------------------------------------------------------------------------
	// Saving Documents
	//------------------------------------------------------------------------------------

	virtual pascal void AboutToSaveFile(CmdNumber itsCmd, Boolean& makingCopy);
		// Called right after the SFPutFile (if any) and before deciding between
        // SaveViaTemp & SaveInPlave. Applications can use the name in an alert and/or
        // modify the makingCopy parameter if they do not want to have the document
        // renamed.

	virtual pascal void DoNeedDiskSpace(long& dataForkBytes, long& rsrcForkBytes);
		// Bytes required to store SAVE file(s) on disk. When called, both parameters are
        // set to 0 for you.

	virtual pascal void CloseFile();
		// Called to free in memory data associated with the currently opened document.
        // The default is to call fFile.Close(TRUE) to force the file close

	virtual pascal Boolean GetSaveInfo(CmdNumber itsCmdNumber,
									   Boolean copyFInfo,
									   CInfoPBRec& cInfo);
		// Called to get the desired file info for saving the file. Default copies the
        // info from the current file if fSaveExists AND copyFInfo, otherwise it just sets
        // the file type and creator based on the fields of SELF. If an error occurs
        // calling the file system, the cInfo record will not be filled in. Returns TRUE
        // if it successfully gets the info from the file.

	virtual pascal void GetTempName(Str63& fileName);
		// Return a temporary name for saving the file. The filename is constructed in two
        // parts: the first part is the document's name, or the application's name if the
        // document is untitled. The second part, appended to the first part, is a pseudo-
        // random number based on the tick count and number of seconds since boot.

	virtual pascal void RequestFileName(CmdNumber itsCmdNumber,
										Boolean makingCopy,
										TFile* aFile);
		// Asks the user for a filename for the document. Calls TFileHandler.SFPutParms
        // and then SFPPutFile. Fails if the user clicks the Cancel button to end the put
        // file dialog.

	virtual pascal void SaveFile(CmdNumber itsCmdNumber,
								 Boolean askForFilename,
								 Boolean copyFInfo,
								 Boolean makingCopy);
		// Try to save the document to disk; If askForFilename, put up a Std File box; If
        // makingCopy, then this is for the Save a Copy In command, so we do not rename
        // the document. This method calls FlushVol (which takes care of the requirements
        // of SaveInPlace, and SaveViaTemp.

	virtual pascal void SaveInPlace(CmdNumber itsCmdNumber,
									Boolean makingCopy,
									Boolean copyFInfo,
									TFile* itsFile);
		// Called to save a document in place. Default is to delete the existing file if
        // NOT (fDataOpen OR fRsrcOpen). If you have disk based document, you have to
        // decide whether to allow saving in place. makingCopy is TRUE if this save is
        // making a copy of the document (vs. a regular Save or Save As) copyFInfo is TRUE
        // if the file information should be copied to the new file Caller is responsible
        // for calling FlushVol after this.

	virtual pascal void SaveViaTemp(CmdNumber itsCmdNumber,
									Boolean makingCopy,
									Boolean copyFInfo,
									TFile* itsFile);
		// Called to save the document by creating a temporary copy of the file and
        // renaming it. MacApp uses this method except when there is not enough disk space
        // available. makingCopy is TRUE if this save is making a copy of the document
        // (vs. a regular Save or Save As) copyFInfo is TRUE if the file information
        // should be copied to the new file Caller is responsible for calling FlushVol
        // after this.


	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual pascal OSErr FileChanged(Boolean checkType);
		// Returns noErr if fSaveExists is FALSE or if the file's modification date
        // matches fModDate. If checkType is TRUE then returns errFTypeChanged if the file
        // type has changed. If the file exists but with a different modification date,
        // returns errFileChanged.

	virtual pascal Boolean FileExists();
		// Default returns the status of fFileExists

	virtual pascal Boolean HasRsrcFork();
		// Returns TRUE if the file has a resource fork

	virtual pascal TFile* GetFile();
		// Return the file reference by TFileHandler

	virtual pascal void GetFileName(Str63& aName);
		// Return the name of the file

	virtual pascal void SetFileName(const Str63& aName);
		// Default behavior calls fFile.SetName with aName
};


#endif


