
#ifndef __UGEOMETRY__
#define __UGEOMETRY__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __GEOMETRY__
#include <Geometry.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kMaxCoord = 30000;


//----------------------------------------------------------------------------------------
// Forward and external typedefs and class declarations.
//----------------------------------------------------------------------------------------

class VPoint;

class VRect;

typedef class VPoint *VPointPtr;

typedef class VRect *VRectPtr;


//----------------------------------------------------------------------------------------
// VCoordinates are 32 bits as compared to QD Coordinates which are 16 bits
//----------------------------------------------------------------------------------------

typedef long VCoordinate;


//----------------------------------------------------------------------------------------
// SimpleVPoint: is used as the field type for some structs in the toolbox that we don't
// want promoted to a class; either because of constructor problems or the struct may
// appear in a union. Simple VPoint is also used as the base class for VPoint; we simply
// inherit the data members.
//----------------------------------------------------------------------------------------

struct SimpleVPoint
{
	VCoordinate v;
	VCoordinate h;
};


//----------------------------------------------------------------------------------------
// VPoint: is a toolbox compatible class for the old struct VPoint. It is bitwise
// compatible with the old struct because it contains no virtual functions.
//----------------------------------------------------------------------------------------

class VPoint : public SimpleVPoint
{
public:
	//------------------------------------------------------------------------------------
	// Constructors
	//------------------------------------------------------------------------------------

	VPoint();
	
	VPoint(VCoordinate iV, VCoordinate iH);
		  
	VPoint(const SimpleVPoint& pt);
	
	VPoint(const Point& pt);
	
	
	//------------------------------------------------------------------------------------
	// Copy and conversion operator methods
	//------------------------------------------------------------------------------------

	VPoint Copy() const;
		// Copy simply returns a copy of the object. Useful for creating automatic copies
        // of an object on the stack, in a function call.
	
	operator char*() const;
		// Conversion operator, for converting VPoint to a textual representation of a
        // VPoint. "VPoint(v,h)".
	
	operator Point() const;
		// Conversion operator for turning a VPoint into a Point. Note: This operator
        // returns a new Point not a VPoint viewed as a Point.
	

	//------------------------------------------------------------------------------------
	// Selector operators, two are needed one for operating on const VPoints and the other
    // on non-const VPoints.
	//------------------------------------------------------------------------------------
	
	VCoordinate& operator[](VHSelect sel);
		// For non-const VPoint
		
	const VCoordinate& operator[](VHSelect sel) const;
		// For const VPoint


	//------------------------------------------------------------------------------------
	// Arithmetic operators
	//------------------------------------------------------------------------------------

	VPoint operator+(const VPoint& pt) const;
		// c = a + b	(a,b and c are VPoints)
		
	VPoint operator-(const VPoint& pt) const;
		// c = a - b	(a,b and c are VPoints)

	VPoint& operator+=(const VPoint& pt);
		// c += a		(a and c are VPoints, and
		//				a reference to a is returned,
		//				so these can be strung together)
		
	VPoint& operator-=(const VPoint& pt);
		// c -= a		( ditto )
		
	VPoint operator-() const;
		// c = -a		negate a


	//------------------------------------------------------------------------------------
	// Relational operators, simply apply the relation to both coordinates if condition
    // holds for both coordinates then holds for the point. Exception is != if either
    // coordinate is not equal then points are not equal.
	//------------------------------------------------------------------------------------

	Boolean operator!=(const VPoint& pt) const;
	
	Boolean operator==(const VPoint& pt) const;
	
	Boolean operator>(const VPoint& pt) const;
	
	Boolean operator<(const VPoint& pt) const;
	
	Boolean operator>=(const VPoint& pt) const;
	
	Boolean operator<=(const VPoint& pt) const;
	
	
	void ConstrainTo(const VRect& rt);
		// If the Point is not already inside the rectangle move the point to the nearest
        // edge.
};


//----------------------------------------------------------------------------------------
// Inline method definitions for VPoint.
//----------------------------------------------------------------------------------------

inline VPoint::VPoint() { }

inline VPoint::VPoint(VCoordinate iV, VCoordinate iH)
{
	v = iV;
	h = iH;
}

inline VPoint::VPoint(const SimpleVPoint& pt)
{
	v = pt.v;
	h = pt.h;
}

inline VPoint::VPoint(const Point& pt)
{
	v = pt.v;
	h = pt.h;
}

inline VPoint VPoint::Copy() const
{
	VPoint pt;
	
	pt.v = v;
	pt.h = h;
	
	return pt;
}


//----------------------------------------------------------------------------------------
// SimpleVRect: is used as the field type for some structs in the toolbox that we don't
// want promoted to a class; either because of constructor problems or the struct may
// appear in a union. SimpleVRect is also used as the base class for VRect; we simply
// inherit the data members.
//----------------------------------------------------------------------------------------

struct SimpleVRect
{
	VCoordinate top;
	VCoordinate left;
	VCoordinate bottom;
	VCoordinate right;
};


//----------------------------------------------------------------------------------------
// VRect is a toolbox compatible class for the old struct VRect. It is bitwise compatible
// with the old struct because it contains no virtual functions.
//----------------------------------------------------------------------------------------

class VRect : public SimpleVRect
{
public:
	//------------------------------------------------------------------------------------
	// VRect constructors
	//------------------------------------------------------------------------------------

	VRect();
	
	VRect(VCoordinate iTop, VCoordinate iLeft, VCoordinate iBottom, VCoordinate iRight);
		 
	VRect(const VRect& rect);
	
	VRect(const VPoint& topLeftPt, const VPoint& botRightPt);
		 
	VRect(const SimpleVRect& rect);
	
	VRect(const Rect& rect);


	//------------------------------------------------------------------------------------
	// Copy and conversion operator methods
	//------------------------------------------------------------------------------------
	
	VRect Copy() const;
		// Copy simply returns a copy of the object. Useful for creating automatic copies
        // of an object on the stack, in a function call.
	
	operator char*() const;
		// Conversion operator, for converting VRect to a textual representation of a
        // VRect. "VRect(top,righ,bottom,left)".
	
	operator Rect() const;
		// Conversion operator for turning a VRect into a Rect. Note: This operator
        // returns a new Rect not a VRect viewed as a Rect.
	
	
	
	//------------------------------------------------------------------------------------
	// VPoint selectors for VRect. One for operating on const VRects and one for non-const
    // VRects.
	//------------------------------------------------------------------------------------

	VPoint& operator[](PointSelector sel);
		// Selector for non-const VRects
		
	const VPoint& operator[](PointSelector sel) const;
		// Selector for const VRects


	//------------------------------------------------------------------------------------
	// Operators for adding and subtracting one VRect from to/from another.
	//------------------------------------------------------------------------------------

	VRect operator+(const VRect& rt) const;
	
	VRect operator-(const VRect& rt) const;
	
	VRect& operator+=(const VRect& rt);
	
	VRect& operator-=(const VRect& rt);


	//------------------------------------------------------------------------------------
	// Operators overloaded for adding and subtracting some increment to/from each point
    // along both axis. Very convenient for translating VRects. These take a point and
    // since VPoint has a constructor that takes two VCoordinates the VRect windowVRect
    // can be translated 100 pixels in the positive y direction by the statement:
	//
	//	windowVRect = windowVRect + VPoint (0, 100);	or
	// 	windowVRect += VPoint (0, 100);
	//------------------------------------------------------------------------------------

	VRect operator+(const VPoint& pt) const;
	
	VRect operator-(const VPoint& pt) const;
	
	VRect& operator+=(const VPoint& pt);
	
	VRect& operator-=(const VPoint& pt);
	
	VRect operator-() const;
		// Negation
	
	
	//------------------------------------------------------------------------------------
	// Miscellaneous methods
	//------------------------------------------------------------------------------------

	Boolean operator==(const VRect& rt) const;
		// Returns true if all four coordinates are equal
	
	Boolean operator!=(const VRect& rt) const;
		// Returns true if at least one of the four coordinates are not equal.

	VRect operator &(const VRect& rt) const;
		// Returns the intersection of 'this' and 'rt'.
	
	VRect operator |(const VRect& rt) const;
		// Returns the union of 'this' and 'rt'. Actually not the true mathematical union
        // but the smallest rectangle that bounds both 'this' and 'rt'.

	VRect& Inset(const VPoint& delta);
		// Inset a VRect by VPoint.
	
	Boolean Valid() const;
		// Returns true if a valid rectangle (left < right and top < bottom).
		
	Boolean Validate();
		// Returns true if a valid rectangle (left < right and top < bottom). If not a
        // valid rectangle then return false and set all coordinates to 0.
	
	Boolean Empty() const;
		// Return true if the VRect is empty (right - left = 0 and bottom - top = 0)
	
	
	VCoordinate Length(VHSelect sel) const;
		// Returns the length of a VRect in a given dimension.
		
	VPoint Size() const;
		// Returns the size of a VRect in a VPoint.
	
	Boolean Contains(const VPoint& pt) const;
		// Returns true if 'pt' is contained in the rectangle 'this'
		
	Boolean Contains(const VRect& rt) const;
		// Return true if 'rt' is contained in the rectangle 'this'

private:

	VCoordinate Min(const VCoordinate a,
			  const VCoordinate b) const;
			  
	VCoordinate Max(const VCoordinate a,
			  const VCoordinate b) const;
};


//----------------------------------------------------------------------------------------
// Inline definitions for methods in VRect
//----------------------------------------------------------------------------------------

inline VRect::VRect() { }

inline VRect::VRect(VCoordinate iTop, VCoordinate iLeft,
					VCoordinate iBottom, VCoordinate iRight)
{
	top = iTop;
	left = iLeft;
	bottom = iBottom;
	right = iRight;
}

inline VRect::VRect(const VRect& rect)
{
	top = rect.top;
	left = rect.left;
	bottom = rect.bottom;
	right = rect.right;
}

inline VRect::VRect(const VPoint& topLeftPt, const VPoint& botRightPt)
{
	top = topLeftPt.v;
	left = topLeftPt.h;
	bottom = botRightPt.v;
	right = botRightPt.h;
}

inline VRect::VRect(const SimpleVRect& rect)
{
	top = rect.top;
	left = rect.left;
	bottom = rect.bottom;
	right = rect.right;
}

inline VRect VRect::Copy() const
{
	VRect rt;
	
	rt.top = top;
	rt.left = left;
	rt.bottom = bottom;
	rt.right = right;
	
	return rt;
}


//----------------------------------------------------------------------------------------
// Global function declarations. From C++ this functionality is accessible from member
// functions.
//----------------------------------------------------------------------------------------

pascal void PtToVPt(const Point thePt, VPoint& theVPt);

pascal Point VPtToPt(const VPoint& theVPt);

pascal void RectToVRect(const Rect& theRect, VRect& theVRect);

pascal void VRectToRect(const VRect& theVRect, Rect& theRect);

pascal void AddVPt(const VPoint& srcVPt, VPoint& dstVPt);

pascal void SubVPt(const VPoint& srcVPt, VPoint& dstVPt);

pascal void SetVPt(VPoint& vPt, VCoordinate h, VCoordinate v);

pascal Boolean EqualVPt(const VPoint& aVPt, const VPoint& bVPt);

pascal void SetVRect(VRect& vRt,
					 VCoordinate left, VCoordinate top,
					 VCoordinate right, VCoordinate bottom);

pascal void OffsetVRect(VRect& vRt, VCoordinate dh, VCoordinate dv);

pascal void InsetVRect(VRect& vRt, VCoordinate dh, VCoordinate dv);

pascal void Pt2VRect(const VPoint& theTopLeft, const VPoint& theBotRight, VRect& vRt);

pascal Boolean PtInVRect(const VPoint& vPt, const VRect& vRt);

pascal Boolean EmptyVRect(const VRect& vRt);

pascal Boolean EqualVRect(const VRect& aVRt, const VRect& bVRt);

pascal VCoordinate LengthVRect(const VRect& vRt, VHSelect whichDim);

pascal void PinVRect(const VRect& vRt, VPoint& vPt);

pascal Boolean SectVRect(const VRect& src1, const VRect& src2, VRect& dst);

pascal void UnionVRect(const VRect& src1, const VRect& src2, VRect& dst);

#endif
