//  UIterator.h
//  Copyright  1984-1991 by Apple Computer Inc. All rights reserved.

#ifndef __UITERATOR__
#define __UITERATOR__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __UMACAPPGLOBALS__
#include <UMacAppGlobals.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TObject;
class TDynamicArray;
class TSortedList;
class THandleList;
class TSortedLongintList;


//----------------------------------------------------------------------------------------
// CIterator
//----------------------------------------------------------------------------------------

class CIterator
{
public:
	virtual Boolean More(void);
		// Returns TRUE if there are more elements to iterate over

	virtual void Reset(void);
		// Resets the iterator to begin again

protected:
	virtual void Advance(void);
		// Advances the iteration
};


//----------------------------------------------------------------------------------------
// CArrayIterator
//----------------------------------------------------------------------------------------

class CArrayIterator : public CIterator
{
public:
	// These items are currently public for searching

	ArrayIndex fCurrentIndex;					// current index of this iteration

	ArrayIndex fLowBound;						// lower bound of iteration in progress

	ArrayIndex fHighBound;						// upper bound of iteration in progress
												// through the list
protected:
	TDynamicArray * fDynamicArray;				// the associated dynamic array

	Boolean fIterateForward;					// if iteration is forward or backward

	FailInfo	fFailInfo;						// the failure record that will point to
                                                // the failure handling method
private:
	CArrayIterator* fPreviousLink;				// link to previous iterator

	CArrayIterator* fNextLink;					// link to next iterator
	
public:
	CArrayIterator(TDynamicArray* itsDynamicArray,
				   ArrayIndex itsLowBound, ArrayIndex itsHighBound,
				Boolean itsForward);

	CArrayIterator(TDynamicArray* itsDynamicArray, Boolean itsForward);

	CArrayIterator(TDynamicArray* itsDynamicArray);

	~CArrayIterator();
	
	void IArrayIterator(TDynamicArray* itsDynamicArray,
						ArrayIndex itsLowBound, ArrayIndex itsHighBound,
						Boolean itsForward);
		// Called by the constructors to do common initialization

	void IArrayIterator();
		// Called by the constructors to do common initialization if the dynamicarray is
        // NULL

	virtual Boolean More(void);		// override
		// Returns TRUE if there are more elements to iterate over

	virtual void Reset(void);		// override
		// Resets the iterator to begin again

	ArrayIndex CurrentIndex(void);
		// returns the current index
		
	ArrayIndex FirstIndex(void);
		// Resets the iterator to begin again and returns the value of the first index in the iteration

	ArrayIndex NextIndex(void);
		// Advances the iteration and then returns the index
	
	void DeleteElementAt(ArrayIndex theIndex, ArrayIndex theCount);
		// bends the index in the Iteration node to account for the deleted elements

	void InsertElementBefore(ArrayIndex theIndex, ArrayIndex theCount);
		// bends the index to account for the inserted elements 

protected:
	virtual void Advance(void);		// override
		// Advances the iteration

	pascal void HandleFailure(OSErr error, long message);
		// Handle failures while this iterator is in scope

private:
	CArrayIterator* AppendToList(CArrayIterator* toList);
		// Add a new node to the list

	CArrayIterator* RemoveFromList();
		// Remove a node from the list
};


//----------------------------------------------------------------------------------------
// CObjectIterator
//----------------------------------------------------------------------------------------

class CObjectIterator : public CArrayIterator
{
public:
	CObjectIterator(TSortedList* itsList,
					ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					Boolean itsForward);

	CObjectIterator(TSortedList* itsList, Boolean itsForward);

	CObjectIterator(TSortedList* itsList);
	
	TObject* CurrentObject(void);
		// returns the current object
		
	TObject* FirstObject(void);
		// Resets the iterator to begin again and returns the value of the first object in the iteration

	TObject* NextObject(void);
		// advances the iteration and then returns the object
};


//----------------------------------------------------------------------------------------
// CHandleIterator
//----------------------------------------------------------------------------------------

class CHandleIterator : public CArrayIterator
{
public:
	CHandleIterator(THandleList* itsHandleList,
					ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					Boolean itsForward);

	CHandleIterator(THandleList* itsHandleList, Boolean itsForward);

	CHandleIterator(THandleList* itsHandleList);
	
	Handle CurrentHandle(void);
		// returns the current handle
		
	Handle FirstHandle(void);
		// Resets the iterator to begin again and returns the value of the first handle in the iteration

	Handle NextHandle(void);
		// advances the iteration and then returns the handle
};


//----------------------------------------------------------------------------------------
// CLongintIterator
//----------------------------------------------------------------------------------------

class CLongintIterator : public CArrayIterator
{
public:
	CLongintIterator(TSortedLongintList* itsSortedLongintList,
					 ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					 Boolean itsForward);

	CLongintIterator(TSortedLongintList* itsSortedLongintList, Boolean itsForward);

	CLongintIterator(TSortedLongintList* itsSortedLongintList);
	
	long CurrentLong(void);
		// return the current long value
		
	long FirstLong(void);
		// Resets the iterator to begin again and returns the value of the first long in the iteration

	long NextLong(void);
	// advances the iteration and then returns the long value
};


//----------------------------------------------------------------------------------------
// inline implementations
//----------------------------------------------------------------------------------------

inline ArrayIndex CArrayIterator::CurrentIndex(void)
{
	return fCurrentIndex;
}

inline ArrayIndex CArrayIterator::FirstIndex(void)
{
	this->Reset();
	return fCurrentIndex;
}

inline ArrayIndex CArrayIterator::NextIndex(void)
{
	this->Advance();
	return fCurrentIndex;
}

#endif
