//  UList.h
//  Copyright  1984-1991 by Apple Computer Inc. All rights reserved.

#ifndef __ULIST__
#define __ULIST__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UOBJECT__
#include "UObject.h"
#endif

#ifndef __UMACAPPGLOBALS__
#include "UMacAppGlobals.h"
#endif


//----------------------------------------------------------------------------------------
// Unit UList defines object types TList, a simple dynamic array whose elements are
// references to TObjects, and TSortedList, which maintains the object references in a
// sequence defined by overriding TSortedList.Compare.
//
// Peter Gaston's scheme for chunky memory allocation has been used by permission. Thanks
// Peter!.
//
// TList: TList is an object which provides a very nice dynamic array of objects. It is
// implemented by appending the dynamic array onto the end of the object.Growing and
// shrinking the array is done by growing and shrinking the handle which holds the object.
//
// TSortedList: TSortedList maintains the object references in a sequence defined by
// overriding TSortedList.Compare
//
//  Some Specific uses that TList is especially good for.
//  LIFO Stack: A push/pop operation is provided.
//  FIFO Stack: (Queue) Use InsertLast for Queue and First for Dequeue.
// Pre-allocation of your list: If you know the size that your list will eventually reach,
// then you can pre-set it to the proper size.
//
//  Delete by Index:
// If you keep track of items by index number, then you can avoid a linear search when you
// want to delete it. Remember though, it is a dynamic list and any insertion/ deletion
// activity can cause the index to point at a different element than you expect.
//
//  Caveats:
// A _small_ amount of memory is always unused with this method.An active list will
// generally waste one-half of the chunk size, or 4*2=8 bytes/TList.
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class CArrayIterator;


const short kAllocationIncrement = 6;			// Initial Allocation increment. Six
                                                // elements of slop shouldn't break
                                                // anybody and provides a _nice_ cushion
                                                // from the memory manager.


//----------------------------------------------------------------------------------------
// Constants for TDynamicArray.CompareElements and TSortedxxx.Compare
//----------------------------------------------------------------------------------------

const short kItem1LessThanItem2 = -1;
const short kItem1EqualItem2 = 0;
const short kItem1GreaterThanItem2 = 1;


//----------------------------------------------------------------------------------------
// Constants for TSortedList.Search. The criteria is considered to be item 2
//----------------------------------------------------------------------------------------

const short kItemGreaterThanCriteria = kItem1LessThanItem2;
const short kItemEqualCriteria = kItem1EqualItem2;
const short kItemLessThanCriteria = kItem1GreaterThanItem2;

typedef short CompareResult;
	// Negative, zero, and positive results are all meaningful even though we have the
    // nice constants above.


//----------------------------------------------------------------------------------------
// procedure typedefs for parameters
//----------------------------------------------------------------------------------------

typedef pascal Boolean (*TestIndexType)(ArrayIndex elementIndex, void* staticLink);
										
typedef pascal CompareResult (*CompareIndexType)(ArrayIndex anItem, void* staticLink);

typedef pascal Boolean (*TestElementType)(ArrayIndex elementIndex, void* staticLink);

typedef pascal Boolean (*TestObjectType)(TObject* item, void* staticLink);

typedef pascal void(* DoToObjectType)(TObject* item, void* staticLink);

typedef pascal CompareResult(* CompareObjectType)(TObject* anItem, void* staticLink);

typedef pascal CompareResult(* CompareObjectsType)(TObject* item1,
												   TObject* item2,
												   void* staticLink);
												   
typedef pascal void (*DoToHandleType)(Handle item, void* staticLink);

typedef pascal Boolean (*TestHandleType)(Handle item, void* staticLink);

typedef pascal CompareResult (*CompareHandleType)(Handle anItem, void* staticLink);

typedef pascal Boolean (*TestItemType)(Handle item, void* staticLink);

typedef pascal Boolean (*TestLongType)(long item, void* staticLink);

typedef pascal void (*DoToLongType)(long item, void* staticLink);

typedef pascal CompareResult (*CompareLongType)(long anItem, void* staticLink);


//----------------------------------------------------------------------------------------
// TDynamicArray
//----------------------------------------------------------------------------------------

class TDynamicArray : public TObject
{
public:
	CArrayIterator* fIteratorPtr;				// pointer to the Iterators in use

	ArrayIndex fSize;							// number of elements ACTUALLY in the
                                                // array, from 0 to the limit of memory

	short fElementSize;							// Size in bytes of an element. MUST be a
                                                // power of 2 ie. 1, 2, 4, 8, 16, etc.

	short fElementSizeShift;					// the power of 2 for the element size.
                                                // Will be used to avoid DIV and MUL

	ArrayIndex fAllocationIncrement;			// Number of elements by which to increase
                                                // of decrease the allocated size of the
                                                // array when it needs to grow or shrink.
                                                // Thus reducing memory manager
                                                // aggravation.

	ArrayIndex fAllocatedSize;					// Number of elements for which storage is
                                                // ALLOCATED

	Boolean fFreeRequested;						// TRUE if the Free method was called but
                                                // couldn't be honored because enumeration
                                                // was in process. Checked at end of
                                                // enumeration and Free is called if true

	Size fClassSize;							// Used with ComputeAddress to create a
                                                // pointer to an element.This breaks
                                                // encapsulation of GetDynamicArea but, is
                                                // being done here for performance reasons
                                                // (small)
		
												
	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TDynamicArray into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IDynamicArray(ArrayIndex initialSize,
									  short elementSize);
		// Initialize a new array with initialSize elements, Always call it once before
        // calling any other method.Never call it twice.


	//------------------------------------------------------------------------------------
	// Array manipulation primitives
	//------------------------------------------------------------------------------------

	virtual pascal ArrayIndex GetSize();
		// Returns the ACTUAL number of elements in the array.

	virtual pascal void SetArraySize(ArrayIndex theSize);
		// Sets the array allocation to handle up to theSize elements

	virtual pascal ArrayIndex EachElementDoTil(TestIndexType TestElement,
											   void* staticLink,
											   Boolean IterateForward);
		// Call TestIndex once for each element of the array, in order, until TestIndex
        // returns TRUE. Return the element that satisfied the test.If none satisfied the
        // test, return kEmptyIndex.

	virtual pascal ArrayIndex EachElementInRangeDoTil(ArrayIndex lowBound,
													  ArrayIndex highBound,
													  TestElementType TestElement,
													  void* staticLink,
													  Boolean IterateForward);
		// The basic array iterator. Call TestIndex once for each element over a subrange
        // the array, in order, until TestIndex returns TRUE. Return the element that
        // satisfied the test.If none satisfied the test, return kEmptyIndex.

	virtual pascal void Free();
		// If enumeration of the array is in process, delete all the array elements, mark
        // the fFreeRequested flag for testing at completion of the enumeration and
        // return.Otherwise really free the array.Gee, wouldn't automatic storage
        // management be great!

	virtual pascal void InsertElementsBefore(ArrayIndex index,
											 void* ElementPtr,
											 ArrayIndex count);
		// Insert Elements before the indicated Element. The index of the new element will
        // be index.If index = 1 this inserts at the start of the array. If index = fSize
        // + 1 this inserts at the end of the array. Signals Failure if unable to change
        // the size of the array. !!! NOTE MULTIPLE ( >1 ) element moves for non-power of
        // 2 element sizes are NOT yet supported!

	virtual pascal void ReplaceElementsAt(ArrayIndex index,
										  void* ElementPtr,
										  ArrayIndex count);
		// Replaces the Elements at the indicated index. !!! NOTE MULTIPLE ( >1 ) element
        // moves for non-power of 2 element sizes are NOT yet supported!

	virtual pascal void DeleteElementsAt(ArrayIndex index,
										 ArrayIndex count);
		// Deletes the Element at the indicated index.Compresses the array !!! NOTE
        // MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT yet
        // supported!

	virtual pascal void GetElementsAt(ArrayIndex index,
									  void* ElementPtr,
									  ArrayIndex count);
		// copies count elements to the location specified by ptr. !!! NOTE MULTIPLE ( >1
        // ) element moves for non-power of 2 element sizes are NOT yet supported!

	virtual pascal Ptr ComputeAddress(ArrayIndex index);
		// Returns a pointer to of the index-th element in the array. PLEASE NOTE: The
        // return value is a direct heap pointer and should be used with care as the heap
        // can compact across calls that move memory; thus invalidating the pointer.


	//------------------------------------------------------------------------------------
	// Misc. functions
	//------------------------------------------------------------------------------------

	virtual pascal Boolean IsEmpty();
		// Test if this array is empty or not.

	virtual pascal void Merge(TDynamicArray* aDynamicArray);
		// merges aDynamicArray with itself. leaves aDynamicArray unchanged. !!! NOTE
        // MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT yet
        // supported!


	//------------------------------------------------------------------------------------
	// Debugging Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

	virtual pascal void DynamicFields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // dynamic area.
};


//----------------------------------------------------------------------------------------
// TSortedDynamicArray: A TDynamicArray with ordering.
//----------------------------------------------------------------------------------------

class TSortedDynamicArray : public TDynamicArray
{
public:
	virtual pascal void ISortedDynamicArray(ArrayIndex initialSize,short elementSize);
		// Initialize a new array with initialSize elements, Always call it once before
        // calling any other method.Never call it twice.

	virtual pascal CompareResult CompareElements(void* Element1, void* Element2);
		// Orders the elements of the array

	virtual pascal void InsertElementInOrder(void* ElementPtr);
		// Insert an element in its correct position according to the ordering defined by
        // the Compare method

	virtual pascal Boolean DoSearchElement(CompareIndexType TestItem,
										   void* staticLink,
										   ArrayIndex& index);
		// !!! Adapted from TSortedList. Performs a binary search for an element for which
        // TestItem returns true. "index" returns the position of a found item, or where
        // it should be inserted if not found

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TSortedList: A sorted dynamicArray of TObjects. Items are sorted based on the results
// of Compare which has to be overridden to provide a comparison mechanism other than
// identity. It IS permissible for descendants to add more named fields
//----------------------------------------------------------------------------------------

class TSortedList : public TSortedDynamicArray
{
public:
	ObjClassID fObjClassID;						// if <> kNilClass then the Class ID of
                                                // the elements of the list


	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created list into a known state. Subclasses should override it and
        // call INHERITED. Typically used to put the object into a state where it can be
        // safely FREED.

	virtual pascal void ISortedList();
		// Initialize a sorted list

	virtual pascal void FreeList();
		// Frees each object in the list, then frees the list.


	//------------------------------------------------------------------------------------
	// Utility Methods
	//------------------------------------------------------------------------------------

	virtual pascal TObject* At(ArrayIndex index);
		// Return the index'th element of the list. It is typical for the caller to coerce
        // the result into a descendant of TObject. All lists are indexed from 1. Range
        // check only if the compile-flag qRangeCheck is TRUE. The static-array equivalent
        // of: object := objList.At(index); is: object := objArray[index];

	virtual pascal TObject* First();
		// Return the first element of the list. It is typical for the caller to coerce
        // the result. Returns NIL if the size is <= 0.

	virtual pascal TObject* Last();
		// Return the last element of the list. It is typical for the caller to coerce the
        // result. Returns NIL if the size is <= 0.


	//------------------------------------------------------------------------------------
	// Iterator Methods
	//------------------------------------------------------------------------------------

	virtual pascal TObject* IterateTil(TestObjectType TestItem,
									   void* staticLink,
									   Boolean IterateForward,
									   ArrayIndex& itsIndex);
		// The basic list iterator. Call TestItem once for each element of the list, in
        // order, until TestItem returns TRUE. Return the element that satisfied the
        // test.If none satisfied the test, return NIL. The actual parameter is typically
        // a procedure whose argument is a descendant of TObject. It is typical for the
        // caller to coerce the result into that descendant of TObject. If TestItem calls
        // InsertLast, the newly added element will NOT be enumerated. If TestItem calls
        // AtPut, InsertBefore, InsertFirst, or DeleteAll, misbehavior will ensue.The
        // static-array equivalent of:
		// object := objList.FirstThat(Func)
		// is:
		//  	object := NIL;
		//  	FOR index := 1 TO fSize DO IF Func(objArray[index]) THEN
		//  	BEGIN
		//  	object := objArray[index];
		//  	LEAVE;
		//  	END;


	virtual pascal void Each(DoToObjectType DoToItem, void* staticLink);
		// Call DoToItem once for each element of the list, in order. The actual parameter
        // is typically a procedure whose argument is a descendant of TObject. If DoToItem
        // calls InsertLast, the newly added element will NOT be enumerated. If TestItem
        // calls AtPut, InsertBefore, InsertFirst, Delete, or DeleteAll, misbehavior will
        // ensue. If DoToItem calls Delete, the item is replaced by the constant
        // kDeletedElement; fSize is decremented and fDeletions is incremented. At the end
        // of all calls to Each, RemoveDeletions will be called. The static-array
        // equivalent of: objList.Each(Proc) is:
		//  	FOR index := 1 TO fSize DO
		// 		Proc(objArray[index]);


	virtual pascal TObject* FirstThat(TestObjectType TestItem, void* staticLink);
		// Call TestItem once for each element of the list, in order, until TestItem
        // returns TRUE. Return the element that satisfied the test. If none satisfied the
        // test, return NIL. The actual parameter is typically a procedure whose argument
        // is a descendant of TObject. It is typical for the caller to coerce the result
        // into that descendant of TObject. If TestItem calls InsertLast, the newly added
        // element will NOT be enumerated. If TestItem calls AtPut, InsertBefore,
        // InsertFirst, Delete, or DeleteAll, misbehavior will ensue.

	virtual pascal TObject* LastThat(TestObjectType TestItem, void* staticLink);
		// Call TestItem once for each element of the list, starting with the last item in
        // the list and working toward the first item, until TestItem returns TRUE. Return
        // the element that satisfied the test. If none satisfied the test, return NIL.
        // The actual parameter is typically a procedure whose argument is a descendant of
        // TObject. It is typical for the caller to coerce the result into that descendant
        // of TObject. If TestItem calls InsertLast, the newly added element will NOT be
        // enumerated. If TestItem calls AtPut, InsertBefore, InsertFirst, Delete, or
        // DeleteAll, misbehavior will ensue.

	virtual pascal TObject* DoSearch(CompareObjectType TestItem,
									 void* staticLink,
									 ArrayIndex& index);
		// Searches the sorted list until TestItem returns zero or there are no more items
        // to search. A binary search is used.

	virtual pascal ArrayIndex GetIdentityItemNo(TObject* item);
		// Return the index of the IDENTITY item in the list, or zero if the item is not
        // in the list.

	virtual pascal ArrayIndex GetEqualItemNo(TObject* item);
		// Return the index of any item that is considered to be equal to the parameter
        // 'item'. Two items are considered equal if comparing them with the Compare
        // method returns zero. You may wish to use the constants
        // kItemGreaterThanCriteria, kItemEqualCriteria, kItemLessThanCriteria.

	virtual pascal void Insert(TObject* item);
		// Inserts the given item into the list in sorted order, using the Compare method
        // to determine the item's location relative to other items in the list.

	virtual pascal CompareResult Compare(TObject* item1, TObject* item2);
		// This method compares two items in the list. A negative result indicates that
        // item1 < item2. A result of zero indicates that item1 = item2. A positive result
        // indicates that item1 > item2. You may wish to use the constants
        // kItem1LessThanItem2, kItem1EqualItem2, kItem1GreaterThanItem2. By default just
        // compare the ordinal value of the items. Subclasses that want to should override
        // this method to do any other kind of comparison (comparing instance variables,
        // for instance (get it?)).

	virtual pascal CompareResult CompareElements(void* Element1, void* Element2);
		// Forward into the Compare function.

	virtual pascal TObject* Search(CompareObjectType TestItem, void* staticLink);
		// This method searches the list for an item that causes your supplied TestItem
        // function to return kItem1EqualItem2 (0). This is useful for cases in which you
        // are not comparing objects in the list with another object, as does Compare. For
        // example, suppose each object has an fTitle field and the objects are inserted
        // into the list in order of fTitle. You can use the search method to look for an
        // object whose fTitle is equal to any string. A negative TestItem result
        // indicates that your search criteria < anItem, a result of zero indicates the
        // item has been found and Search returns that item. A positive TestItem result
        // indicates that your search criteria > your anItem.

	virtual pascal void Sort();
		// Sorts the list usingthe Compare and SortBy methods.

	virtual pascal void SortBy(CompareObjectsType CompareItems, void* staticLink);
		// Sorts the list using the supplied CompareItems function.Uses Shell sort (see
        // Sedgewick, "Algorithms", pp. 97-9)


	//------------------------------------------------------------------------------------
	// Item Deletion Methods
	//------------------------------------------------------------------------------------

	virtual pascal void AtDelete(ArrayIndex index);
		// Deletes via an index (rather than having to search for an item)

	virtual pascal void Delete(TObject* item);
		// Delete the first reference to item from the list, but do not free item. If item
        // does not occur, do nothing. If item does occur, reduce fSize by 1. !!! NOTE:
        // This name conflicts with the Pascal builtin: DELETE and we will be changing
        // it's name, but changing the name at the _last minute_ isn't a good idea. If you
        // need to use the builtin DELETE in a TSortedList subclass, then you will have to
        // create a wrapper procedure that forwards to it, for now. Sorry the Management.

	virtual pascal void DeleteAll();
		// Delete every element from the list, but do not free any objects. Leave fSize at
        // 0.

	virtual pascal void FreeAll();
		// Frees each element in the list.Leave fSize at 0.


	//------------------------------------------------------------------------------------
	// Debugging Methods
	//------------------------------------------------------------------------------------

	virtual pascal void SetEltType(const MAName& toClass);
		// Call this once and pass the className of objects you intend to insert into the
        // list. The insertion methods will do a coercion to the type you specify, to
        // ensure that the list contains only elements of the right type.

	virtual pascal void SetEltTypeID(ObjClassID toClassID);
		// Call this once and pass the ObjClassID of objects you intend to insert into the
        // list. The insertion methods will do a coercion to the type you specify, to
        // ensure that the list contains only elements of the right type.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

	virtual pascal void DynamicFields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // dynamic area.

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Used by the Inspector to display the name of this class.
		
private:
	virtual pascal void QuickSort(ArrayIndex beginIndex,
								  ArrayIndex endIndex,
								  CompareObjectsType CompareItems,
								  void* staticLink);
};


//----------------------------------------------------------------------------------------
// TList: A dynamic list of TObjects.
//----------------------------------------------------------------------------------------

class TList : public TSortedList
{
public:
	virtual pascal void IList();
		// Initialize a new list with no elements, i.e., fSize = 0. Always call it once
        // before calling any other method.Never call it twice.

	virtual pascal Boolean DoSearchElement(CompareIndexType TestItem,
										   void* staticLink,
										   ArrayIndex& index);
		// Overridden to do a linear search

	virtual pascal void InsertElementInOrder(void* ElementPtr);
		// Overridden to do arrival sequence insertion

	virtual pascal void AtPut(ArrayIndex index, TObject* newItem);
		// Replace the index'th item of the list. Does not free the old item. If you do
        // this from within an Each the results may be unpredictable.

	virtual pascal void InsertBefore(ArrayIndex index, TObject* item);
		// Insert a reference to an item before the indicated item. The index of the new
        // element will be index. If index = 1 this inserts at the start of the list. If
        // index = fSize + 1 this inserts at the end of the list. Signals Failure if
        // unable to increase the size of the list.

	virtual pascal void InsertFirst(TObject* item);
		// Insert a reference to item at the front of the list. If the compile-flag qDebug
        // is TRUE & SetEltType was called, verify item's type. Increase fSize by 1. The
        // index of the new element will be 1. Signals Failure if unable to increase the
        // size of the list.

	virtual pascal void InsertLast(TObject* item);
		// Insert a reference to item at the back of the list. If the compile-flag qDebug
        // is TRUE & SetEltType was called, verify item's type. Increase fSize by 1. The
        // index of the new element will be fSize. Signals Failure if unable to increase
        // the size of the list.

	virtual pascal void Push(TObject* item);
		// LIFO stack push.(same as insertLast)

	virtual pascal TObject* Pop();
		// LIFO stack pop.

	virtual pascal void Queue(TObject* item);
		// FIFO queue addition.(same as insertLast)

	virtual pascal TObject* Dequeue();
		// FIFO queue relief

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// THandleList: A dynamic list of Handles. It IS permissible for descendants to add more
// named fields
//----------------------------------------------------------------------------------------

class THandleList : public TSortedDynamicArray
{
public:
	virtual pascal void IHandleList();
		// Initialize a new list with no elements, i.e., fSize = 0

	virtual pascal Handle At(ArrayIndex index);
		// Return the index'th element of the list.

	virtual pascal void AtDelete(ArrayIndex index);
		// Deletes via an index.

	virtual pascal void Delete(Handle item);

	virtual pascal void DeleteAll();

	virtual pascal void Each(DoToHandleType DoToItem, void* staticLink);

	virtual pascal Handle First();
		// Return the first element of the list. If the compile-flag qDebug is TRUE &
        // SetEltType was called, verify item's type.

	virtual pascal Handle FirstThat(TestHandleType TestItem, void* staticLink);

	virtual pascal ArrayIndex GetIdentityItemNo(Handle item);
		// Find the first reference to the IDENTITY item in the list. If item does not
        // occur, return 0.

	virtual pascal ArrayIndex GetEqualItemNo(Handle item);
		// Find the first reference to item in the list. If item does not occur, return 0.
        //

	virtual pascal void Insert(Handle item);
		// Inserts the given item into the list in sorted order, using the Compare method
        // to determine the item's location relative to other items in the list.

	virtual pascal Handle DoSearch(CompareHandleType TestItem,
								   void* staticLink,
								   ArrayIndex& index);

	virtual pascal CompareResult Compare(Handle item1, Handle item2);
		// By default just compare the ordinal value of the items.Subclasses that want to
        // use the items as pointers or other types should override if the comparison is
        // based on the pointed at value

	virtual pascal CompareResult CompareElements(void* Element1, void* Element2);
		// Forward into the Compare function.

	virtual pascal Handle IterateTil(TestItemType TestItem,
									 void* staticLink,
									 Boolean IterateForward,
									 ArrayIndex& itsIndex);
		// Call TestItem once for each element of the list, in order.

	virtual pascal Handle Last();
		// Return the last element of the list. If the compile-flag qDebug is TRUE &
        // SetEltType was called, verify item's type.

	virtual pascal Handle LastThat(TestHandleType TestItem, void* staticLink);
		// Call DoToItem once for each element of the list, in order.

	virtual pascal void Fields(TObject* obj);

	virtual pascal void DynamicFields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TSortedLongintList: A dynamic list of Longints. It IS permissible for descendants to
// add more named fields
//----------------------------------------------------------------------------------------

class TSortedLongintList : public TSortedDynamicArray
{
public:
	virtual pascal void ISortedLongintList();
		// Initialize a new list with no elements, i.e., fSize = 0

	virtual pascal void Insert(long item);

	virtual pascal long At(ArrayIndex index);
		// Return the index'th element of the list.

	virtual pascal void AtDelete(ArrayIndex index);
		// Deletes via an index.

	virtual pascal void Delete(long item);

	virtual pascal void DeleteAll();

	virtual pascal long First();
		// Return the first element of the list. If the compile-flag qDebug is TRUE &
        // SetEltType was called, verify item's type.

	virtual pascal long Last();
		// Return the last element of the list. If the compile-flag qDebug is TRUE &
        // SetEltType was called, verify item's type.

	virtual pascal long LastThat(TestLongType TestItem, void* staticLink);
		// Call DoToItem once for each element of the list, in order.

	virtual pascal long IterateTil(TestLongType TestItem,
								   void* staticLink,
								   Boolean IterateForward,
								   ArrayIndex& itsIndex);
		// Call TestItem once for each element of the list, in order.

	virtual pascal void Each(DoToLongType DoToItem, void* staticLink);	

	virtual pascal long FirstThat(TestLongType TestItem, void* staticLink);
		// Call DoToItem once for each element of the list, in order.

	virtual pascal ArrayIndex GetIdentityItemNo(long item);
		// Find the first reference to the IDENTITY item in the list. If item does not
        // occur, return 0.

	virtual pascal ArrayIndex GetEqualItemNo(long item);
		// Find the first reference to item in the list. If item does not occur, return 0.

	virtual pascal CompareResult Compare(long item1, long item2);
		// By default just compare the ordinal value of the items.Subclasses that want to
        // use the items as pointers or other types should override if the comparison is
        // based on the pointed at value

	virtual pascal CompareResult CompareElements(void* Element1, void* Element2);
		// Forward into the Compare function.

	virtual pascal long DoSearch(CompareLongType TestItem,
								 void* staticLink,
								 ArrayIndex& index);
		// DON'T use EXIT to get out of this routine from your TestItem function or you
        // will be really sad!(our debugger will check for you) That's why you can return
        // TRUE to stop enumerating. Signaling Failure is OK too.

	virtual pascal long Search(CompareLongType TestItem, void* staticLink);

	virtual pascal void DynamicFields(TObject* obj);

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TLongintList: A dynamic list of Longints. It IS permissible for descendants to add more
// named fields
//----------------------------------------------------------------------------------------

class TLongintList : public TSortedLongintList
{
public:
	virtual pascal void ILongintList();
		// Initialize a new list with no elements, i.e., fSize = 0

	virtual pascal Boolean DoSearchElement(CompareIndexType TestItem,
										   void* staticLink,
										   ArrayIndex& index); 
		// Overridden to do a linear search

	virtual pascal void InsertElementInOrder(void* ElementPtr); 
		// Overridden to do arrival sequence insertion

	virtual pascal void AtPut(ArrayIndex index, long newItem);
		// Replace the index'th element of the list. Range check only if the compile-flag
        // qRangeCheck is TRUE.

	virtual pascal void InsertBefore(ArrayIndex index, long item);
		// Insert a reference to item at the indicated index. If the compile-flag qDebug
        // is TRUE & SetEltType was called, verify item's type.

	virtual pascal void InsertFirst(long item);
		// Insert a reference to item at the front of the list. If the compile-flag qDebug
        // is TRUE & SetEltType was called, verify item's type.

	virtual pascal void InsertLast(long item);
		// Insert a reference to item at the back of the list. If qDebug is TRUE &
        // SetEltType was called, check that item's type is correct.

	virtual pascal void Fields(TObject* obj);

	virtual pascal void Push(long item);
		// LIFO stack push.

	virtual pascal long Pop();
		// LIFO stack pop
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern pascal TList* NewList();
	// A convenience function. Create a TList, initialize it, and return a reference to
    // it. Signals Failure if it cannot allocate the object.

extern pascal TSortedList* NewSortedList();
	// A convenience function. Create a TSortedList, initialize it, and return a reference
    // to it. Signals Failure if it cannot allocate the object.

extern pascal TList* NewAllocatedList(ArrayIndex iSize);
	// A convenience function. The same as NewList, but the initial allocation size can be
    // set.

extern pascal TSortedList* FreeListIfObject(TSortedList* list);
	// A convenience function. if list is non-nil then the same as list.FreeList. Returns
    // NIL for convenient assignment back to the reference passed in.


#endif


