// UMenuView.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UMENUVIEW__
#define __UMENUVIEW__

//----------------------------------------------------------------------------------------
// Theory of Operation
//
// This is Larry Rosenstein's excellent unit: UMenu, that supports an object-oriented way
// of defining custom menus. Instead of writing a menu defproc, you simply create a
// subtype of TMenuView and override certain methods.Not only is this easier to program,
// but this unit takes care of some of the housekeeping that a menu defproc would normally
// have to do.
//
// Changes since MacApp 2.0: You can now add subviews to your subclass of TMenuView, and
// these subviews can have their own trackers. This is accomplished in HandleChooseMessage
// by creating a dummy mouse down event and passing it to the TMenuView via the
// HandleMouseDown method which then asks its subviews if any of them are interested in
// the mousedown (eg. to create a tracker). The tracker should be posted via PostCommand,
// and the TMenuView override of PostCommand will check to see if the command is a
// tracker, in which case it will track it immediately.
//----------------------------------------------------------------------------------------


#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __GEOMETRY__
#include <Geometry.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kNoMenuItem = 0;					// used to indicate that no item is
                                                // selected


//----------------------------------------------------------------------------------------
// MenuColors: a composite of the MCEntries
//----------------------------------------------------------------------------------------

struct MenuColors
{
	RGBColor itemColor;							// the foreground color to draw the item
                                                // with (the color of the title if title
                                                // or menubar)

	RGBColor backgroundColor;					// the background color to draw the item
                                                // with (the color of the menubar if title
                                                // or menubar)

	RGBColor markColor;							// the foreground color to draw the mark
                                                // with (checkmarks, etc.)

	RGBColor commandColor;						// the foreground color to draw the
                                                // command key equivalent with (command-v
                                                // for paste, etc).
};

typedef MenuColors *MenuColorsPtr, **MenuColorsHandle;


//----------------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------------

class TEvent;
class TDocument;
class TToolboxEvent;


//----------------------------------------------------------------------------------------
// TMenuView
//----------------------------------------------------------------------------------------

class TMenuView : public TView
{
public:
	MenuHandle fMenuHandle;						// handle to menu itself

	long fFlashInterval;						// Time (in ticks) between flashes of item
                                                // highlighting; default is -1 which means
                                                // only change highlighting w when item
                                                // changes

	long fNextFlash;							// Used internally

	Boolean fHighlighted;						// True if an item is highlighted

	Rect fBorder;								// Added to menuRect to get actual hit
                                                // rect; FindItem will not be called if
                                                // the mouse is in the border.


	//------------------------------------------------------------------------------------
	// Initialization
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TMenuView into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

	virtual pascal void IMenuView(short rsrcID, short menuWidth, short menuHeight);
		// IMenu reads the menu and sets up the MenuHandle and defproc so that the Menu
        // Manager can communicate with the TMenuView object.


	//------------------------------------------------------------------------------------
	// Menu Definition; all these methods must be overridden
	//------------------------------------------------------------------------------------

	virtual pascal short FindItem(Point hitPt);
		// This is called to convert a point to an item number.

	virtual pascal void PostCommand(TCommand* command);
		// ensure that trackers get tracked immediately

	virtual pascal void Highlight(short whichItem, Boolean turnItOn);
		// Given an item number as returned by FindItem, this is called to highlight the
        // item. This will not be called with whichItem = kNoMenuItem.


	//------------------------------------------------------------------------------------
	// Other methods
	//------------------------------------------------------------------------------------

	virtual pascal void GetMenuViewColors(short theMenu,
										  short theItem,
										  MenuColors& theMenuColors); 
		// Called to get the appropriate menu colors for the menu and item. If the menu
        // and item are 0 then info is for the menubar. If item is 0 then info is for the
        // menu title. Attempts to return best guess at colors based on all available
        // info.

	virtual pascal Boolean IsItemEnabled(short item);
		// Returns the enabled status of an item

	virtual pascal void UpdateHighlight(short oldItem, short newItem);
		// Called to update menu item highlighting.


	//------------------------------------------------------------------------------------
	// Private
	//------------------------------------------------------------------------------------

	virtual pascal void HandleDefproc(short message,
									  MenuHandle theMenu,
									  Rect& menuRect,
									  Point hitPt,
									  short& whichItem);
		// This is called by the custom defproc; it does a CASE on the message and calls 1
        // or more of the methods above.

	virtual pascal void HandleChooseMessage(short message,
											MenuHandle theMenu,
											Rect& menuRect,
											Point hitPt,
											short& whichItem);
		// Called by the message dispatcher when a choose message has been sent

	virtual pascal void HandleDrawMessage(short message,
										  MenuHandle theMenu,
										  Rect& menuRect,
										  Point hitPt,
										  short& whichItem);
	// Called by the message dispatcher when a draw message has been sent

	virtual pascal void HandleSizeMessage(short message,
										  MenuHandle theMenu,
										  Rect& menuRect,
										  Point hitPt,
										  short& whichItem);

	virtual pascal void HandlePopUpMessage(short message,
										   MenuHandle theMenu,
										   Rect& menuRect,
										   Point hitPt,
										   short& whichItem);


	//------------------------------------------------------------------------------------
	// Focusing Methods
	//------------------------------------------------------------------------------------

	virtual pascal Boolean Focus();

	virtual pascal Boolean FocusOnSuperView();


	//------------------------------------------------------------------------------------
	// Misc. Methods
	//------------------------------------------------------------------------------------

	virtual pascal GrafPtr GetGrafPort();

	virtual pascal void CreateMouseDownEventInfo(Point hitPt,
												 EventRecord& anEventRecord,
												 TToolboxEvent* event);
		// Called by HandleChooseMessage when creating a phony mousedown event which is
        // then passed off to the subviews of TMenuView via HandleMouseDown


	//------------------------------------------------------------------------------------
	// Debugging Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// Global functions declarations
//----------------------------------------------------------------------------------------

extern pascal void InitUMenuView();
	// Call this at the start of your program, before creating any TMenuView objects. This
    // can signal Failure.


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

extern Boolean gTrackingInMenu;

#endif


