// UObject.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UOBJECT__
#define __UOBJECT__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UPASCALOBJECT__
#include <UPascalObject.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations.
//----------------------------------------------------------------------------------------

class TObject;
class TStream;


//----------------------------------------------------------------------------------------
// Typedefs for procedureal parameters.
//----------------------------------------------------------------------------------------

typedef pascal void (*DoToObjectType)(TObject* aDependent, void* staticLink);

typedef pascal void (*DoToClassType)(ObjClassID theClass, void* staticLink);													
																   
																   
//----------------------------------------------------------------------------------------
// TObject: Definition of the system's root object
//----------------------------------------------------------------------------------------

class TObject : public PascalObject
{
public:

	//------------------------------------------------------------------------------------
	// Initializer and Pascal I<Method>.
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
        // Call this to put a newly created object into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. Defaults to nothing.

	virtual pascal void IObject();
        // The ancestral initializer. Should be called in the I<ClassName> chain. i.e.
        // IView -> IEvtHandler -> IOBJECT, followd by a called to <ClassName> i.e without
        // the T.

	//------------------------------------------------------------------------------------
	// Change notification
	//------------------------------------------------------------------------------------

	virtual pascal void Changed(ChangeID theChange, TObject* changedBy);
		// Informs this object that it has been changed. In MacApp, "theChange" will often
        // contain a command number, and "changedBy" a command object. Marks all
        // dependents (direct and indirect) as needing updating, asks SELF to
        // "UpdateDependents", then unmarks all dependents. Called often, overridden
        // rarely.

	virtual pascal void DoUpdate(ChangeID& theChange, TObject* changedObject,
								 TObject* changedBy);
		// Informs this object that an object on which it is dependent has been changed.
        // By returning kNoChange in "theChange" the propagation of DoUpdate calls to
        // dependents can be halted. Called rarely, overridden often.

	virtual pascal void MarkDependents(Boolean state);
		// Asks each of the dependents of this object to"MarkRecursively"

	virtual pascal void MarkRecursively(Boolean state);
		// If it is not already marked, marks this object,and recursively asks each
        // dependent to mark itself

	virtual pascal void SetMark(Boolean state);
		// Marks (state=TRUE) or unmarks (state=FALSE) this object

	virtual pascal Boolean IsMarked();
		// Returns TRUE if this object is marked

	virtual pascal void UpdateDependents(ChangeID theChange, TObject* changedObject,
										 TObject* changedBy);
		// Asks each dependent to "HandleUpdate"

	virtual pascal void UpdateNotifiers(ChangeID theChange,
										TObject* changedObject,
										TObject* changedBy);
		// Asks each object on which this object is dependent to "HandleUpdate"

	virtual pascal void HandleUpdate(ChangeID theChange,
									 TObject* changedObject,
									 TObject* changedBy);
		// If SELF is marked as needing updating, update all objects on which SELF is
        // dependent, update SELF ("DoUpdate"), then unmark SELF, and update all
        // dependents of SELF.

	virtual pascal void AddDependent(TObject* dependent);
		// Registers the specified object as a dependent of this object, so that it will
        // be notified of changes in this object.

	virtual pascal void RemoveDependent(TObject* dependent);
		// Remove the specified object from the list of dependents of this object.

	virtual pascal void RemoveAllDependencies();
		// Used by TObject.Free to remove all dependency relations which involve this
        // object, either as notifier or as dependent

	virtual pascal void EachDependent(DoToObjectType DoToDependent, void* staticLink);
		// Performs DoToDependent on every dependent of this object

	virtual pascal void EachNotifier(DoToObjectType DoToNotifier, void* staticLink);
		// Performs DoToNotifier on every object on which this object is dependent


	//------------------------------------------------------------------------------------
	// Stream I/O protocol support.
	//------------------------------------------------------------------------------------

	virtual pascal void ReadFrom(TStream* aStream);
	
	virtual pascal void WriteTo(TStream* aStream);


	//------------------------------------------------------------------------------------
	// Miscellaneous protocol and support methods.
	//------------------------------------------------------------------------------------

	virtual pascal TObject* Clone();
		// Makes a duplicate copy of SELF. The default calls SELF.ShallowClone, which
        // makes a literal copy of instance variables but does not attempt to clone owned
        // objects. A subclass which owns other objects should override this to clone the
        // owned objects and data structures as well.

	virtual pascal void DoToField(const Str255& fieldName, void* fieldAddr,
								  short fieldType);
		// Called by DoToField once for each instance variable in SELF. fieldName is a
        // string representing the name of the variable. fieldAddr is the address of that
        // variable. fieldType is one of the types defined in UMacAppUtilities.

	virtual pascal void Fields(TObject* obj);
		// Calls obj.DoToField once for each instance variable in SELF. Also, It is legal
        // to use a local variable to hold data and pass the address of that variable to
        // DoToField as if it were part of the object's instance variables. By convention,
        // the first call to DoToField should have the class name as the fieldName, NIL as
        // the fieldAddr, and bClass as the fieldType. Each subclass should override this
        // method, particularly with qDebug on, dump its own information, then call
        // INHERITED.

	virtual pascal void DynamicFields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // dynamic area. Defaults to nothing. Override if your object uses the dynamic
        // area See TList for an example.

	virtual pascal void ForAllSubClassesDo(DoToClassType DoToSubClass,
										   void* staticLink);
		// Calls DoToSubClass for each subclass of SELF. The order of traversal is
        // arbitrary; see EachSubClass global procedure.

	virtual pascal void ForAllSuperClassesDo(DoToClassType DoToSubClass,
											 void* staticLink);
		// Calls DoToSubClass for each superclass of SELF. The order of traversal is the
        // order of ancestry starting from the class of SELF; see EachSuperClass global
        // procedure.

	virtual pascal void Free();
		// Called to dispose of an object. Gives object a chance to cleanup after itself.
        // Default simply calls SELF.ShallowFree, which makes no attempt to free instance
        // variables. Should be overridden by any class which allocates space or owns
        // other objects in its instance variables.. Be sure to call INHERITED!

	virtual pascal ObjClassID GetClass();
        // Returns the class ID of SELF.

	virtual pascal void GetClassName(MAName& clName);
        // Returns the class name of SELF.

	virtual pascal Size GetClassSize();
        // Returns the basic instantiation size. i.e. the size in bytes of a newly created
        // object of this class.

	virtual pascal Ptr GetDynamicPtr();
        // Our objects have a dynamic area available. In 2.x it is at the end of the
        // object handle but it may not always be so.Returns a direct heap pointer to the
        // start of the dynamic area.Use with caution as this pointer can be invalidated
        // if the heap compacts.

	virtual pascal Size GetDynamicSize();
        // Our objects can have a dynamic area that can be allocated with a SetDynamicSize
        // call.Returns the size of the dynamic area in bytes. Preferred.

	virtual pascal ObjClassID GetSuperClass();
        // Returns immediate superclass of class. Returns kNilClass for TObject. If we
        // ever get MI this will have to be an enumerator.

	virtual pascal void GetInspectorName(Str255& inspectorName);
        // Returns additional information useful to the inspector. For instance (ha ha!) a
        // TDocument could give its title.

	virtual pascal Boolean IsMemberClass(ObjClassID testClass);
        // True if I am a MEMBER of the testClass. i.e. the same class OR a subclass.

	virtual pascal Boolean IsSameClass(ObjClassID testClass);
        // True if I belong to the immediate class.

	virtual pascal Boolean Lock(Boolean lockIt);
        // Locks down or unlocks an object AND its dynamic area if any. Returns old state.
        // Since objects can float around in memory some occasions may arise where they
        // need to be locked down (The Inspect method is an example: since addresses of
        // instance variables are being passed, the object had better not move during the
        // Inspect!) Typically the old state is saved in a local and re-applied when
        // ascending the call chain. Don't forget to re-apply the old state from failure
        // handlers. _PLEASE_ use this sparingly and don't just go locking stuff down
        // willy-nilly just because you might be afraid of handles.

	virtual pascal void SetDynamicSize(Size newSize);
        // Set this instance's Dynamic Area size in bytes.The dynamic area starts out
        // unallocated. Signals Failure on memory error.The size applies only to the size
        // of the Dynamic Area. Preferred.

	virtual pascal TObject* ShallowClone();
        // Lowest level method for copying an object; should not be overridden except in
        // very unusual cases. Simply calls HandToHand to copy the object data.

	virtual pascal void ShallowFree();
        // Lowest level method for freeing an object; should not be overridden except in
        // very unusual cases. Simply calls Dispose (the Pascal builtin) to free the
        // object.

	virtual pascal void SubClassResponsibility();
        // Called from methods that MUST be overridden in a subclass.Signals failure.
};


//----------------------------------------------------------------------------------------
// In Object Pascal a TObject is a double indirect reference to the object structure. And
// a pointer to a TObject is a pointer to that indirection. However since C++ can have
// value references to objects, there a TObject is the object structure and (here's where
// it gets tricky) a TObject* is a double indirection. Remember that for PascalObject
// derived objects CFront does a double indirection for every single indirection thus, a
// TObject* in C++ is equivalent to a TObject in Object Pascal. This brings us to
// TObjectPtr which reads in C++ as a double indirection, reads in Object Pascal as a
// single indirection but, is in fact a triple indirection
//----------------------------------------------------------------------------------------

typedef TObject* TObjectPtr;					// Same as a TObject variable in Pascal

typedef TObjectPtr* TObjectPtrPtr;				// Same as a ^TObject variable in Pascal

typedef TObjectPtrPtr* TObjectPtrHandle;		// And, of course, would not be complete
                                                // without the quad indirection!

#endif


