// Copyright  1988-1991 by Apple Computer Inc.All rights reserved.

//----------------------------------------------------------------------------------------
// Theory of Operation
//
// This unit allows Popup menus to we instantiated which are managed by the Popup CDEF and
// MDEF that are included in System 7.0 and in the installer script for the Communications
// Toolbox.
//	
// To use this Unit and the Popup CDEF with MacApp applications that run under System 6.x,
// the Popup CDEF and MDEF will need to be included in the application's resource fork.
//	
// This unit actually extends the capabilities provided by the Popup CDEF as follows:
//		allows for non resource-based MENUs (via a patch to GetResource)
//		allows for hierarchical popups (via a patch to PopupMenuSelect)
//	
// When a menu item from a CMNU menu is selected in the Popup, the Popup's DoEvent method
// then propagates the command number down the handler chain beginning with the Popup
// view:
//
//			currentCommand = GetCurrentCommand;
//			if (currentCommand != cNoCommand)
//				this->DoMenuCommand(currentCommand);
//
// It's still under debate whether the command should be propagated down the handler chain
// or down the target chain as follows:
//				gApplication.GetTarget->DoMenuCommand(currentCommand)
//
// Note that this is a significant improvement over MacApp 2.0's popup support because the
// command number associated with a CMNU was not propagated down the handler chain.
//	
// Usage
//	
// For Sys 6.0 support: include the Popup CDEF in the resource fork of your application
// with the following lines in your app.r file:
//		include "Popup.rsrc";
//	
// Also, include either the CMNU or MENU resource in your application's resource fork.
	
// To handle the command number associated with the selected item in the popup, include
// code such as the following in the holding view (e.g. a TDialogView subclass) or in the
// document (this code was taken from DemoDialogs):
//
//		pascal void TTestDialogView::DoMenuCommand(CmdNumber aCmdNumber) // override 
//		
//			{
//			switch (aCmdNumber) {
//				case cResetTextStyle:
//					{
//					}
//				case cTextStyleChanged:
//					{
//					}
//				default:
//					inherited::DoMenuCommand(aCmdNumber);
//			}
//			}
//	
// Limitations
//
// The first item in the popup can't be a hierarchical menu. This is actually a bug in the
// Popup CDEF as it exists in System 7.0b1 and earlier versions (including the Comm
// Toolbox release). A bug report has been filed with SQA.
//----------------------------------------------------------------------------------------


#ifndef __UPOPUP__
#define __UPOPUP__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UCONTROL__
#include <UControl.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const	Boolean		kUseAddResMenu	= TRUE;
const	Boolean		kDontUseAddResMenu	= !kUseAddResMenu;
			

//----------------------------------------------------------------------------------------
// PopupTemplate
//----------------------------------------------------------------------------------------

struct	PopupTemplate
{
	short	rsrcID;
	short	currentItem;
	short	itemOffset;
	// !!! need to add these:
	// Style	itsStyle;
	// short	itsJust;
	// Boolean	useAddResMenu;
	// ResType	useAddResMenuResType;
	// Str255	itsLabel;							// Actually a variable length P-String
};

typedef	PopupTemplate* PopupTemplatePtr;


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class	TDocument;


//----------------------------------------------------------------------------------------
// TPopup: Mimics a typical toolbox button as used in a dialog.
//----------------------------------------------------------------------------------------

class TPopup : public TCtlMgr
{
public:
	short		fItemOffset;
	short		fItsCurrentItem;

	short fTitleStyle;							// flags for both title's style &&
                                                // justification

	MenuHandle	fItsPopupMenuHandle;

	long		fLastPopupResult;


	virtual pascal void Initialize();	// override 
		// Puts a newly created TPopup into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IPopup(TView* 			itsSuperView,
								 const VPoint&	itsLocation, 
								 const VPoint&	itsSize,
								 SizeDeterminer	itsHSizeDet,
								 SizeDeterminer	itsVSizeDet,
								 short			itsMenuID,					// rsrc ID of 'MENU' or 'CMNU' rsrc 
								 short			itsCurrentItem,
								 short			itsItemOffset,				// width of the popup title area 
								 const Str255&	itsLabel,
								 /* Style */ short itsStyle,				// title's style 
								 short			itsJust,					// title's justification 
								 Boolean		useAddResMenu,				// TRUE if want to AddResMenu (can use
																			// this option iff menuID != kNoResource)
								 ResType		useAddResMenuResType);		// used if useAddResMenu is TRUE
		// Initializes the popup.

	virtual pascal short GetPopupTitleStyle(/* Style */ short itsStyle, short itsJust);
		// map itsStyle and itsJust to popupTitleStyle.

	virtual pascal void IRes(TDocument*	itsDocument,
							 TView*		itsSuperView,
							 Ptr&		itsParams);	 // override 
		// Initialize the view from a resource template.

	virtual pascal void CreateCMgrControl(const VRect& 	itsBounds,
										  const Str255&	itsTitle,
										  long		itsVal,
										  long		itsMin,
										  long		itsMax,
										  short 	itsProcID); // override 

	virtual pascal TObject* Clone(void); // override 
		// calls inherited::Clone and then clones owned objects
		
	virtual pascal void WRes(ViewRsrcHandle	theResource, Ptr& itsParams); // override 
		// Write this object out as a view resource. 

	virtual pascal void WriteRes(ViewRsrcHandle theResource, 
								 Ptr& itsParams); // override 
		// Set up the type and signature of this object and call WRes. 


	//------------------------------------------------------------------------------------
	// Imaging
	//------------------------------------------------------------------------------------

	virtual pascal void Draw(const VRect& area); // override 
		// sets a patch to GetResource in certain cases 


	//------------------------------------------------------------------------------------
	// tracking
	//------------------------------------------------------------------------------------

	virtual pascal void DoMouseCommand(VPoint&			theMouse,
									   TToolboxEvent*	event,
									   Point			hysteresis); // override 

	//------------------------------------------------------------------------------------
	// accessors
	//------------------------------------------------------------------------------------

	virtual pascal void AttachMenuHandle(MenuHandle itsMenuHandle);
		// attaches itsMenuHandle to this popup 

	virtual pascal MenuHandle GetMenuHandle(void);
		// returns the MenuHandle by accessing the control handle's private storage 
		
	virtual pascal CmdNumber GetCurrentCommand(void);
		// For the currently selected menu item, returns its CmdNumber. 

	virtual pascal short GetCurrentItem(void);
		// Returns the menu item of the last selected item.

	virtual pascal void GetItemText(short item, Str255& theText);
		// Returns in "theText" the text for the specified item in the menu. 

	virtual pascal short GetMenuID(void);
		// Returns the menu ID.
		
	virtual pascal short GetNumberOfItems(void);
		// Returns the number of items in the menu. 
		
	virtual pascal short GetRsrcID(void);
		// Returns the resource ID, this is always the same as the menu ID.

	virtual pascal void GetTitle(Str255& title);
		// returns the popup's title 

	virtual pascal void SetCurrentItem(short item, Boolean redraw);
		// Sets the currently selected item, redrawing if required. 
		
	virtual pascal void SetPopup(MenuHandle itsMenuHandle,
								 short itsCurrentItem,
								 Boolean redraw);
		// Sets the fields of the popup to contain theMenu, with currentItem selected,
        // redrawing if required.

	virtual pascal void SetTitle(const Str255& title);
		// sets the popup's title 

	virtual pascal void Fields(TObject* obj); // override 
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};
	
#endif