// UPrintHandler.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UPRINTHANDLER__
#define __UPRINTHANDLER__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UVIEWBEHAVIOR__
#include <UViewBehavior.h>
#endif


//----------------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------------

class TView;
class TDocument;
class TPrintHandler;


//----------------------------------------------------------------------------------------
// TPrintMenuBehavior
//----------------------------------------------------------------------------------------

class TPrintMenuBehavior : public TBehavior
{
public:

	TPrintHandler* fPrintHandler;				// the managed print handler

	virtual pascal void Initialize();
		// Puts a newly created TPrintMenuHandler into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IPrintMenuBehavior(TPrintHandler*	itsPrintHandler);
		// initializes this behavior, and sets fPrintHandler to itsPrintHandler


	//------------------------------------------------------------------------------------
	// Menu Commands
	//------------------------------------------------------------------------------------

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
		// forwards into fPrintHandler's DoPrintCommand

	virtual pascal void DoSetupMenus();
		// forwards into fPrintHandler's DoSetupPrintMenus
};


//----------------------------------------------------------------------------------------
// TPrintHandler: Handles printing at a specified resolution on behalf of a view. This
// class is really a non-functional stub for programs that don't use the printing building
// block.
//----------------------------------------------------------------------------------------

class TPrintHandler : public TViewBehavior
{
public:
	TView* fView;								// The view whose printing is handled

	TDocument* fDocument;						// the document printed by this handler

	Point fDeviceRes;							// formal printer resolution, spots per
                                                // inch

	VPoint fViewPerPage;						// amount of the view visible on a page

	long fFocusedPage;							// page number that is currently focused


	//----------------------------------------------------------------------------------------
	// Initialization and termination
	//----------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TStdPrintHandler into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IPrintHandler(TView* itsView);
		// Initialize the printHandler, and associate it and 'itsView' with each other

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Used by the inspector to get the name of this class

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

	virtual pascal void FocusOnInterior();
		// Stub

	virtual pascal Handle GetPrintInfo();
		// Stub. Returns NIL by default

	virtual pascal void SetDefaultPrintInfo();
		// Stub


	//------------------------------------------------------------------------------------
	// Printing-related reformatting and computations
	//------------------------------------------------------------------------------------

	virtual pascal VCoordinate BreakFollowing(VHSelect vhs,
											  VCoordinate prevBreak,
											  Boolean& automatic);
		// Returns the location of the page break which follows the page break located at
        // 'prevBreak', in direction vhs; returns automatic = TRUE if the page-break is
        // thought of as an 'automatic' rather than a 'manual' (user-specified) one. Note
        // that page-breaks in dimension 'v' are drawn as vertical lines, those in
        // dimension 'h' as horizontal lines

	virtual pascal void CalcPageStrips(VPoint& pageStrips);
		// Recalculate the number of strips of pages in each dimension

	virtual pascal void CalcViewPerPage(VPoint& amtPerPage);
		// Computes the amount of view, in each dimension, to be allocated to a printed
        // page

	virtual pascal void CheckPrinter();
		// See if there are changed printer-configuration parameters which need to be
        // absorbed, and if so, absorb them

	virtual pascal void LocatePageInterior(long pageNumber, VPoint& loc);
		// Decide where the top-left-most point of the the interior of the page should be

	virtual pascal void PrinterChanged();
		// The metrics relating to printer use have changed; absorb the information and
        // react

	virtual pascal void RedoPageBreaks();
		// Recompute the dividing lines between areas of the view which will be mapped
        // into different printed pages

	virtual pascal void Reset();
		// Resets the print handler to the default values


	//------------------------------------------------------------------------------------
	// Printing-related menu mgmt
	//------------------------------------------------------------------------------------

	virtual pascal Boolean DoPrintCommand(CmdNumber aCmdNumber);
		// handles posting the print command, returns TRUE if handled

	virtual pascal void DoSetupPrintMenus();
		// sets up the print menus


	//------------------------------------------------------------------------------------
	// Printing-related commands
	//------------------------------------------------------------------------------------

	virtual pascal void Print(CmdNumber itsCmdNumber, Boolean& proceed);
		// By default sets 'proceed' to TRUE and returns NIL.

	virtual pascal Boolean SetupForFinder();
		// Intended to set up the print handler for finder printing.


	//------------------------------------------------------------------------------------
	// Printing-related screen feedback
	//------------------------------------------------------------------------------------

	virtual pascal void DrawPrintFeedback(const VRect& area);
		// Draw page-breaks, page-numbers, view-borders, rulers, etc.

	virtual pascal void DrawPageBreak(VHSelect vhs,
									  long whichBreak,
									  VCoordinate loc,
									  Boolean automatic);
		// Does nothing.


	//------------------------------------------------------------------------------------
	// Actual printing
	//------------------------------------------------------------------------------------

	virtual pascal long MaxPageNumber();
		// Returns the largest page number which could be reasonably printed, given the
        // properties of the view; in cases like a SpreadSheet, this is not necessarily as
        // large as the product (row strips) x (column strips)

	virtual pascal void SetPageInterior(long pageNumber);
		// Set up the pad-space Interior rectangle for the given page number

	virtual pascal void SetPageOffset(const VPoint& coord);
		// Given the view coordinates of the top-left-most point of the view which is
        // being mapped into the current page (in coord), this method's job is to compute
        // the correct fRelOrigin field for the current page
};


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

extern  Boolean gFinderPrintingProceed;			// proceed with Finder printing


#endif


