// UScroller.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __USCROLLER__
#define __USCROLLER__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TDocument;
class TScrollerScrollBar;
class TToolboxEvent;


//----------------------------------------------------------------------------------------
// ScrollerTemplate
//----------------------------------------------------------------------------------------

struct ScrollerTemplate
{
	Boolean wantVSBar;
	Boolean wantHSBar;
	long vertMax;
	long horzMax;
	short vScrollUnits;
	short hScrollUnits;
	Boolean vConstrain;
	Boolean hConstrain;
	Rect sBarOffsets;
};

typedef ScrollerTemplate* ScrollerTemplatePtr;


//----------------------------------------------------------------------------------------
// TScroller: A view which can "scroll" its contents by translating coordinates.
//----------------------------------------------------------------------------------------

class TScroller : public TView
{
public:
	VPoint fScrollLimit;						// the 'limit' of the subviews that should
                                                // be scrollable

	VPoint fMaxTranslation;						// the computed maximum amount of
                                                // translation taking into account the
                                                // visible portion of the view. This keeps
                                                // us from scrolling the bottom of a view
                                                // to the top of a window

	TScrollerScrollBar* fScrollBars[2];			// associated scroll bars if any

	VPoint fScrollUnit;							// increment of scroll

	Boolean fConstrain[2];						// should scrolling be constrained to a
                                                // multiple of my scrollunit

	VRect fSBarOffsets;							// how much to offset the associated
                                                // scroll bars from my bounds

	Boolean fRespondsToFunctionKeys;			// if it responds to PgUp/ PgDn, etc.


	//------------------------------------------------------------------------------------
	// Creation/Destruction Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TScroller into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

	virtual pascal void IScroller(TView* itsSuperView,
								  const VPoint& itsLocation,
								  const VPoint& itsSize,
								  SizeDeterminer itsHSizeDet,
								  SizeDeterminer itsVSizeDet,
								  const VPoint& itsMax,
								  Boolean wantHorzSBar,
								  Boolean wantVertSBar);
		// Called to initialize a scroller procedurally.

	virtual pascal void CreateScrollBar(VHSelect itsDirection);
		// Called to create a scrollbar in 'itsDirection'.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize a scroller from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void CreateTemplateScrollBar(VHSelect itsDirection);
		// Called to create a scrollbar in 'itsDirection' from a resource template.

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
			// For writing a scroller view out to a resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a scroller to a resource, works in conjunction with tbe previous
        // method.

	virtual pascal void Free();
		// If there are attached scrollbars frees them before calling inherited Free.


	//------------------------------------------------------------------------------------
	// SubView Management Methods
	//------------------------------------------------------------------------------------

	virtual pascal void AddSubView(TView* theSubView);
		// Calls inherited AddSubView and then calls 'theSubView' to BeInScroller passing
        // SELF as the scroller to be in.

	virtual pascal void RemoveSubView(TView* theSubView);
		// Calls BeInScroller with a parameter of NIL, for 'theSubView', before calling
        // inherited RemoveSubView.

	virtual pascal void ForceRedraw();
		// Invalidates the entire view forcing a redraw.


	//------------------------------------------------------------------------------------
	// Size Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Resize(const VPoint& newSize, Boolean invalidate);
		// Resizes the scroller to 'width' & 'height', redrawing if 'invalidate' is TRUE.


	//------------------------------------------------------------------------------------
	// Location Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Locate(const VPoint& newLoc, Boolean invalidate);
		// Calls inherited Locate before calling AdjustScrollbars.

	virtual pascal void AdjustScrollBars(Boolean invalidate);
		// Resizes and re-Locates the scrollbars if required.

	virtual pascal void SubViewChangedSize(TView* theSubView, const VPoint& delta);
		// Adjust the scroll limits based on the size change of the subviews.

	virtual pascal void SetScrollLimits(const VPoint& scrollLimit,
										Boolean drawScrollBars);
		// Sets the scroll of the attached scrollbars, if there are any.

	virtual pascal void SetScrollParameters(const VPoint& units,
											Boolean horzConstraint,
											Boolean vertConstraint);
		// Sets up the fScrollUnit and fConstrain fields of the scroller.


	//------------------------------------------------------------------------------------
	// Scrolling Methods
	//------------------------------------------------------------------------------------

	virtual pascal void HaveScrollBar(TScrollerScrollBar* theScrollBar,
									  VHSelect direction);
		// Sets the fScrollbar field in the given 'direction' to 'theScrollBar'.

	virtual pascal void ScrollDraw(const VPoint& delta, Boolean invalidate);
		// Method responsible for actually doing the scrolling by calling moving bits.
        // Called by DoScroll.Invalidates newly revealed area if invalidate is true else
        // updates the redraws the newly revealed area if invalidate is false. !!! NOTE
        // this does not match the behaviour of other invalidate parameters because they
        // interpret invalidate equals FALSE to neither draw nor invalidate.Well we
        // already know that the way that invalidate/ redraw/ don't-render control is
        // handled needs _MAJOR_ overhauling anyways Sigh!

	virtual pascal void DoScroll(const VPoint& delta, Boolean redraw);
		// Method responsible for handling the scrolling, calls ScrollDraw if required.

	virtual pascal VCoordinate ScrollStep(VHSelect vhs, short partCode);
		// Depending on the 'partCode' returns the amount to be scrolled.

	virtual pascal VCoordinate ScrollRelative(VHSelect vhs, VCoordinate sBarValue);
		// Returns the relative amount to scroll.

	virtual pascal void ScrollBy(const VPoint& delta, Boolean redraw);
		// Called to scroll by 'deltaH & 'deltaV'.

	virtual pascal void ScrollTo(const VPoint& scrollLoc, Boolean redraw);
		// Scrolls to the required coordinates, calls SetLocalOrigin.

	virtual pascal void SetLocalOrigin(const VPoint& newOrigin, Boolean redraw);
		// Sets the local origin by scrolling to the required coordinates, calls ScrollBy.

	virtual pascal void AutoScroll(const VPoint& viewPt, VPoint& delta);
		// Called during auto scrolling to return the delta to be scrolled.

	virtual pascal void RevealRect(const VRect& rectToReveal,
								   const VPoint& minToSee,
								   Boolean redraw);
		// Shows 'rectToReveal' with the minimum amount to be seen set to 'minToSee' by
        // calling ScrollBy.


	//------------------------------------------------------------------------------------
	// Miscellaneous Methods
	//------------------------------------------------------------------------------------

	virtual pascal void DoKeyCommand(TToolboxEvent* event);
		// Handles when the Home, End, PageUp & PageDown keys are used.

	virtual pascal TScroller* GetScroller(Boolean immediateSuperView);
		// Returns SELF.


	//------------------------------------------------------------------------------------
	// Inspecting Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
	// Used by the Inspector and the Debugger to display the contents of this class's
    // fields.
};



#endif


