// USection.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __USECTION__
#define __USECTION__

//----------------------------------------------------------------------------------------
//  Unit: USection
//
//  OVERVIEW
//
// TSection is designed to provide Edition Manager support. TSection is an abstract
// superclass which references the main Edition Manager data structure (the
// SectionHandle), and a designator (designating which portion of the document is
// published/ subscribed). Specialized subclasses of TSection are required to represet the
// TPublisher and TSubscriber.
//
//  CLASS HIERARCHY
//
//  TSectionabstract superclass for sections
//  TPublisheran Edition Manager publisher
//  TSubscriberan Edition Manager subscriber
//
//  USAGE
//
// TPublisher/ TSubscriber is designed to be used with the Edition Manager support that is
// built into MacApp. Whenever a portion of the document is publisher/ subscribed, a
// TSection object is instantiated. The TSection object references a TDesignator.
//----------------------------------------------------------------------------------------

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif

#ifndef __USTREAM__
#include <UStream.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const int kFound = True;						// the edition file was found

const int kNotFound = False;					// the edition file wasn't found


//----------------------------------------------------------------------------------------
// passed into PROCEDURE TSection.CancelSection()
//----------------------------------------------------------------------------------------

const int kCancel = True;						// cancel the section

const int kDontCancel = False;					// don't cancel the section

const unsigned long kDesignatorResType = 'DESG';// the ResType for saving a designator to
                                                // a resource


//----------------------------------------------------------------------------------------
// Section Change ID's
//----------------------------------------------------------------------------------------

const short cSubscriberChanged = 610;			// change id indicating that the
                                                // subscriber changed

const short cPublisherChanged = 611;			// change id indicating that the publisher
                                                // changed


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TDesignator;
class TEditionDocument;
class TFile;
class TFileHandler;


//----------------------------------------------------------------------------------------
// TSection: TSection is an abstract superclass providing common support for sections -
// both subscribers and publishers.
//----------------------------------------------------------------------------------------

class TSection : public TObject
{
public:
	TEditionDocument* fDocument;				// the document that owns this section

	TDesignator* fDesignator;					// the designated portion of the document

	SectionHandle fSectionHandle;				// reference to the section record for
                                                // this edition

	short fRsrcID;								// rsrc id of stored form (needed for
                                                // Read)

	EditionInfoRecord fLastEditionInfo;			// stored at various points for later use

	Boolean fCanceled;							// whether this section is canceled

	Boolean fDirty;								// section hasn't been saved

	Boolean fFound;								// whether this edition was found by
                                                // RegisterSection

	Boolean fNewSection;						// this is a brand spankin' new section


	//------------------------------------------------------------------------------------
	//   contructors/destructors 
	//------------------------------------------------------------------------------------

	virtual pascal void ISection(TEditionDocument* itsDocument,
								 TDesignator* itsDesignator,
								 SectionHandle itsSectionHandle,
								 short itsRsrcID);
		// initializes the edition object

	virtual pascal void Free();
		// calls Unregister, disposes the SectionHandle and AliasHandle


	//------------------------------------------------------------------------------------
	//   data accessors 
	//------------------------------------------------------------------------------------

	virtual pascal void AttachSectionHandle(SectionHandle itsSectionHandle);
		// attach a SectionHandle to this TSection object

	virtual pascal void GetEditionName(Str255& name);
		// Returns the name of the edition container file.

	virtual pascal TFileHandler* GetFileHandler();
		// returns the TFileHandler of the document with which this section is associated

	virtual pascal SectionType GetSectionType();
		// returns the fSectionHandle^^.kind

	virtual pascal UpdateMode GetUpdateMode();
		// returns the fSectionHandle^^.mode


	//------------------------------------------------------------------------------------
	//   saving/restoring 
	//------------------------------------------------------------------------------------

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);
		// writes the section handle as a sect resource and the alias handle as a alis
        // resource

	virtual pascal void DoRead(TFile* aFile);
		// reads the section handle from a sect resource and the alias handle from a alis
        // resource

	virtual pascal void DoNeedDiskSpace(long& dataForkBytes, long& rsrcForkBytes);
		// Bytes required to store saved sections on disk. When called, both parameters
        // are set to 0 for you.

	virtual pascal void Associate(FSSpecPtr newSectionDocument);
		// call this to associate the section handle with a new section document


	//------------------------------------------------------------------------------------
	//  miscellaneous 
	//------------------------------------------------------------------------------------

	virtual pascal void Delete();
		// removes the section resources

	virtual pascal Boolean IsDirty();
		// returns TRUE if this section is dirty

	virtual pascal Boolean IsRegistered();
		// returns TRUE if fRegistered & (IsRegisteredSection returns noErr)

	virtual pascal Boolean IsSectionSelected();
		// returns TRUE if the given section is in the user selection

	virtual pascal void MarkDirty();
		// marks this section as dirty so that it gets saved

	virtual pascal OSErr Register();
		// calls RegisterSection on the Section Handle and sets fRegisteredSection

	virtual pascal void UnRegister();
		// calls UnRegisterSection on the Section Handle and sets fRegisteredSection

	virtual pascal void UpdateEditionInfo();
		// updates fEditionInfo for later use

	virtual pascal void CancelSection(Boolean cancel);
		// if cancel is TRUE then calls UnRegister and sets fCanceled to TRUE else if
        // cancel is FALSE then calls Register and sets fCanceled to FALSE

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


//----------------------------------------------------------------------------------------
// TPublisher: each instance of this class represents a part of the document that is
// published
//----------------------------------------------------------------------------------------

class TPublisher : public TSection
{
public:

	//------------------------------------------------------------------------------------
	//   contructors/destructors 
	//------------------------------------------------------------------------------------

	virtual pascal void IPublisher(TEditionDocument* itsDocument,
								   TDesignator* itsDesignator,
								   SectionHandle itsSectionHandle,
								   short itsRsrcID);
		// initializes a publisher object


	//------------------------------------------------------------------------------------
	//   change notification 
	//------------------------------------------------------------------------------------

	virtual pascal void DoUpdate(ChangeID& theChange,
								 TObject* changedObject,
								 TObject* changedBy);
		// marks this publisher as changed calling MarkAsChanged


	//------------------------------------------------------------------------------------
	//   saving/restoring 
	//------------------------------------------------------------------------------------

	virtual pascal void Publish();
		// publishes this edition to the edition file

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);
	// calls INHERITTED Write if (mode is pumOnSave) & IsChanged then Publish


	//------------------------------------------------------------------------------------
	//   miscellaneous 
	//------------------------------------------------------------------------------------

	virtual pascal void Delete();
		// deletes the edition file; called when the doc is saved if the user canceled the
        // publisher

	virtual pascal void DeleteEditionFile();
		// delete the edition container file

	virtual pascal Boolean IsChanged();
		// returns TRUE if the mdDate of the SectionRecord is newer than the mdDate of the
        // edition on disk

	virtual pascal void MarkAsChanged(TimeStamp mdDate);
		// marks this publisher as changed; should be called each time the user edits the
        // data in the publisher

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


//----------------------------------------------------------------------------------------
// TSubscriber - each instance represents a subscriber for this document.
//----------------------------------------------------------------------------------------

class TSubscriber : public TSection
{
public:

    //------------------------------------------------------------------------------------
	//   contructors/destructors 
    //------------------------------------------------------------------------------------

	virtual pascal void ISubscriber(TEditionDocument* itsDocument,
									TDesignator* itsDesignator,
									SectionHandle itsSectionHandle,
									short itsRsrcID);
		// initializes the subscriber object


    //------------------------------------------------------------------------------------
	//   saving/restoring 
    //------------------------------------------------------------------------------------

	virtual pascal void Subscribe();
		// updates this subscriber from the edition on disk

	virtual pascal void SubscribeIfNewer();
		// calls Subscribe if a newer edition is available


    //------------------------------------------------------------------------------------
	//   miscellaneous 
    //------------------------------------------------------------------------------------

	virtual pascal Boolean IsNewerEditionAvailable();
		// return TRUE if there is a newer version of this edition on disk

	virtual pascal void OpenPublisher();
		// calls GetEditionInfo & GotoPublisherSection

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


//----------------------------------------------------------------------------------------
// TSectionStream reads data from and writes data to an already open EditionContainerFile.
// Should be enhanced to support streaming data concurrently of various OS Types (e.g.
// PICT, TEXT, etc) rather than simply streaming one type at a time.
//----------------------------------------------------------------------------------------

class TSectionStream : public TStream
{
public:
	EditionRefNum fEditionRefNum;				// refnum of EditionContainerFile

	OSType fEditionOSType;						// OSType to write to EditionContainerFile

	long fPosition;								// offset into the EditionContainerFile

	long fSize;									// size of the fEditionOSType

	virtual pascal void Initialize();
		// Puts a newly created TSectionStream into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void ISectionStream(EditionRefNum itsEditionRefNum, OSType itsEditionOSType);
		// Initialize the TSectionStream.

	virtual pascal void Reset(OSType itsEditionOSType, long itsSize);
		// Resets fPosition to 0, fEditionOSType to itsEditionOSType, and fSize to
        // itsSize.

	virtual pascal long GetPosition();
		// Returns fPosition.

	virtual pascal void SetPosition(long newPosition);
		// Sets fPosition.

	virtual pascal long GetSize(void);
		// Returns fSize.

	virtual pascal void SetSize(long newSize);
		// Sets fSize.

	virtual pascal void ReadBytes(void* p, long count);
		// Reads bytes from the Edition Container File.

	virtual pascal void WriteBytes(const void* p, long count);
		// Write bytes to the Edition Container File.
};


#endif


