// UStream.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.
// Originally written by Larry S. Rosenstein.Used by permission

#ifndef __USTREAM__
#define __USTREAM__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __GEOMETRY__
#include <Geometry.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif


//----------------------------------------------------------------------------------------
// This unit implements a rudimentary stream abstract type, and subclasses that do I/ O to
// files and handles.This can be used to implement document I/ O and clipboard I/ O with the
// same code. Streams can write out and read back objects, using the features in UObject
// to get an object's class ID and name.The format of an object in the stream is as
// follows:
//
//		2 bytesclass ID (at the time it was written)
//		4 bytessize of object in stream, excluding class ID
//		1 bytesize of class name
//		n bytesclass name
//		m bytesdata specific to object
//
// The class ID in the stream might not match the class ID of the same class in the
// current application.(It depends on how the class IDs were assigned.) The stream code
// uses the class name to identify the actual class.It is not necessary, however, to write
// out the class name for every class. The stream will associate the classID in the stream
// with the correct class ID for the application.The first object of a given class has its
// name written into the stream, along with its classID.When read back, the name is used
// to find the class ID in the current application, which is associated with the class ID
// in the stream.Subsequent objects of that class can be written with an empty class name,
// and the table of class IDs will be consulted to find out the proper class ID.
//
// TStream does not handle structures in which objects are referenced more than once.To do
// this would require that the stream maintain a table of all the objects written out.The
// first time an object was written it would be added to the table.If the same object was
// written again, only its table index (or some other ID) would be written.A similar
// process would be done to read the object back.
//
// TStream will skip over objects that it can't understand.To do this, it writes the size
// of each object in a known position.This means that writing an object is a 2 step
// process.You first write the object header, by calling WriteObject, and after the entire
// object is written you fill in its size, by calling WriteObjectSize.
//
// TStream.ReadObject will return NIL if it can't understand a particular object, or if
// the object was actually NIL.It returns a flag indicating the difference.
//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Forward class declarations
//----------------------------------------------------------------------------------------

class TFile;


//----------------------------------------------------------------------------------------
// TStream: This is the abstract stream class.You would normally create instances of one
// of the concrete subclasses below. 
//----------------------------------------------------------------------------------------

class TStream : public TObject
{
public:
	short fClassMapSize;						// handle to class ID mapping table 

	Handle fClassMap;							// number of entries in the table 

	//------------------------------------------------------------------------------------
	// Creation/Destruction
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TStream into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED. 

	virtual pascal void IStream();

	virtual pascal void Free();

	//------------------------------------------------------------------------------------
	// Access: These methods have default implementations that assume a 0-length stream. 
	//------------------------------------------------------------------------------------

	virtual pascal long GetPosition();

	virtual pascal void SetPosition(long newPosition);

	virtual pascal long GetSize();

	virtual pascal void SetSize(long newSize);


	//------------------------------------------------------------------------------------
	// Note - all reading and writing methods signal failure in case of an I/ O error.
	//------------------------------------------------------------------------------------

	virtual pascal Boolean AtEnd();				
		// Return TRUE if at the end of the stream. 

	virtual pascal void ReadBytes(void* p, long count); 
		// This must be overridden by subclasses; all other reading methods call this. 

	virtual pascal void ReadCharacter(short& data);

	virtual pascal void ReadInteger(short& data);

	virtual pascal void ReadLong(long& data);

	virtual pascal void ReadPoint(Point& data);

	virtual pascal void ReadRect(Rect& data);

	virtual pascal void ReadString(String& data, short maxSize); 
		// Read a string.maxSize is the amount of memory available, which is the maximum
        // string length + 1.If the size of the string in the stream is too large, then it
        // signals Failure with err=paramErr. 

	virtual pascal void ReadObject(TObject** data, Boolean& known);
		// Returns an uninitialized object; known is set to TRUE if the object was valid
        // (i.e., the class of the object is known to the program. The function result
        // will be NIL if the object was unknown or if NIL was found in the stream. 

	virtual pascal void ReadStreamObject(TObject** data, Boolean& known);
		// First calls ReadObject and if the result is not NIL, calls ReadFrom on the new
        // object. 


	//------------------------------------------------------------------------------------
	// Writing
	//------------------------------------------------------------------------------------

	virtual pascal void WriteBytes(void* p,
								   long count); 
		// This must be overridden by subclasses; all other writing methods call this. 

	virtual pascal void WriteCharacter(short data);

	virtual pascal void WriteInteger(short data);

	virtual pascal void WriteLong(long data);

	virtual pascal void WritePoint(Point data);

	virtual pascal void WriteRect(const Rect& data);

	virtual pascal void WriteString(const String& data); 
		// Write a string.This takes a StringPtr, so it can accommodate any size string. 


	//------------------------------------------------------------------------------------
	// These methods are used to write an object to a stream.You must call WriteObject
    // first, then tell the object to write its specific data, and then call
    // WriteObjectSize. 
	//------------------------------------------------------------------------------------

	virtual pascal void WriteObject(TObject* data, long& sizePosition);

	virtual pascal void WriteObjectSize(long sizePosition);

	virtual pascal void WriteStreamObject(TObject* data);
		// This calls WriteObject, then data.WriteTo, and finally WriteObjectSize. 


	//------------------------------------------------------------------------------------
	// Class handling: used internally 
	//------------------------------------------------------------------------------------

	virtual pascal ObjClassID LookupClassID(ObjClassID id);

	virtual pascal void RegisterClassID(ObjClassID oldID, ObjClassID newID);


	//------------------------------------------------------------------------------------
	// Debugging
	//------------------------------------------------------------------------------------
	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TFileStream: This implements a stream on a file. The client is responsible for opening
// and closing the file. 
//----------------------------------------------------------------------------------------

class TFileStream : public TStream
{
public:
	TFile* fFile;								// file refnum 


	//------------------------------------------------------------------------------------
	// Creation/Destruction 
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TFileStream into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED. 

	virtual pascal void IFileStream(TFile* itsFile);


	//------------------------------------------------------------------------------------
	// SubClass Implementations 
	//------------------------------------------------------------------------------------

	virtual pascal long GetPosition();

	virtual pascal void SetPosition(long newPosition);

	virtual pascal long GetSize();

	virtual pascal void SetSize(long newSize);

	virtual pascal void ReadBytes(void* p, long count); 

	virtual pascal void WriteBytes(void* p, long count); 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// THandleStream: This implements a stream into or out of a handle.The client is
// responsible for allocating and disposing the handle. 
//----------------------------------------------------------------------------------------

class THandleStream : public TStream
{
public:
	Handle fHandle;								// the actual handle 

	long fPosition;								// offset into the handle 

	long fSize;									// size of the stream (may be smaller than
                                                // handle) 

	long fGrowthSize;							// how much to grow the handle when needed
                                                // 


	//------------------------------------------------------------------------------------
	// Creation/Destruction
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created THandleStream into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void IHandleStream(Handle itsHandle, long growth);
		// Initialize stream; growth specifies how much to grow the handle when it needs
        // to grow. You use this so that the handle isn't grown by a small number of bytes
        // on each write operation. 
									  
	virtual pascal void Free();
		// This also shrinks the handle to the current position, eliminating any extra
        // space at the end. 


	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------

	virtual pascal long GetPosition();

	virtual pascal void SetPosition(long newPosition);

	virtual pascal long GetSize();

	virtual pascal void SetSize(long newSize);

	virtual pascal void ReadBytes(void* p, long count); 

	virtual pascal void WriteBytes(void* p, long count); 


	//------------------------------------------------------------------------------------
	// Growing 
	//------------------------------------------------------------------------------------

	virtual pascal long GrowthSize(long needed);
		// Returns amount to grow by, given that we need 'needed' bytes.Default is to grow
        // by Max(fGrowth, needed). 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TCountingStream: This implements a stream that counts up the total number of bytes. It
// is only usable for writing. You will get a debugging break if you try to read from this
// kind of stream. 
//----------------------------------------------------------------------------------------

class TCountingStream : public TStream
{
public:
	long fPosition;								// current position in the stream 

	long fSize;									// current size of stream 


	//------------------------------------------------------------------------------------
	// Creation/Destruction 
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TCountingStream into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void ICountingStream();


	//------------------------------------------------------------------------------------
	// Subclass implementations 
	//------------------------------------------------------------------------------------

	virtual pascal long GetSize();

	virtual pascal void WriteBytes(void*, long count); 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TResourceStream: This implements a stream that reads and writes to partial resources.
// Since the partial resources traps are only available in System 7, this stream class can
// only be used under System 7. 
//----------------------------------------------------------------------------------------

class TResourceStream : public TStream
{
public:
	Handle fResource;							// the resource to write to/ read from

	long fPosition;								// offset into the resource

	long fSize;									// offset into the resource

	virtual pascal void Initialize();
		// Puts a newly created TResourceStream into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void IResourceStream(ResType theType, short theID);
		// Initialize the TResourceStream.

	virtual pascal long GetPosition();
		// Returns fPosition.

	virtual pascal void SetPosition(long newPosition);
		// Sets fPosition.
	
	virtual pascal long GetSize(void);
		// Returns fSize.

	virtual pascal void SetSize(long newSize);
		// Sets fSize.
	
	virtual pascal void ReadBytes(void* p, long count);
		// Reads bytes from the resource calling ReadPartialResource.

	virtual pascal void WriteBytes(const void* p, long count);
		// Write bytes to the resource calling WritePartialResource.
};

#endif


