// USynchScroller.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __USYNCHSCROLLER__
#define __USYNCHSCROLLER__

//----------------------------------------------------------------------------------------
// Theory of Operation
//
// This is Larry Tesler's excellent unit: USynchScroller, that supports smoother
// synchronized scrolling.It addresses two problems.
//
// Problem 1: In programs like Calc and DoubleVision, when the main view scrolls, the side
// and/ or top views scroll by the same amount.However, each view scrolls using a separate
// ScrollRect, causing a jumpy appearance, especially when redraw time is significant.
//
// Problem 2: Packaging the synchronous scrolling functionality of (the old) Calc as a
// building block would be difficult. In addition to defining a special subclass of
// TScroller for the main view's scroller, Calc used to override methods of the side views
// themselves, as well as methods of various command objects. Solution: Combine ideas from
// both Calc and DoubleVision. USynchScroller defines a subclass of TScroller:
// TSynchScroller and two subclasses of TSynchScroller: TPrimaryScroller
// TSecondaryScroller
//
// A primary scroller can control a list of secondary scrollers.The scroll bars belong
// exclusively to the primary scroller. The extent and scrollLimit of a secondary scroller
// must match those of the primary scroller in the pertinent axis.There are other
// constraints as well-- most or all of which are obvious--that are documented herein. The
// unit overrides the following methods of TScroller:
//  In TSynchScroller:
//  SetScrollLimits
//  In TPrimaryScroller and TSecondaryScroller:
//  DoScroll
//  Fields
//  In TPrimaryScroller only:
//  ScrollDraw
//
// It also implements a method, TPrimaryScroller.AddSecondaryScroller, to connect the
// primary scroller bidirectionally with each of its secondary scrollers. Once connected,
// the following behavior transpires to perform scrolling, whether invoked by the scroll
// bars, function keys, or autoscrolling:
//  When addressed to the primary scroller:
//  -> TPrimaryScroller.ScrollBy
//  -> TPrimaryScroller.DoScroll
//  -> TPrimaryScroller.ScrollDraw
//  If it wants to scroll more than a view-full:
//  ForceRedraw three times;
//  Else if scrolling is unidirectional (the usual case):
//  ScrollRect once with an enlarged rectangle;
//  Else if it is bidirectional:
//  ScrollRect three times.
//  Then update.
//  When addressed to the secondary scroller:
//  -> TSecondaryScroller.ScrollBy
//  -> TSecondaryScroller.DoScroll
//  Delegates to TPrimaryScroller
//  -> TPrimaryScroller.ScrollBy (see above)
//
// One complication is that resizing (and potentially other activities) can tell all three
// scrollers to ScrollTo without redrawing. If this were allowed to happen, some state
// would be altered twice, and the secondary scrollers would fail to update properly when
// resized larger after being scrolled to the limit. All such activities bottleneck
// through SetScrollLimits.Upon reflection, we realize that SetScrollLimits has no
// business affecting any peer scroller.There is an override of it in class TSynchScroller
// to temporarily set an instance variable, fLimiting, that tells both overrides of
// DoScroll to exhibit inherited behavior only. (??? there must be a better way!!!)
//
// The unit is not very difficult to use. The following comments from USynchScroller.p
// illustrate the changes one would make to a typical program. How to use this unit:
//  In the .r file resource 'view':
//  'WIND', 'SCLP',
//  { 17, 32 }, { 200, 300 }, sizeRelSuperView, sizeRelSuperView, shown, enabled,
//  Scroller {
//  "TPrimaryScroller",
//  VertScrollBar, HorzScrollBar,
//  0, 0, 16, 16,
//  VertConstrain, HorzConstrain,
//  {-16, -16, 0, 0 }},// offset more to make bars even longer
//  'WIND', 'SCLV',
//  { 0, 32 }, { 17, 300 }, sizeFixed, sizeRelSuperView, shown, enabled,
//  Scroller {
//  "TSecondaryScroller",
//  noVertScrollBar, noHorzScrollBar,
//  0, 0, 16, 0,
//  noVertConstrain, noHorzConstrain,
//  noInset },
//  'WIND', 'SCLH',
//  { 17, 0 }, { 200, 32 }, sizeRelSuperView, sizeFixed, shown, enabled,
//  Scroller {
//  "TSecondaryScroller",
//  noVertScrollBar, noHorzScrollBar,
//  0, 0, 0, 16,
//  noVertConstrain, noHorzConstrain,
//  noInset },
//  In the U--.p file:
//  USES , USynchScroller
//  In the U--.inc1.p file DoMakeViews:
//  VAR
//  aMainScroller:TPrimaryScroller;
//  aLeftScroller:TSecondaryScroller;
//  aTopScroller:TSecondaryScroller;
//  aMainScroller := TPrimaryScroller(aMainView.GetScroller(TRUE));
//  aLeftScroller := TSecondaryScroller(aLeftView.GetScroller(TRUE));
//  aTopScroller:= TSecondaryScroller(aTopView.GetScroller(TRUE));
//  aMainScroller.Controls(aLeftScroller, aTopScroller);
//  In the M--.p file:
//  USES , USynchScroller
//  InitUSynchScroller;
//  Code is generated in the following code segments:
//  AInit
//  ARes
//  AFields
//----------------------------------------------------------------------------------------


#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __USCROLLER__
#include <UScroller.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kVDependent = 1;					// Dependent scroller is dependent on
                                                // vertical translation 

const short kNotVDependent = 0;					// Dependent scroller is NOT dependent on
                                                // vertical translation 

const short kHDependent = 1;					// Dependent scroller is dependent on
                                                // horizontal translation 

const short kNotHDependent = 0;					// Dependent scroller is NOT dependent on
                                                // horizontal translation 

//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TDocument;
class TSecondaryScroller;
class TList;


//----------------------------------------------------------------------------------------
// TSyncScroller: The common superclass of TPrimaryScroller and TSecondaryScroller. Merely
// overrides SetScrollLimits to assure that resizing is reliable. 
//----------------------------------------------------------------------------------------

class TSynchScroller : public TScroller
{
public:
	Boolean fLimiting;							// Pushed and popped by SetScrollLimits to
                                                // tell DoScroll not to affect peer
                                                // scrollers 

	virtual pascal void Initialize();
		// Puts a newly created TPrimaryScroller into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperview,
							 Ptr& itsParams);
		// Initialize from templates 

	virtual pascal void SetScrollLimits(const VPoint& scrollLimit,
										Boolean drawScrollBars);
		// Avoid affecting any peer scroller 

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TPrimaryScroller: A TPrimaryScroller has a list of secondary scrollers (if non-NIL):
// When it scrolls it makes the scrollers on the list scroll too in the very same
// ScrollRect toolbox call if possible. If some of the secondary scrollers can't be
// scrolled in the same ScrollRect call because they only scroll in one direction then
// they will be scrolled separately as necessary. 
//----------------------------------------------------------------------------------------

class TPrimaryScroller : public TSynchScroller
{
public:
	TList* fSecondaryScrollers;					// The list of secondary scrollers, if any


	virtual pascal void Initialize();
		// Puts a newly created TPrimaryScroller into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperview,
							 Ptr& itsParams);
		// Initialize from templates 

	virtual pascal void Free();
		// Frees its list 

	virtual pascal void AddSecondaryScroller(TSecondaryScroller* itsSecondaryScroller,
											 VCoordinate itsHDependency,
											 VCoordinate itsVDependency);
		// Become associated with the secondary scroller(s). The V and H dependencies
        // could someday be scale factors but for now are kVDependent and kHDependent for
        // now. 

	virtual pascal void RemoveSecondaryScroller(TSecondaryScroller* itsSecondaryScroller);
		// Remove the secondary scroller. 

	virtual pascal void DoScroll(const VPoint& delta, Boolean redraw);
		// Assure that fTranslation is always updated in the secondary scrollers 

	virtual pascal void ScrollDraw(const VPoint& delta, Boolean invalidate);
		// Scroll the secondary scrollers as well (unless called via SetScrollLimits) 

	virtual pascal void Fields(TObject* obj);

};


//----------------------------------------------------------------------------------------
// TSecondaryScroller: A TSecondaryScroller is controlled by a TPrimaryScroller (see
// above). When it is told to scroll, it delegates the job to the TPrimaryScroller.
// Constraints (none are checked by this unit at this time): 1. It and its subview must
// have the same length in the desired synch-scroll directions. 2. Commands for dragging
// within it must have it as their fScroller. 3. No scroll bars! 
//----------------------------------------------------------------------------------------

class TSecondaryScroller : public TSynchScroller
{
public:
	TPrimaryScroller* fPrimaryScroller;			// the primary scroller 

	Point fDeltaFactor;							// kVDependent and kHDependent (or not) 

	virtual pascal void Initialize();
		// Puts a newly created TSecondaryScroller into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void DoScroll(const VPoint& delta, Boolean redraw);
		// Polarize and delegate scroll attempts to the primary scroller (unless called
        // via SetScrollLimits) 

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern pascal void InitUSynchScroller();


#endif


