// UTECommands.p
// Copyright  1986-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UTECOMMANDS__
#define __UTECOMMANDS__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

#ifndef __UTEVIEW__
#include <UTEVIEW.h>
#endif


//----------------------------------------------------------------------------------------
// TTECommand: A command that adds characters to, or delete characters from, a TEView 
//----------------------------------------------------------------------------------------

class TTECommand : public TCommand
{
public:
	TTEView* fTEView;							// The TEView operated on 

	TEHandle fHTE;								// same as fTEView's fHTE; duplicated for
                                                // code efficiency 

	short fOldStart;							// The beginning position of the selection
                                                // at the moment just before the command
                                                // was done. 

	short fOldEnd;								// The end position of the selection at
                                                // the moment just before the command was
                                                // done. 

	Handle fOldText;							// If fOldStart = fOldEnd, i.e., if old
                                                // selection had been an insertion point,
                                                // this will be NIL. Otherwise, provides a
                                                // temporary home for the characters
                                                // comprising the old selection 

	StScrpHandle fOldStyles;

	short fNewStart;							// The beginning location in the Text of
                                                // the new text that is added by the
                                                // command, if any. 

	short fNewEnd;								// The ending location in the Text of the
                                                // new text that is added by the command,
                                                // if any. 

	Handle fNewText;							// A Handle to the characters added by the
                                                // command 

	StScrpHandle fNewStyles;

	Handle fPadding;							// Handle to fill size between new and
                                                // old. This insures that we can always
                                                // undo and redo. 

	short fTextPad;								// Size difference between New and Old
                                                // text 

	long fStylePad;								// Size difference between New and Old
                                                // styles 


	//------------------------------------------------------------------------------------
	// Initialize and Free 
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TTECommand into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED. 

	virtual pascal void ITECommand(TTEView* itsTEView,
								   CmdNumber itsCmdNumber,
								   Boolean itsSaveText);
		// Initialize the command; if unsuccessful, exit is via Failure mechanism. 

	virtual pascal void Free();
		// Free the text handles holding information needed for Undo/ Redo, then Frees SELF


	//------------------------------------------------------------------------------------
	// Command execution phase overrides 
	//------------------------------------------------------------------------------------

	virtual pascal void DoIt();
		// Focuses, then calls DoMainFunction 

	virtual pascal void RedoIt();
		// Focuses, calls RestoreSelection to get selection right, then calls
        // DoMainFunction to reinstate the changes done in DoIt which were undone by a
        // preceding UndoIt 

	virtual pascal void UndoIt();
		// Focuses, then dispatches to RemoveAdditions, ReviveDeletions, and
        // RestoreSelection to accomplish the Undo 


	//----------------------------------------------------------------------------------------
	// Command execution - Restoration of Selection -- called in both UNDO and REDO phases
	//----------------------------------------------------------------------------------------

	virtual pascal void RestoreSelection();
		// Set the Selection to be what it was just before the DO phase of the command was
        // performed 


	//------------------------------------------------------------------------------------
	// Command execution - steps in DO and REDO phases 
	//------------------------------------------------------------------------------------

	virtual pascal void BanishOldText();
		// Remove text that was selected at the outset of the Do phase 

	virtual pascal void InstallNewText();
		// Install the new text 

	virtual pascal void DoMainFunction();
		// Forwards to the relevant methods for Do and Redo phases 


	//------------------------------------------------------------------------------------
	// Command execution - steps in UNDO phase 
	//------------------------------------------------------------------------------------

	virtual pascal void RemoveAdditions();
		// Remove any characters which were added by the DO phase of the command 

	virtual pascal void ReviveDeletions();
		// Bring back the characters which were removed during the DO phase 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// TTECutCopyCommand: Command for handling Cut & Copy for the TEView.
//----------------------------------------------------------------------------------------

class TTECutCopyCommand : public TTECommand
{
public:
	Boolean fClipCreated;						// Clipboard view created OK 


	//------------------------------------------------------------------------------------
	// Creation and Destruction 
	//------------------------------------------------------------------------------------

	virtual pascal void ITECutCopyCommand(TTEView* itsTEView, CmdNumber itsCmdNumber);


	//------------------------------------------------------------------------------------
	// Initializes the command 
	//------------------------------------------------------------------------------------

	virtual pascal void Free();
		// Free the command 


	//------------------------------------------------------------------------------------
	// Command execution 
	//------------------------------------------------------------------------------------

	virtual pascal void DoIt();
		// Launches a TEView for installation in the Clipboard, then calls DoMainFunction
        // 

	virtual pascal void ReviveDeletions();
		// Bring back the characters which were removed during the DO phase; called during
        // UNDO phase 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// TTEPasteCommand: Command that handles Pasting for the TEView. 
//----------------------------------------------------------------------------------------

class TTEPasteCommand : public TTECommand
{
public:
	//------------------------------------------------------------------------------------
	// Initialization 
	//------------------------------------------------------------------------------------

	virtual pascal void ITEPasteCommand(TTEView* itsTEView);
		// Initialize the command; if unsuccessful, signalled by Failure mechanism 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
	// Used by the Inspector and the Debugger to display the contents of this class's
    // fields. 
};


//----------------------------------------------------------------------------------------
// TTEStyleCommand: Command for applying a new style to the TEView.
//----------------------------------------------------------------------------------------
class TTEStyleCommand : public TTECommand
{
public:
	//------------------------------------------------------------------------------------
	// These two fields are only used in non-styled TextEdit records 
	//------------------------------------------------------------------------------------

	short fMode;								// Mode for style change 

	TextStyle fOldTextStyle;					// The original text style 

	TextStyle fNewTextStyle;					// What we're replacing it with 


	//------------------------------------------------------------------------------------
	// Initialization 
	//------------------------------------------------------------------------------------

	virtual pascal void ITEStyleCommand(TTEView* itsTEView,
										const TextStyle& itsNewStyle,
										CmdNumber itsCmdNumber,
										short itsMode);
		// Initialize the command; if unsuccessful, signalled by Failure mechanism 


	//------------------------------------------------------------------------------------
	// Command execution 
	//------------------------------------------------------------------------------------

	virtual pascal void InstallOneStyle(const TextStyle& newStyl);
		// Installs one style over entire record - for use with old TextEdit 

	virtual pascal void InstallManyStyles(StScrpHandle newStyls);
		// Installs styles over selection range - for use with styled TextEdit 

	virtual pascal void DoIt();
		// Applies style to selection range 

	virtual pascal void UndoIt();
		// Undoes the action performed by DoIt. 

	virtual pascal void RedoIt();
		// Calls DoIt. 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// TTETypingCommand: The command that handles typing in a TEView.
//----------------------------------------------------------------------------------------

class TTETypingCommand : public TTECommand
{
public:
	Boolean fCompleted;							// Indicates whether further keystrokes
                                                // will be extensions to this command
                                                // (FALSE) or whether the command has
                                                // already been completed (TRUE) 

	short fFirstChar;							// First character typed 
	

	//------------------------------------------------------------------------------------
	// Creation and destruction 
	//------------------------------------------------------------------------------------

	virtual pascal void ITETypingCommand(TTEView* itsTEView, short itsFirstChar);
		// Initialize the command; if not successful, exit is via Failure mechanism 

	virtual pascal void Free();
		// Deallocate the command and dependent structures 


	//------------------------------------------------------------------------------------
	// Command processing 
	//------------------------------------------------------------------------------------

	virtual pascal void DoNormalChar(short aChar);
		// Handle any typed character except a backspace 

	virtual pascal void BkSpcLeft(Handle theText, short curStart);
		// Handle backspace to the left of the original selection 

	virtual pascal void BkSpcRight(Handle theText, short curStart);
		// Handle backspace to the right of the original selection 

	virtual pascal void FwdDelete(Handle theText, short curStart, short curEnd);


	//------------------------------------------------------------------------------------
	// Handle forward delete 
	//------------------------------------------------------------------------------------

	virtual pascal void AddCharacter(short aChar);
		// Add another character to an already-launched TETypingCommand 

	virtual pascal void DoIt();
		// First processing for command; adds first character 

	virtual pascal void RedoIt();
		// Handles undoing forward backspace 

	virtual pascal void UndoIt();
		// Process undo typing 

	virtual pascal void CompleteTyping();
		// Mark as no more typing allowed and fix up style scrap, if any 


	//------------------------------------------------------------------------------------
	// Debugging 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

pascal void DumpTTECommand(TTECommand* theTTECommand);
	// writes TTECommand info out to Debug window 


#endif

