////////////////////////////////////////////////////////////////////////////
//	UViewBehavior.h
//
// Contains:		Header file for View Behavior abstract class
//
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.
//
//	Change History:
//
//			3/1/91	rtm	Initial creation of file
//			3/5/91	rtm	Added comments
//
////////////////////////////////////////////////////////////////////////////
/*

	VIEW BEHAVIOR OVERVIEW
	_______________________________________________________________________________________
	
	TViewBehavior is an abstract class that adds additonal event handling to
	standard TBehavior abstract class.  This additional filtering is view specific
	and this class should only be use to create behaviors that will be used
	with views and their sub-classes.
	
	View behaviors add support for the handling of mousedown, drawing and cursor
	manipulation from behaviors.  Additional support is provided in TViewBehavior
	for the behavior to be in an active state.  This means that when it is active
	it is managing the view that owns it.
	
	 See the description of Behaviors in the UBehavior.h header file for more
	information about behaviors in general 
	

	OTHER POINTS OF INTEREST
	_______________________________________________________________________________________

	The following methods of TView have been modified to provide support for
	behaviors., these methods will be listed here in the hope that should you 
	end up overriding one of them you will attempt to maintain the connection 
	to behaviors ( unless of course your intent is to disable behaviors!! )
	
		MODIFIED EXISTING MACAPP METHODS:
		
				TView::HandleMouseDown
				TView::DrawContents
				TView::HandleCursor


	SUMMARY
	_______________________________________________________________________________________

	View behaviors add additional functionality to support view specific event
	handling from behaviors.  TViewBehavior should be used as the abstract class
	for all behaviors to attached to views and their sub-classes.
	
	
*/

#ifndef __UVIEWBEHAVIOR__
#define __UVIEWBEHAVIOR__

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UBehavior__
#include <UBehavior.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif


//	
//		TViewBehavior								
//	

// This sub-class of TBehavior is used as the abstract class for view specific Behaviors
// and should be subclassed. It adds view specific information to the behavior such as the
// ability to handle of mousedown events and drawing. This class also adds the notion of
// the behavior being active and therefore adds a new field "fActive" and its supporting
// methods 

class	TViewBehavior:	public TBehavior 
{
public:
	Boolean fActive;							// Used to indicate whether the behavior
                                                // is active or not

		
	virtual pascal void Initialize ( void );	//	OVERRIDE
		// Overridden so that we can initialize our field 

	virtual pascal void IViewBehavior( void );
		// Initialization method 
			
	virtual pascal void ActivateViewBehavior ( Boolean activate );
		// Called to activate a view behavior, this method would normally be overridden if
        // any additional functionality is required when activating a view behavior 
		
	virtual pascal TView* GetOwningView ( void );
		// A convenience method that returns the owning EvtHandler as a view 
		
	virtual pascal Boolean IsBehaviorActive ( void );
		// Returns the current state of the 'fActive' flag 
		
	virtual pascal Boolean DoMouseCommand(VPoint& theMouse,
										  TToolboxEvent* event,
										  Point hysteresis);
		// Allow view behaviors to intercept mouse commands, if there is a next behavior
        // it passes the message on down the chain otherwise by default returns false 
		
	virtual pascal void Draw ( const VRect& area );
		// Called during a views drawing process, after the view has been called to draw
        // itself and before the views subviews are called to draw themselves. The 'area'
        // in need of drawing is passed in. 
		
	virtual pascal Boolean DoSetCursor(const VPoint& localPoint, RgnHandle cursorRgn);
		// Gives the view behavior the opportunity to setup the cursor, if there is a nex
        // behavior it passes the message on down the chain otherwise by default returns
        // false 
		
};	//	<-- TViewBehavior
	
	
#endif __UVIEWBEHAVIOR__