(*
////////////////////////////////////////////////////////////////////////////
//	UBehavior.p
//
// Contains:		Interface file for Behavior abstract class
//
//	Written by:	Robin Mair
//
//	Change History:
//
//			3/4/91		rtm		Initial creation of file
//			
////////////////////////////////////////////////////////////////////////////
*)

(*

	BEHAVIOR OVERVIEW
	_______________________________________________________________________________________
	
	Behaviors provide a mechanism for modify the behavior of any EvtHandler
	object.  This is accomplished by a simple filtering mechanism whereby
	the behavior gets to look at certain events as they occur.  The behavior
	can then choose to handle the event and return control to MacApp or
	prevent control from being passed on down the line.
	
	TBehavior is the abstract class from which standard behaviors will be
	derived.  It also serves as the superclass for TViewBehavior which is the
	abstract class from which behaviors that are attached to views are derived.
	View behaviors add additional event filtering that is specific to views and 
	should only be attached to views and their subclasses.  TViewBehavior is
	likely to be the most commonly subclassed behavior as behavior provide the
	greatest flexibility in modifying the behavior of views.
	
	Multiple behaviors can be attached to objects.  A doubly linked chain of
	behaviors is created if there are more than one.
	
	BEHAVIOR USAGE
	_______________________________________________________________________________________
	
	To implement a behavior one would write the following code fragment :
	
		VAR
			aMisBehavior:	TMisBehavior;
			
			.
			.
			.
			
			{  Create the behavior		}
			NEW ( aMisBehavior );
			aMisBehavior.IViewBehavior ( viewToAttachToo, NIL );
			.
			.
			.
			
	then attach the behavior to the appropriate view:
			
			.
			.
			{  Attach behavior to view	}
			viewToAttachToo.SetBehavior ( aMisBehavior );
			.
			.
			
	thats it, now when this view gets any event that the behavior is 
	interested in it will be passed to the behavior first.  If the
	behavior handles the event it informs its owner by return 'true'
	which causes the normal view mechanism for that event to be by
	passed.
	
	EXAMPLE
	_______________________________________________________________________________________
	
	As an example of where behaviors could be used with MacApp to great
	advantage lets look at how we could use behaviors to implement a
	view manipulation tool.
	
	What we would like to be able to do is allow the user to create views
	in our tool and then allow them to be re-located or resized.  Typically
	to achieve this we would have to write some "wrapping" mechanism around 
	the standard MacApp view objects.  The purpose of this mechanism would
	be to filter events that are passed to the view in case we need to 
	intercept them so we can implement the editing features.  This means a 
	lot of wasted effort writing these "wrappers".
	
	Enter a behavior, by capturing the view manipulation capabilities ( or
	certainly the points from which manipulation commands can be dispatched )
	in a behavior.  We can then attach that behavior to any view that we would
	like to be able to manipulate.  Now because there is a behavior attached 
	to the view it will get a look at certain events allowing it to modify the
	views behavior when appropriate.  Because the field that behaviors are
	attached to has been added to TEvtHandler all existing MacApp classes that
	are descendants of TEvtHandler will be able to take advantage of this 
	feature.
	
	Eventually MacApp will provide some behaviors that people can use, in particular
	one that can be used to perform the operations associated with manipulating
	views.
	
	Now you to can construct your own View editing tool using this built in MacApp
	mechanism!!!!!!
	
	SUMMARY
	_______________________________________________________________________________________

	In summary, behaviors provide a mechanism for capturing common behavior in an
	object that can then be attached to other objects to extend their behavior without
	requiring that they be sub-classed.  This allows existing objects to stay lean
	and mean but be extended when required.  In many cases one can avoid having to 
	create a sub-class of an object by instead using a behavior to provide the required
	extensions.  This is especially useful in the case where this behavior is used
	in a number of different objects.
	
	One very important point that has not been mentioned is the fact that one could 
	very easily support the addition of behavior to objects - dynamically - within
	your application.
	
	Eventually a mechanism will also be provided for creating behaviors from resources
	and for adding them to views from resources.
	
	We feel that people will find many uses for behaviors and we intend to use them
	ourselves for future MacApp tools, as always we are interested in getting your
	feedback on how you have used behaviors and what can be done to make them even
	more powerful!!!!
	
	Go forth and multiply!!!!!
			
*)


UNIT UBehavior;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, Events, AppleEvents, UEvtHandler, 
			Balloons, UEvent, UView;

		TYPE

			TEvtHandler			= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;
			TEvent				= OBJECT; EXTERNAL;

			{}
			{	TBehavior												}
			{}

			TBehavior			= OBJECT (TObject)
			(*
			
				 This is the abstract class for the general Behaviors that can be attached to any
				 TEvtHandler.  This must be subclassed to do anything useful.  It will also be 
				 subclassed to create a specific abstract class from which view behaviors will 
				 be sub-classed.
				 
			*)
				fOwner:			TEvtHandler;	{ The event handler that owns this behavior	}
				fPreviousBehavior:	TBehavior;		{ The previous behavior in the chain 			}
				fNextBehavior:	TBehavior;		{ The next behavior in the chain					}
				
				PROCEDURE TBehavior.Initialize; OVERRIDE;
					{ Overridden so that we can initialize our fields }
					
				PROCEDURE TBehavior.IBehavior;
					{ Initialization method	}	
					
				FUNCTION TBehavior.Clone :TObject; OVERRIDE;
					{ This has been overridden to support deep cloning of behaviors }
					
				PROCEDURE TBehavior.Free; OVERRIDE;
					{ Pass the Free message on to all behaviors in the chain
		 			before calling inherited Free	}
					
				PROCEDURE TBehavior.SetNextBehavior ( nextBehavior: TBehavior );
					{ Set the fNextBehavior field if it is NIL, otherwise calls fNextBehavior.SetNextBehavior	}

				PROCEDURE TBehavior.SetPreviousBehavior ( previousBehavior: TBehavior );
					{ Set the fPreviousBehavior field	}
					
				FUNCTION TBehavior.GetNextBehavior: TBehavior;
					{ Return the next behavior	}

				PROCEDURE TBehavior.SetOwner ( itsOwner: TEvtHandler );
					{ Set the fOwner field }
					
				PROCEDURE TBehavior.InsertBefore ( 	insertMe: TBehavior;
																before:	TBehavior );
					{ Insert behavior 'insertMe' ahead of behavior 'before' in the chain	}
					
				PROCEDURE TBehavior.InsertAfter (	insertMe: TBehavior;
																after: TBehavior );
					{ Insert behavior 'insertMe' after of behavior 'after' in the chain }					
					
				PROCEDURE TBehavior.Remove ( removeMe: TBehavior );
					{ Remove behavior 'removeMe' from the chain	}
					
				PROCEDURE TBehavior.DoAppleCommand (	aCmdNumber: CmdNumber;
																		message: AppleEvent;
																		reply: AppleEvent );
					{ Allows behaviors to respond to AppleEvents, if there is a next
					behavior it passes the message on down the change otherwise it calls
					the corresponding method of its Owner	}		
					
				PROCEDURE TBehavior.DoCommandKey (	event: TToolboxEvent );
					{ Allows behaviors to respond to CommandKey events, if there is a next
					behavior it passes the message on down the change otherwise it calls
					the corresponding method of its Owner	}
					
				PROCEDURE TBehavior.DoEvent ( 	eventNumber: EvtNumber;
														source: TEvtHandler;
														event: TEvent );
					{ Allows behaviors to handle events, if there is a next
					behavior it passes the message on down the change otherwise it calls
					the corresponding method of its Owner	}							
					
				PROCEDURE TBehavior.DoKeyCommand (	event: TToolboxEvent );
					{ Allows behaviors to respond to KeyCommands, if there is a next
					behavior it passes the message on down the change otherwise it calls
					the corresponding method of its Owner	}
					
				PROCEDURE TBehavior.DoMenuCommand ( aCmdNumber: CmdNumber );
					{ Allows behaviors to handle menu events, if there is a next 
					behavior passes the message on down the chain otherwise it calls
					the corresponding method of its Owner	}
					
				PROCEDURE TBehavior.DoSetupMenus;
					{ Allows behaviors to handle menu setup if there is a next 
					behavior passes the message on down the chain otherwise it calls
					the corresponding method of its Owner		}
					
					
			END;

END.