{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UBusyCursor.p }
{ Copyright  1985-1991 Apple Computer, Inc.	All rights reserved. }
{[f-]}
{
			T H E O R Y   O F	 O P E R A T I O N

The UBusyCursor unit implements a mechanism for automatically setting
the cursor to the busy cursor when the application is "busy."

An application is considered "busy" if it hasn't called GetNextEvent
or EventAvail for a given number of ticks, where 60 ticks equals one
second.  The default time is defined by kBusyDelay, which is 120 ticks
or 2 seconds--it can be changed by calling BusyDelay.	After this period
of time has elapsed with no call to GetNextEvent or EventAvail, the
cursor is changed to the busy cursor.  On the next call to GetNextEvent or
EventAvail, the cursor is restored to its state before it was changed to
the busy cursor.

Changing the cursor to the busy cursor is done by the VBL task ABusyTask.
ABusyTask's execution frequency is set to kBusyDelay, or by calling
BusyDelay.	When ABusyTask is executed it sets the cursor to the
busy cursor and resets ABusyTask's vblCount.

The traps InitCursor, SetCursor and SetCCursor are patched so that
they "remember" the cursor being set before executing the trap.  This
will allow us to restore the cursor after it has been changed to the
busy cursor.

The EventAvail and GetNextEvent traps are patched such that, before
executing the trap, ABusyTask's vblCount is reset, and if the busy
cursor is displayed, then the cursor is restored to the last cursor
set by SetCursor or SetCCursor.


Manual vs. Automatic Animation

Now, there are two philosophies as to how to make a busy cursor animate: automatic and
manual. The automatic technique requires no intervention by the application, and is
essentially an extension of the current MacApp busy cursor philosophy. One way of
implementing automated animation is by installing a VBL task which changes the cursor at
appropriate time intervals, without the applications knowledge. The advantage of this
approach, of course, is that you dont have to do anything in your application to make it
work, plus you get a cursor that animates in a smooth manner. The drawback is the
possibility of a run away cursor. By using a VBL task the watch may spin even though the
application has locked up, say in an endless loop, and isnt actually doing anything. This
is because the VBL task has no idea what the application is doing. This happened to me (Curt
Bianchi) once with a well-known word processor. I tried to save a file and the word
processor locked up for hours, but the watch kept spinning. Eventually I gave up and pressed
the program interrupt switch on the side of the computer. Since I didnt have a debugger
installed, the bomb alert came up, and all the while the hands kept spinning, oblivious to
what was happening!

With the manual technique the application tells the cursor when to animate. This is done by
sprinkling cursor animation calls throughout your code, which can be a real pain. You also
wind up with jumpy or irregular animation. However, it does prevent the run away cursor
problem. Both the Finder (6.0) and MPW (3.2) use this technique.

It turns out that theres a simple way to combine the advantages of both approaches, which
Ill use here. That is, you can get smooth animation and avoid the run away
cursor problem. Its done by animating the cursor as long as the application lets us know
its still working. If the application fails to check in, then the animation times out.
You can choose between automatic and manual animation by setting the length of time before
time out occurs. A very large time out value, say MaxLongInt which for our purposes is
infinity, means the application never has to check in and we get automatic cursor animation.
A small time out value, say a few minutes, forces the application to check in from time to
time, or the cursor quits animating after the period of time has elapsed.

To implement this scheme, we need a way for the application to check in, and a way for the
application to set the period of time after which the animation times out. To set the period
of time well add the routine SetBusyTimeout, which accepts a value measured in ticks, or
sixtieths (1/60) of a second. For example, five minutes is 18000 ticks: 5 * 60 * 60 = 18000.
Well also implement the routine BusyAnimate, which the application calls to notify us that
its still working. Provided BusyAnimate is called more often than the time out value, the
cursor continues to animate smoothly. So, if you pass 18000 (5 minutes) to SetBusyTimeout,
the busy cursor will continue to animate as long as BusyAnimate is called at least every
five minutes, or until the application attempts to handle another event.
}
{[f+]}
UNIT UBusyCursor;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			QuickDraw;

		CONST
			kBusyDelay			= 2 * 60;				{default # of 1/60 sec. ticks before cursor
														 changes to a busy cursor }
			kAnimateDelay		= 10;					{ Animate every 1/6 of a second. }
			kTicksBeforeTimeout = MAXLONGINT;			{ Default is to never time out }

		TYPE
			{ These types are used to represent the acur resources }
			AcurRsrcRecord		= RECORD
				noOfCursors:		INTEGER;
				whichCursor:		INTEGER;
								{ The following is actually a variable length array }
				cursors:			ARRAY [1..1] OF
										RECORD
									CASE BOOLEAN OF
										TRUE:
											(rsrcId:			 INTEGER;
											 filler:			 INTEGER);
										FALSE:
											(h: 				 CursHandle);
									END;
				END;
			AcurRsrcPtr 		= ^AcurRsrcRecord;
			AcurRsrcHandle		= ^AcurRsrcPtr;

			TBusyCursor 		= OBJECT (TObject)
				PROCEDURE TBusyCursor.IBusyCursor;
				{ IBusyCursor installs the busy cursor mechanism.	Typically this is
				called once during program initialization.	It installs the
				VBL task and patches the traps }

				PROCEDURE TBusyCursor.Free; OVERRIDE;
				{ Free uninstalls the busy cursor mechanism.	Typically this is
				called once during program termination.  It removes the VBL task
				and unpatches the patched traps. }

				FUNCTION TBusyCursor.InitAnimatedCursor(acurRsrcId: INTEGER): AcurRsrcHandle;
				{ Loads the acur resource whose id is acurRsrcId, and then loads the CURS
				resources referred to by the acur resource.  All of these resources must be
				locked in memory since they are accessed at VBL interrupt time.  We also detach
				them from the resource map to prevent the Resource Mgr from unlocking them out
				from under us. }

				PROCEDURE TBusyCursor.Activate(entering: BOOLEAN);
				{ Call BusyActivate if you want to activate or deactivate the busy cursor mechanism. This is
				call by MacApp when losing/gaining control to a desk accessory or another application. }

				PROCEDURE TBusyCursor.SetDelay(newDelay: INTEGER);
				{ Call SetDelay if you want to change the busy cursor delay.
				newDelay should be in 1/60 seconds; a value <= 0 means don't change the
				(SetDelay respects the state flags in the cursor info record (ie.,
				changeToBusy and inControl.) }

				PROCEDURE TBusyCursor.ForceBusy;
				{ Puts up the busy cursor immediately }

				PROCEDURE TBusyCursor.Reset(delayTicks: INTEGER);
				{ Call Reset if you want to change the cursor back to an arrow
				and reset the time before changing back to a busy cursor.  This is not
				usually called directly by the application.  Instead, it
				is called each time GetNextEvent and EventAvail is called, by
				patches installed by IBusyCursor. }

				PROCEDURE TBusyCursor.TurnOff;
				{ This is called from InitMacAppCursor, SetMacAppCursor and SetCMacAppCursor.
				It sets pCursorInfo fields to indicate that the cursor is not the
				busy cursor. }

				PROCEDURE TBusyCursor.Animate;
				{ call this to prevent the animation from timing out. }

				PROCEDURE TBusyCursor.SetTimeout(ticksBeforeTimeout: LONGINT);
				{ call this to set the length of time before animation times out. }
				END;

		VAR
			{$Push} 									{$J+}
			gBusyCursor:		TBusyCursor;			{ THE busycursor handler }
			{$Pop}

		FUNCTION NextAnimatedCursor(acurRsrc: AcurRsrcHandle): CursHandle;
		{ This routine will return the next cursor from the animation list }

		PROCEDURE InitUBusyCursor;
		{ Inits the unit.  Creates the global gBusyCursor }
END.
