{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UList.p }
{ Copyright  1984-1991 by Apple Computer Inc. All rights reserved. }

{ Unit UList defines object types TList, a simple dynamic array whose elements are references to
  TObjects, and TSortedList, which maintains the object references in a sequence defined by
  overriding TSortedList.Compare.

	Peter Gaston's scheme for chunky memory allocation has been used by permission.
	Thanks Peter!.

TList:
	TList is an object which provides a very nice dynamic array of objects.
	It is implemented by appending the dynamic array onto the end of the
	object.  Growing and shrinking the array is done by growing and shrinking
	the handle which holds the object.

TSortedList:
	TSortedList maintains the object references in a sequence defined by
	overriding TSortedList.Compare


Some Specific uses that TList is especially good for.

	LIFO Stack:
		A push/pop operation is provided.

	FIFO Stack: (Queue)
		Use InsertLast for Queue and First for Dequeue.

	Pre-allocation of your list:
		If you know the size that your list will eventually reach, then you can
		pre-set it to the proper size.

	Delete by Index:
		If you keep track of items by index number, then you can avoid
		a linear search when you want to delete it. Remember though, it
		is a dynamic list and any insertion/deletion activity can cause
		the index to point at a different element than you expect.

Caveats:
	A _small_ amount of memory is always unused with this method.  An active
	list will generally waste one-half of the chunk size, or
	4*2=8 bytes/TList.
}

UNIT UList;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject;

		CONST
			kAllocationIncrement = 6;					{ Initial Allocation increment. Six
														 elements of slop shouldn't break anybody
														 and provides a _nice_ cushion from the
														 memory manager. }
			kIterateForward 	= TRUE;
			kIterateBackward	= NOT kIterateForward;

			kEmptyIndex 		= 0;					{ Value to use when no valid index is
														 available Indexes are always positive }

			{ Constants for TDynamicArray.CompareElements and TSortedxxx.Compare }
			kItem1LessThanItem2 = - 1;
			kItem1EqualItem2	= 0;
			kItem1GreaterThanItem2 = 1;

			{ Constants for TSortedList.Search.
			The criteria is considered to be item 2}
			kItemGreaterThanCriteria = kItem1LessThanItem2;
			kItemEqualCriteria	= kItem1EqualItem2;
			kItemLessThanCriteria = kItem1GreaterThanItem2;

		TYPE

			ArrayIndex			= kEmptyIndex..MaxLongint; { At least always positive ( in this
															universe ) !!! kEmptyIndex..MaxLong
															would be a nice enhancement }
			CompareResult		= integer;				{ Negative, zero, and positive results are
														 all meaningful even though we have the
														 nice constants above. }
														 
			TDynamicArray		= OBJECT (TObject)
			
				fIteratorPtr:		Ptr;				{ pointer to the first iterator in use }
				

				fSize:				ArrayIndex; 		{ number of elements ACTUALLY in the array,
														 from 0 to the limit of memory}
				fElementSize:		integer;			{ Size in bytes of an element. MUST be a
														 power of 2 ie. 1, 2, 4, 8, 16, etc. }
				fElementSizeShift:	integer;			{ the power of 2 for the element size. Will
														 be used to avoid DIV and MUL }

				fAllocationIncrement: ArrayIndex;		{ Number of elements by which to increase of
														 decrease the allocated size of the array
														 when it needs to grow or shrink. Thus
														 reducing memory manager aggravation. }
				fAllocatedSize: 	ArrayIndex; 		{ Number of elements for which storage is
														 ALLOCATED }
				fFreeRequested: 	Boolean;			{ TRUE if the Free method was called but
														 couldn't be honored because enumeration
														 was in process. Checked at end of
														 enumeration and Free is called if true }
				fClassSize:			Size;				{ Used with ComputeAddress to create a
														pointer to an element.  This breaks
														encapsulation of GetDynamicArea but, is being
														done here for performance reasons (small) }
								{Creation/Destruction Methods}

				PROCEDURE TDynamicArray.Initialize; OVERRIDE;
				{ Puts a newly created TDynamicArray into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TDynamicArray.IDynamicArray(initialSize: ArrayIndex;
													  elementSize: integer);
				{ Initialize a new array with initialSize elements, Always call it once before
				calling any other method.  Never call it twice.}

				{ Array manipulation primitives }

				FUNCTION TDynamicArray.GetSize: ArrayIndex;
				{ Returns the ACTUAL number of elements in the array.}

				PROCEDURE TDynamicArray.SetArraySize(theSize: ArrayIndex);
				{ Sets the array allocation to handle up to theSize elements }

				FUNCTION TDynamicArray.EachElementDoTil(FUNCTION
														TestElement(elementIndex: ArrayIndex):
														Boolean;
														IterateForward: Boolean): ArrayIndex;
				{ Call TestIndex once for each element of the array, in order, until TestIndex returns TRUE.
				  Return the element that satisfied the test.  If none satisfied the test, return kEmptyIndex.
				}

				FUNCTION  TDynamicArray.EachElementInRangeDoTil(lowBound : ArrayIndex;
																highBound : ArrayIndex;
																FUNCTION TestElement(elementIndex: ArrayIndex): Boolean;
																IterateForward: Boolean): ArrayIndex;
				{ The basic array iterator. Call TestIndex once for each element over a subrange the array,
				in order, until TestIndex returns TRUE.
				  Return the element that satisfied the test.  If none satisfied the test, return kEmptyIndex.
				}

				PROCEDURE TDynamicArray.Free; OVERRIDE;
				{ If enumeration of the array is in process, delete all the array elements,
				mark the fFreeRequested flag for testing at completion of the enumeration and
				return.  Otherwise really free the array.  Gee, wouldn't automatic storage
				management be great! }

				PROCEDURE TDynamicArray.InsertElementsBefore(index: ArrayIndex;
															 ElementPtr: UNIV Ptr;
															 count: ArrayIndex);
				{ Insert Elements before the indicated Element. The index of the new element
				  will be index.   If index = 1 this inserts at the start of the array. If index = fSize + 1
				  this inserts at the end of the array. Signals Failure if unable to change the size of
				  the array.

				  !!! NOTE MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT
				  yet supported! }

				PROCEDURE TDynamicArray.ReplaceElementsAt(index: ArrayIndex;
														  ElementPtr: UNIV Ptr;
														  count: ArrayIndex);
				{ Replaces the Elements at the indicated index.

				  !!! NOTE MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT
				  yet supported! }

				PROCEDURE TDynamicArray.DeleteElementsAt(index: ArrayIndex;
														 count: ArrayIndex);
				{ Deletes the Element at the indicated index.  Compresses the array

				  !!! NOTE MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT
				  yet supported! }

				PROCEDURE TDynamicArray.GetElementsAt(index: ArrayIndex;
													  ElementPtr: UNIV Ptr;
													  count: ArrayIndex);
				{ copies count elements to the location specified by ptr.

				  !!! NOTE MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT
				  yet supported! }

				FUNCTION TDynamicArray.ComputeAddress(index: ArrayIndex): Ptr;
				{ Returns a pointer to of the index-th element in the array.

				PLEASE NOTE: The return value is a direct heap pointer and should be used with
				care as the heap can compact across calls that move memory; thus invalidating
				the pointer. }

				{ Misc. functions }

				FUNCTION TDynamicArray.IsEmpty: Boolean;
				{ Test if this array is empty or not. }

				PROCEDURE TDynamicArray.Merge(aDynamicArray: TDynamicArray);
				{ merges aDynamicArray with itself. leaves aDynamicArray unchanged.

				  !!! NOTE MULTIPLE ( >1 ) element moves for non-power of 2 element sizes are NOT
				  yet supported! }

				{ Debugging Methods }

				PROCEDURE TDynamicArray.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				PROCEDURE TDynamicArray.DynamicFields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				dynamic area. }

				END;

			TSortedDynamicArray = OBJECT(TDynamicArray) { A TDynamicArray with ordering }
				PROCEDURE TSortedDynamicArray.ISortedDynamicArray(initialSize: ArrayIndex;
													  elementSize: integer);
				{ Initialize a new array with initialSize elements, Always call it once before
				calling any other method.  Never call it twice.}

				FUNCTION  TSortedDynamicArray.CompareElements(Element1,Element2 : UNIV Ptr) : CompareResult;
				{ Orders the elements of the array }
				
				PROCEDURE TSortedDynamicArray.InsertElementInOrder(ElementPtr: UNIV Ptr);
				{ Insert an element in its correct position according to the ordering defined
				by the Compare method }
				
				FUNCTION  TSortedDynamicArray.DoSearchElement(FUNCTION TestItem(anItem: ArrayIndex):
				CompareResult; VAR index : ArrayIndex): BOOLEAN;
				{ !!! Adapted from TSortedList. Performs a binary search for an element for which TestItem returns true.
				"index" returns the position of a found item, or where it should be inserted if not found }

				PROCEDURE TSortedDynamicArray.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
			
			TSortedList 		= OBJECT (TSortedDynamicArray)		{ A sorted dynamicArray of TObjects. Items are
														 sorted based on the results of Compare
														 which has to be overridden to provide a
														 comparison mechanism other than identity. It IS
														  permissible for descendants to add more
														  named fields }
				fObjClassID:		ObjClassID; 		{ if <> kNilClass then the Class ID of the
														 elements of the list }
								{Creation/Destruction Methods}

				PROCEDURE TSortedList.Initialize; OVERRIDE;
				{ Puts a newly created list into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TSortedList.ISortedList;
				{ Initialize a sorted list }

				PROCEDURE TSortedList.FreeList;
				{ Frees each object in the list, then frees the list.}

				{ Utility Methods}

				FUNCTION TSortedList.At(index: ArrayIndex): TObject;
				{ Return the index'th element of the list. It is typical for the caller to coerce the
				result into a descendant of TObject. All lists are indexed from 1. Range check only
				if the compile-flag qRangeCheck is TRUE. The static-array equivalent of:
				object := objList.At(index); is: object := objArray[index]; }

				FUNCTION TSortedList.First: TObject;
				{ Return the first element of the list. It is typical for the caller to coerce the
				result. Returns NIL if the size is <= 0. }

				FUNCTION TSortedList.Last: TObject;
				{ Return the last element of the list. It is typical for the caller to coerce the
				result. Returns NIL if the size is <= 0. }

				{Iterator Methods}

				FUNCTION TSortedList.IterateTil(FUNCTION TestItem(item: TObject): Boolean;
										  IterateForward: Boolean;
										  VAR itsIndex: ArrayIndex): TObject;
				{ The basic list iterator.	Call TestItem once for each element of the list, in order,
				until TestItem returns TRUE.
				  Return the element that satisfied the test.  If none satisfied the test, return NIL.
				  The actual parameter is typically a procedure whose argument is a descendant of TObject.
				  It is typical for the caller to coerce the result into that descendant of TObject.
				  If TestItem calls InsertLast, the newly added element will NOT be enumerated.
				  If TestItem calls AtPut, InsertBefore, InsertFirst, or DeleteAll, misbehavior will
				  ensue.  The static-array equivalent of:
					object := objList.FirstThat(Func)
				  is:
					object := NIL;
					FOR index := 1 TO fSize DO IF Func(objArray[index]) THEN
						BEGIN
						object := objArray[index];
						LEAVE;
						END;
				}

				PROCEDURE TSortedList.Each(PROCEDURE DoToItem(item: TObject));
				{ Call DoToItem once for each element of the list, in order. The actual parameter is
				typically a procedure whose argument is a descendant of TObject. If DoToItem calls
				InsertLast, the newly added element will NOT be enumerated. If TestItem calls
				AtPut, InsertBefore, InsertFirst, Delete, or DeleteAll, misbehavior will ensue. If
				DoToItem calls Delete, the item is replaced by the constant kDeletedElement; fSize
				is decremented and fDeletions is incremented. At the end of all calls to Each,
				RemoveDeletions will be called. The static-array equivalent of:
				objList.Each(Proc) is:
				FOR index := 1 TO fSize DO
					Proc(objArray[index]);
				}

				FUNCTION TSortedList.FirstThat(FUNCTION TestItem(item: TObject): Boolean): TObject;
				{ Call TestItem once for each element of the list, in order, until TestItem returns
				TRUE. Return the element that satisfied the test. If none satisfied the test,
				return NIL. The actual parameter is typically a procedure whose argument is a
				descendant of TObject. It is typical for the caller to coerce the result into that
				descendant of TObject. If TestItem calls InsertLast, the newly added element will
				NOT be enumerated. If TestItem calls AtPut, InsertBefore, InsertFirst, Delete, or
				DeleteAll, misbehavior will ensue. }

				FUNCTION TSortedList.LastThat(FUNCTION TestItem(item: TObject): Boolean): TObject;
				{ Call TestItem once for each element of the list, starting with the last item in the
				list and working toward the first item, until TestItem returns TRUE. Return the
				element that satisfied the test. If none satisfied the test, return NIL. The actual
				parameter is typically a procedure whose argument is a descendant of TObject. It is
				typical for the caller to coerce the result into that descendant of TObject. If
				TestItem calls InsertLast, the newly added element will NOT be enumerated. If
				TestItem calls AtPut, InsertBefore, InsertFirst, Delete, or DeleteAll, misbehavior
				will ensue. }

				FUNCTION TSortedList.DoSearch(FUNCTION
											  TestItem(anItem: TObject): CompareResult;
											  VAR index: ArrayIndex): TObject;
				{ Searches the sorted list until TestItem returns zero or there are no more
				  items to search.	A binary search is used. }

				FUNCTION TSortedList.GetIdentityItemNo(item: TObject): ArrayIndex;
				{ Return the index of the IDENTITY item in the list, or zero if the item is not in the
				list. }

				FUNCTION TSortedList.GetEqualItemNo(item: TObject): ArrayIndex;
				{ Return the index of any item that is considered to be equal to the parameter
				'item'. Two items are considered equal if comparing them with the Compare method
				returns zero. You may wish to use the constants kItemGreaterThanCriteria,
				kItemEqualCriteria, kItemLessThanCriteria. }

				PROCEDURE TSortedList.Insert(item: TObject);
				{ Inserts the given item into the list in sorted order, using the Compare
				  method to determine the item's location relative to other items in the list. }

				FUNCTION TSortedList.Compare(item1, item2: TObject): CompareResult;
				{ This method compares two items in the list. A negative result indicates that item1
				< item2. A result of zero indicates that item1 = item2. A positive result indicates
				that item1 > item2. You may wish to use the constants kItem1LessThanItem2,
				kItem1EqualItem2, kItem1GreaterThanItem2. By default just compare the ordinal value
				of the items. Subclasses that want to should override this method to do any other
				kind of comparison (comparing instance variables, for instance (get it?)). }

				FUNCTION TSortedList.CompareElements(Element1,Element2 : UNIV Ptr): CompareResult; OVERRIDE;
				{ Forward into the Compare function. }

				FUNCTION TSortedList.Search(FUNCTION
											TestItem(anItem: TObject): CompareResult): TObject;
				{ This method searches the list for an item that causes your supplied TestItem
				function to return kItem1EqualItem2 (0). This is useful for cases in which you are not 
				comparing objects in the list with another object, as does Compare. For example, 
				suppose each object has an fTitle field and the objects are inserted into the list 
				in order of fTitle. You can use the search method to look for an object whose fTitle
				is equal to any string. A negative TestItem result indicates that your search criteria <
				anItem, a result of zero indicates the item has been found and Search returns that
				item. A positive TestItem result indicates that your search criteria > your anItem.
				}

				PROCEDURE TSortedList.Sort;
				{ Sorts the list using  the Compare and SortBy methods.}

				PROCEDURE TSortedList.SortBy(FUNCTION
											 CompareItems(item1, item2: TObject): CompareResult);
				{ Sorts the list using the supplied CompareItems function.   Uses Shell sort (see
				Sedgewick, "Algorithms", pp. 97-9)  }

				{Item Deletion Methods}

				PROCEDURE TSortedList.AtDelete(index: ArrayIndex);
				{ Deletes via an index (rather than having to search for an item) }

				PROCEDURE TSortedList.Delete(item: TObject);
				{ Delete the first reference to item from the list, but do not free item. If item
				does not occur, do nothing. If item does occur, reduce fSize by 1.

				!!! NOTE: This name conflicts with the Pascal builtin: DELETE and we will be
				changing it's name, but changing the name at the _last minute_ isn't a good idea.
				If you need to use the builtin DELETE in a TSortedList subclass, then you will have to
				create a wrapper procedure that forwards to it, for now. Sorry the Management. }

				PROCEDURE TSortedList.DeleteAll;
			   { Delete every element from the list, but do not free any objects. Leave fSize at 0.}

				PROCEDURE TSortedList.FreeAll;
				{ Frees each element in the list.  Leave fSize at 0.}

				{ Debugging Methods }

				PROCEDURE TSortedList.SetEltType(toClass: MAName);
				{ Call this once and pass the className of objects you intend to insert into the
				list. The insertion methods will do a coercion to the type you specify, to ensure
				that the list contains only elements of the right type.}

				PROCEDURE TSortedList.SetEltTypeID(toClassID: ObjClassID);
				{ Call this once and pass the ObjClassID of objects you intend to insert into the
				list. The insertion methods will do a coercion to the type you specify, to
				ensure that the list contains only elements of the right type.}

				PROCEDURE TSortedList.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				PROCEDURE TSortedList.DynamicFields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				dynamic area. }
				PROCEDURE TSortedList.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;
				{ Used by the Inspector to display the name of this class. }

				END;

			TList				= OBJECT (TSortedList) {A dynamic list of TObjects.}
				PROCEDURE TList.IList;
				{ Initialize a new list with no elements, i.e., fSize = 0.	Always
				  call it once before calling any other method.  Never call it
				  twice.}

				{ O V E R R I D E S  T O  R E M O V E  O R D E R I N G }
				FUNCTION  TList.DoSearchElement(FUNCTION TestItem(anItem: ArrayIndex):
				CompareResult; VAR index : ArrayIndex): BOOLEAN; OVERRIDE;
				{ Overridden to do a linear search }

				PROCEDURE TList.InsertElementInOrder(ElementPtr: UNIV Ptr); OVERRIDE;
				{ Overridden to do arrival sequence insertion }
				
				{ U N O R D E R E D  I T E M  I N S E R T I O N }
				PROCEDURE TList.AtPut(index: ArrayIndex;
									  newItem: TObject);
				{ Replace the index'th item of the list. Does not free the old item. If you do this
				from within an Each the results may be unpredictable.}

				PROCEDURE TList.InsertBefore(index: ArrayIndex;
											 item: TObject);
				{ Insert a reference to an item before the indicated item. The index of the new
				element will be index. If index = 1 this inserts at the start of the list. If index
				= fSize + 1 this inserts at the end of the list. Signals Failure if unable to
				increase the size of the list.}

				PROCEDURE TList.InsertFirst(item: TObject);
				{ Insert a reference to item at the front of the list. If the compile-flag qDebug is
				TRUE & SetEltType was called, verify item's type. Increase fSize by 1. The index of
				the new element will be 1. Signals Failure if unable to increase the size of the
				list.}

				PROCEDURE TList.InsertLast(item: TObject);
				{ Insert a reference to item at the back of the list. If the compile-flag qDebug is
				TRUE & SetEltType was called, verify item's type. Increase fSize by 1. The index of
				the new element will be fSize. Signals Failure if unable to increase the size of
				the list.}

				PROCEDURE TList.Push(item: TObject);
				{ LIFO stack push.(same as insertLast) }

				FUNCTION TList.Pop: TObject;
				{ LIFO stack pop. }

				PROCEDURE TList.Queue(item: TObject);
				{ FIFO queue addition.(same as insertLast) }

				FUNCTION TList.Dequeue: TObject;
				{ FIFO queue relief }

				PROCEDURE TList.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			THandleList 		= OBJECT (TSortedDynamicArray) {A dynamic list of Handles. It IS
														  permissible for descendants to add more
														  named fields}
				PROCEDURE THandleList.IHandleList;
				{ Initialize a new list with no elements, i.e., fSize = 0 }

				FUNCTION THandleList.At(index: ArrayIndex): Handle;
				{ Return the index'th element of the list. }

				PROCEDURE THandleList.AtDelete(index: ArrayIndex);
				{ Deletes via an index. }

				PROCEDURE THandleList.Delete(item: Handle);

				PROCEDURE THandleList.DeleteAll;

				PROCEDURE THandleList.Each(PROCEDURE
										   DoToItem(item: Handle));

				FUNCTION THandleList.First: Handle;
				  { Return the first element of the list.
				   If the compile-flag qDebug is TRUE & SetEltType was called, verify item's type. }

				FUNCTION THandleList.FirstThat(FUNCTION
											   TestItem(item: Handle): Boolean): Handle;
				{ Call DoToItem once for each element of the list, in order. }

				FUNCTION THandleList.GetIdentityItemNo(item: Handle): ArrayIndex;
				 { Find the first reference to the IDENTITY item in the list.
				  If item does not occur, return 0. }

				FUNCTION THandleList.GetEqualItemNo(item: Handle): ArrayIndex;
				 { Find the first reference to item in the list.
				  If item does not occur, return 0. }

				PROCEDURE THandleList.Insert(item: Handle);
				{ Inserts the given item into the list in sorted order, using the Compare
				  method to determine the item's location relative to other items in the list. }

				FUNCTION THandleList.DoSearch(FUNCTION TestItem(anItem: Handle): CompareResult;
													 VAR index: ArrayIndex): Handle;

				FUNCTION THandleList.Compare(item1, item2: Handle): CompareResult;
				{ By default just compare the ordinal value of the items.  Subclasses that want to use the items
				as pointers or other types should override if the comparison is based on the pointed at value }
				
				FUNCTION THandleList.CompareElements(Element1,Element2 : UNIV Ptr): CompareResult; OVERRIDE;
				{ Forward into the Compare function. }

				FUNCTION THandleList.IterateTil(FUNCTION
												TestItem(item: Handle): Boolean;
												IterateForward: Boolean;
												VAR itsIndex: ArrayIndex): Handle;
				{ Call TestItem once for each element of the list, in order. }

				FUNCTION THandleList.Last: Handle;
				 { Return the last element of the list.
				   If the compile-flag qDebug is TRUE & SetEltType was called, verify item's type. }

				FUNCTION THandleList.LastThat(FUNCTION
											  TestItem(item: Handle): Boolean): Handle;
				{ Call DoToItem once for each element of the list, in order. }

				PROCEDURE THandleList.Fields(obj: TObject); OVERRIDE;

				PROCEDURE THandleList.DynamicFields(obj: TObject); OVERRIDE;
				END;

			TSortedLongintList				= OBJECT (TSortedDynamicArray) {A dynamic list of Longints. It IS
														  permissible for descendants to add more
														  named fields}
				PROCEDURE TSortedLongintList.ISortedLongintList;
				{ Initialize a new list with no elements, i.e., fSize = 0 }
				
				PROCEDURE TSortedLongintList.Insert(item: longint);
				
				FUNCTION TSortedLongintList.At(index: ArrayIndex): longint;
				{ Return the index'th element of the list. }
				
				PROCEDURE TSortedLongintList.AtDelete(index: ArrayIndex);
				{ Deletes via an index. }
				
				PROCEDURE TSortedLongintList.Delete(item: longint);
				
				PROCEDURE TSortedLongintList.DeleteAll;
				
				FUNCTION TSortedLongintList.First: longint;
				  { Return the first element of the list.
				   If the compile-flag qDebug is TRUE & SetEltType was called, verify item's type. }
				
				FUNCTION TSortedLongintList.Last: longint;
				 { Return the last element of the list.
				   If the compile-flag qDebug is TRUE & SetEltType was called, verify item's type. }
				
				FUNCTION TSortedLongintList.LastThat(FUNCTION TestItem(item: longint): Boolean): longint;
				{ Call DoToItem once for each element of the list, in order. }
				
				FUNCTION TSortedLongintList.IterateTil(FUNCTION TestItem(item: longint): Boolean;
												 IterateForward: Boolean;
												 VAR itsIndex: ArrayIndex): longint;
				{ Call TestItem once for each element of the list, in order. }
				
				PROCEDURE TSortedLongintList.Each(PROCEDURE DoToItem(item: longint));
				
				FUNCTION TSortedLongintList.FirstThat(FUNCTION TestItem(item: longint): Boolean): longint;
				{ Call DoToItem once for each element of the list, in order. }
				
				FUNCTION TSortedLongintList.GetIdentityItemNo(item: longint): ArrayIndex;
				 { Find the first reference to the IDENTITY item in the list.
				  If item does not occur, return 0. }
				
				FUNCTION TSortedLongintList.GetEqualItemNo(item: longint): ArrayIndex;
				 { Find the first reference to item in the list.
				  If item does not occur, return 0. }
				
				FUNCTION TSortedLongintList.Compare(item1, item2: longint): CompareResult;
				{ By default just compare the ordinal value of the items.  Subclasses that want to use the items
				as pointers or other types should override if the comparison is based on the pointed at value }
				
				FUNCTION TSortedLongintList.CompareElements(Element1,Element2 : UNIV Ptr): CompareResult; OVERRIDE;
				{ Forward into the Compare function. }

				FUNCTION TSortedLongintList.DoSearch(FUNCTION TestItem(anItem: longint): CompareResult;
													 VAR index: ArrayIndex): ArrayIndex;
				{ DON'T use EXIT to get out of this routine from your TestItem function or you will be really
				sad!  (our debugger will check for you) That's why you can return TRUE to stop enumerating.
				Signaling Failure is OK too. }
				
				FUNCTION TSortedLongintList.Search(FUNCTION TestItem(anItem: longint): CompareResult): longint;

				PROCEDURE TSortedLongintList.DynamicFields(obj: TObject); OVERRIDE;
				
				PROCEDURE TSortedLongintList.Fields(obj: TObject); OVERRIDE;
				
				END;

			TLongintList				= OBJECT (TSortedLongintList) {A dynamic list of Longints. It IS
														  permissible for descendants to add more
														  named fields}
				PROCEDURE TLongintList.ILongintList;
				{ Initialize a new list with no elements, i.e., fSize = 0 }
				
				{ O V E R R I D E S  T O  R E M O V E  O R D E R I N G }
				FUNCTION  TLongintList.DoSearchElement(FUNCTION TestItem(anItem: ArrayIndex):
				CompareResult; VAR index : ArrayIndex): BOOLEAN; OVERRIDE;
				{ Overridden to do a linear search }

				PROCEDURE TLongintList.InsertElementInOrder(ElementPtr: UNIV Ptr); OVERRIDE;
				{ Overridden to do arrival sequence insertion }
				
				PROCEDURE TLongintList.AtPut(index: ArrayIndex;
											 newItem: longint);
				 { Replace the index'th element of the list.
				   Range check only if the compile-flag qRangeCheck is TRUE. }
				
				PROCEDURE TLongintList.InsertBefore(index: ArrayIndex;
													item: longint);
				 { Insert a reference to item at the indicated index.
				   If the compile-flag qDebug is TRUE & SetEltType was called, verify item's type. }
				
				PROCEDURE TLongintList.InsertFirst(item: longint);
				 { Insert a reference to item at the front of the list.
				   If the compile-flag qDebug is TRUE & SetEltType was called, verify item's type. }
				
				PROCEDURE TLongintList.InsertLast(item: longint);
				 { Insert a reference to item at the back of the list.
				   If qDebug is TRUE & SetEltType was called, check that item's type is correct. }
				
				PROCEDURE TLongintList.Fields(obj: TObject); OVERRIDE;
				
				PROCEDURE TLongintList.Push(item: longint);
				{ LIFO stack push. }
				
				FUNCTION TLongintList.Pop: longint;
				{ LIFO stack pop }
				END;
				
		FUNCTION NewList: TList;
		{ A convenience function. Create a TList, initialize it, and return a reference to it.
		Signals Failure if it cannot allocate the object.}

		FUNCTION NewSortedList: TSortedList;
		{ A convenience function. Create a TSortedList, initialize it, and return a reference to it.
		Signals Failure if it cannot allocate the object.}

		FUNCTION NewAllocatedList(iSize: ArrayIndex): TList;
		{ A convenience function. The same as NewList, but the initial allocation size can be set. }

		FUNCTION FreeListIfObject(list: TSortedList): TSortedList;
		{ A convenience function. if list is non-nil then the same as list.FreeList.
		Returns NIL for convenient assignment back to the reference passed in. }
END.
