{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UMacApp.p }
{ Copyright  1990-1991 by Apple Computer Inc.	All rights reserved. }
{ Originally written by Larry S. Rosenstein.  Used by permission }

UNIT UStream;

	INTERFACE

		USES Types, MacAppTypes,
			{}
			 Memory, UPascalObject, UObject;

{	This unit implements a rudimentary stream abstract type, and subclasses
		that do I/O to files and handles.  This can be used to implement
		document I/O and clipboard I/O with the same code.

	Streams can write out and read back objects, using the features in UObject
		to get an object's class ID and name.  The format of an object in the
		stream is as follows:

		2 bytes 	class ID (at the time it was written)
		4 bytes 	size of object in stream, excluding class ID
		1 byte		size of class name
		n bytes 	class name
		m bytes 	data specific to object

	The class ID in the stream might not match the class ID of the same class in the
		current application.  (It depends on how the class IDs were assigned.)	The
		stream code uses the class name to identify the actual class.  It is not
		necessary, however, to write out the class name for every
		class.	The stream will associate the classID in the stream with the
		correct class ID for the application.  The first object of a given class
		has its name written into the stream, along with its classID.  When read back,
		the name is used to find the class ID in the current application, which is
		associated with the class ID in the stream.  Subsequent objects of that class
		can be written with an empty class name, and the table of class IDs will be
		consulted to find out the proper class ID.

	TStream does not handle structures in which objects are referenced more than
		once.  To do this would require that the stream maintain a table of all the
		objects written out.  The first time an object was written it would be added
		to the table.  If the same object was written again, only its table index
		(or some other ID) would be written.  A similar process would be done to read
		the object back.

	TStream will skip over objects that it can't understand.  To do this, it writes the
		size of each object in a known position.  This means that writing an object
		is a 2 step process.  You first write the object header, by calling WriteObject,
		and after the entire object is written you fill in its size, by calling
		WriteObjectSize.

	TStream.ReadObject will return NIL if it can't understand a particular object, or
		if the object was actually NIL.  It returns a flag indicating the difference.
}

		TYPE

			TFile = OBJECT; EXTERNAL;

			{**** TStream ****}

{ This is the abstract stream class.  You would normally create instances of one
	of the concrete subclasses below. }

			TStream 			= OBJECT (TObject)
				fClassMapSize:		INTEGER;			{ handle to class ID mapping table }
				fClassMap:			Handle; 			{ number of entries in the table }

								{ CREATION/DESTRUCTION }

				PROCEDURE TStream.Initialize; OVERRIDE;
				{ Puts a newly created TStream into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TStream.IStream;
				PROCEDURE TStream.Free; OVERRIDE;

				{ ACCESS }								{ These methods have default implementations
														 that assume a 0-length stream. }
				FUNCTION TStream.GetPosition: LONGINT;
				PROCEDURE TStream.SetPosition(newPosition: LONGINT);
				FUNCTION TStream.GetSize: LONGINT;
				PROCEDURE TStream.SetSize(newSize: LONGINT);

										{ Note: all reading and writing methods signal
											failure in case of an I/O error. }

				{ READING }

				FUNCTION TStream.AtEnd: BOOLEAN;		{ Return TRUE if at the end of the stream. }

										{ This must be overridden by subclasses; all other
											reading methods call this.	}
				PROCEDURE TStream.ReadBytes(p: UNIV Ptr; count: LONGINT);

				{ Read various kinds of data. }
				PROCEDURE TStream.ReadCharacter(VAR data: Char);
				PROCEDURE TStream.ReadInteger(VAR data: UNIV INTEGER);
				PROCEDURE TStream.ReadLong(VAR data: UNIV LONGINT);
				PROCEDURE TStream.ReadPoint(VAR data: Point);
				PROCEDURE TStream.ReadRect(VAR data: Rect);

										{ Read a string.  maxSize is the amount of memory
											available, which is the maximum string
											length + 1.  If the size of the string in the
											stream is too large, then it signals Failure
											with err=paramErr. }
				PROCEDURE TStream.ReadString(data: UNIV StringPtr; maxSize: INTEGER);

										{ Returns an uninitialized object; known is set
											to TRUE if the object was valid (i.e., the class
											of the object is known to the program.	The
											function result will be NIL if the object was
											unknown or if NIL was found in the stream. }
				PROCEDURE TStream.ReadObject(VAR data: TObject; VAR known: BOOLEAN);

										{ First calls ReadObject and if the result is not
											NIL, calls ReadFrom on the new object. }
				PROCEDURE TStream.ReadStreamObject(VAR data: TObject; VAR known: BOOLEAN);

				{ WRITING }

										{ This must be overridden by subclasses; all other
											writing methods call this.	}
				PROCEDURE TStream.WriteBytes(p: UNIV Ptr; count: LONGINT);

				{ Write various kinds of data. }
				PROCEDURE TStream.WriteCharacter(data: Char);
				PROCEDURE TStream.WriteInteger(data: UNIV INTEGER);
				PROCEDURE TStream.WriteLong(data: UNIV LONGINT);
				PROCEDURE TStream.WritePoint(data: Point);
				PROCEDURE TStream.WriteRect(data: Rect);

										{ Write a string.  This takes a StringPtr, so
											it can accommodate any size string. }
				PROCEDURE TStream.WriteString(data: UNIV StringPtr);

										{ These methods are used to write an object to a
											stream.  You must call WriteObject first, then
											tell the object to write its specific data, and
											then call WriteObjectSize. }
				PROCEDURE TStream.WriteObject(data: TObject; VAR sizePosition: LONGINT);
				PROCEDURE TStream.WriteObjectSize(sizePosition: LONGINT);

										{ This calls WriteObject, then data.WriteTo, and
											finally WriteObjectSize. }
				PROCEDURE TStream.WriteStreamObject(data: TObject);

				{ CLASS HANDLING: used internally }

				FUNCTION TStream.LookupClassID(id: ObjClassID): ObjClassID;
				PROCEDURE TStream.RegisterClassID(oldID, newID: ObjClassID);

				{ DEBUGGING }
				PROCEDURE TStream.Fields(obj: TObject); OVERRIDE;
				END;

			{**** TFileStream ****}

{ This implements a stream on a file.
	The client is responsible for opening and closing the file. }

			TFileStream 		= OBJECT (TStream)

				fFile:			TFile;			{ file refnum }

								{ CREATION/DESTRUCTION }

				PROCEDURE TFileStream.Initialize; OVERRIDE;
				{ Puts a newly created TFileStream into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }
				PROCEDURE TFileStream.IFileStream(itsFile: TFile);

				{ SUBCLASS IMPLEMENTATIONS }

				FUNCTION TFileStream.GetPosition: LONGINT; OVERRIDE;
				PROCEDURE TFileStream.SetPosition(newPosition: LONGINT); OVERRIDE;
				FUNCTION TFileStream.GetSize: LONGINT; OVERRIDE;
				PROCEDURE TFileStream.SetSize(newSize: LONGINT); OVERRIDE;
				PROCEDURE TFileStream.ReadBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;
				PROCEDURE TFileStream.WriteBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;

				{ DEBUGGING }
				PROCEDURE TFileStream.Fields(obj: TObject); OVERRIDE;
				END;

			{**** THandleStream ****}

{ This implements a stream into or out of a handle.  The client is
	responsible for allocating and disposing the handle. }

			THandleStream		= OBJECT (TStream)

				fHandle:			Handle; 			{ the actual handle }
				fPosition:			LONGINT;			{ offset into the handle }
				fSize:				LONGINT;			{ size of the stream (may be smaller than
														 handle) }
				fGrowthSize:		LONGINT;			{ how much to grow the handle when needed }

								{ CREATION/DESTRUCTION }

				PROCEDURE THandleStream.Initialize; OVERRIDE;
				{ Puts a newly created THandleStream into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

										{ Initialize stream; growth specifies how much
											to grow the handle when it needs to grow.
											You use this so that the handle isn't grown by
											a small number of bytes on each write operation. }
				PROCEDURE THandleStream.IHandleStream(itsHandle: Handle; growth: LONGINT);

										{ This also shrinks the handle to the current
											position, eliminating any extra space at the end. }
				PROCEDURE THandleStream.Free; OVERRIDE;

				{ SUBCLASS IMPLEMENTATIONS }

				FUNCTION THandleStream.GetPosition: LONGINT; OVERRIDE;
				PROCEDURE THandleStream.SetPosition(newPosition: LONGINT); OVERRIDE;
				FUNCTION THandleStream.GetSize: LONGINT; OVERRIDE;
				PROCEDURE THandleStream.SetSize(newSize: LONGINT); OVERRIDE;
				PROCEDURE THandleStream.ReadBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;
				PROCEDURE THandleStream.WriteBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;

				{ GROWING }
										{ Returns amount to grow by, given that we need
											'needed' bytes.  Default is to grow by
											Max(fGrowth, needed). }
				FUNCTION THandleStream.GrowthSize(needed: LONGINT): LONGINT;

				{ DEBUGGING }
				PROCEDURE THandleStream.Fields(obj: TObject); OVERRIDE;
				END;

			{**** TCountingStream ****}

			{ This implements a stream that counts up the total number of bytes.
			 It is only usable for writing.	You will get a debugging break if
			 you try to read from this kind of stream.	}

			TCountingStream 	= OBJECT (TStream)

				fPosition:			LONGINT;			{ current position in the stream }
				fSize:				LONGINT;			{ current size of stream }

								{ CREATION/DESTRUCTION }

				PROCEDURE TCountingStream.Initialize; OVERRIDE;
				{ Puts a newly created TCountingStream into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TCountingStream.ICountingStream;

				{ SUBCLASS IMPLEMENTATIONS }

				FUNCTION TCountingStream.GetSize: LONGINT; OVERRIDE;
				PROCEDURE TCountingStream.WriteBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;

				{ DEBUGGING }
				PROCEDURE TCountingStream.Fields(obj: TObject); OVERRIDE;
				END;

			{ **** TResourceStream **** }

			{  This implements a stream that reads and writes to partial resources.
			  Since the partial resources traps are only available in System 7, this
			  stream class can only be used under System 7.	 }

			TResourceStream 	= OBJECT (TStream)

				fResource:			Handle;				{ resource handle }

				fPosition:			LONGINT;			{ current position in the stream }

				fSize:				LONGINT;			{ current size of stream }

				PROCEDURE TResourceStream.Initialize; OVERRIDE;
				{ Puts a newly created TResourceStream into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TResourceStream.IResourceStream(theType: ResType;theID: INTEGER);
				{ Initialize the TResourceStream. }

				FUNCTION TResourceStream.GetPosition: LONGINT; OVERRIDE;
				{ Returns fPosition. }

				PROCEDURE TResourceStream.SetPosition(newPosition: LONGINT); OVERRIDE;
				{ Sets fPosition. }

				FUNCTION TResourceStream.GetSize: LONGINT; OVERRIDE;
				{ Returns fSize. }

				PROCEDURE TResourceStream.SetSize(newSize: LONGINT); OVERRIDE;
				{ Sets fSize. }

				PROCEDURE TResourceStream.ReadBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;
				{ Reads bytes from the resource calling ReadPartialResource. }

				PROCEDURE TResourceStream.WriteBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;
				{ Write bytes to the resource calling WritePartialResource. }

				END;

END.
