(*
////////////////////////////////////////////////////////////////////////////
//	UViewBehavior.p
//
// Contains:		Interface file for ViewBehavior abstract class
//
//	Written by:	Robin Mair
//
//	Change History:
//
//			3/4/91		rtm		Initial creation of file
//
////////////////////////////////////////////////////////////////////////////
*)

(*

	VIEW BEHAVIOR OVERVIEW
	_______________________________________________________________________________________
	
	TViewBehavior is an abstract class that adds additonal event handling to
	standard TBehavior abstract class.  This additional filtering is view specific
	and this class should only be use to create behaviors that will be used
	with views and their sub-classes.
	
	View behaviors add support for the handling of mousedown, drawing and cursor
	manipulation from behaviors.  Additional support is provided in TViewBehavior
	for the behavior to be in an active state.  This means that when it is active
	it is managing the view that owns it.
	
	 See the description of Behaviors in the UBehavior.h header file for more
	information about behaviors in general 
	

	OTHER POINTS OF INTEREST
	_______________________________________________________________________________________

	The following methods of TView have been modified to provide support for
	behaviors., these methods will be listed here in the hope that should you 
	end up overriding one of them you will attempt to maintain the connection 
	to behaviors ( unless of course your intent is to disable behaviors!! )
	
		MODIFIED EXISTING MACAPP METHODS:
		
				TView.HandleMouseDown
				TView.DrawContents
				TView.HandleCursor


	SUMMARY
	_______________________________________________________________________________________

	View behaviors add additional functionality to support view specific event
	handling from behaviors.  TViewBehavior should be used as the abstract class
	for all behaviors to attached to views and their sub-classes.
	
	
*)


UNIT UViewBehavior;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, Events, AppleEvents, UEvtHandler, 
			Balloons, UEvent, UView, UBehavior;

		TYPE

			TBehavior			= OBJECT; EXTERNAL;
			TView					= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;
			TEvent				= OBJECT; EXTERNAL;

			{}
			{	TViewBehavior											}
			{}

			TViewBehavior			= OBJECT (TBehavior)
			(*
			
				This sub-class of TBehavior is used as the abstract class for view specific Behaviors
				and should be subclassed.  It adds view specific information to the behavior such as the
				ability to handle of mousedown events and drawing.  This class also adds the notion
				of the behavior being active and therefore adds a new field "fActive" and its supporting
				methods
				
			*)
				fActive:			BOOLEAN;		{ Used to indicate that the behavior is active }
				
				PROCEDURE TViewBehavior.Initialize; OVERRIDE;
					{ Overridden so that we can initialize our field	}
					
				PROCEDURE TViewBehavior.IViewBehavior;
					{ Initialization method	}
																			
				PROCEDURE TViewBehavior.ActivateViewBehavior ( activate: BOOLEAN );
					{ Called to activate a view behavior, this method would normally
					be overridden if any additional functionality is required when
					activating a view behavior	}
					
				FUNCTION TViewBehavior.GetOwningView: TView;
					{ A convenience method that returns the owning EvtHandler as a view	}
					
				FUNCTION TViewBehavior.IsBehaviorActive: BOOLEAN;
					{ Returns the current state of the 'fActive' flag	}
					
				FUNCTION TViewBehavior.DoMouseCommand (theMouse: VPoint;
																	event: TToolboxEvent;
																	hysteresis: Point ): BOOLEAN;
					{ Allows view behaviors to intercept mouse commands, if there is
					a next behavior it passes the message on down the chain otherwise
					by default it returns false	}
					
				PROCEDURE TViewBehavior.Draw ( area: VRect );
					{ Called during a views drawing process, after the view has been
					called to draw itself and before the views subviews are called
					to draw themselves.  The 'area' in need of drawing is passed in.	}
					
				FUNCTION TViewBehavior.DoSetCursor (	localPoint: VPoint;
																	cursorRgn: RgnHandle ): BOOLEAN;
					{ Gives the view behavior the opportunity to setup the cursor, if
					there is a nex behavior it passes the message on down the chain
					otherwise by default it returns false	}
					
			END;

END.